@@ -1,11 +1,10 @@
 //
 // File:        UserOptions.java
 // Package:     gov.llnl.babel
-// Release:     $Name: release-0-8-8 $
-// Revision:     @(#) $Id: UserOptions.java,v 1.30 2003/10/05 17:49:25 norris Exp $
+// Revision:     @(#) $Id: UserOptions.java 4434 2005-03-17 17:05:29Z epperly $
 // Description: singleton Map for configuring general, user-related options
 // 
-// Copyright (c) 2000-2002, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2004, The Regents of the University of Calfornia.
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -31,8 +30,6 @@
 
 package gov.llnl.babel;
 
-import java.lang.InstantiationException;
-import java.lang.IllegalAccessException;
 import gnu.getopt.Getopt;
 import gnu.getopt.LongOpt;
 import gov.llnl.babel.symbols.RegexMatch;
@@ -40,7 +37,6 @@ import gov.llnl.babel.symbols.RegexUnsupportedException;
 import gov.llnl.babel.symbols.BadRegexException;
 import gov.llnl.babel.backend.CodeGenerationFactory;
 
-
 /**
  * <code>UserOptions</code> is a singleton class that manages the
  * wide variety of settings available to a user.  The configuration 
@@ -70,9 +66,8 @@ import gov.llnl.babel.backend.CodeGenerationFactory;
  *
  * DIRECTIONS FOR ADDING SUPPORT FOR NEW LANGUAGE(S):
  * 1. Update printUsage method.
- * 2. Update the appropriate validClientLanguage (for client), 
- *    validRemoteLanguage (for remote client), and validServerLanguage
- *    (for server).
+ * 2. Update the appropriate validClientLanguage (for client) and 
+ *    validServerLanguage (for server).
  * 3. TEST
  * 4. cvs commit
  */
@@ -84,26 +79,29 @@ public class UserOptions
    */
   private static final LongOpt[] d_long_opts = 
   {
+    new LongOpt("generate-assertions",   LongOpt.REQUIRED_ARGUMENT, null, 'a'),
     new LongOpt("client",                LongOpt.REQUIRED_ARGUMENT, null, 'c'),
+    new LongOpt("exclude",               LongOpt.REQUIRED_ARGUMENT, null, 'e'),
+    new LongOpt("exclude-external",      LongOpt.NO_ARGUMENT,       null, 'E'), 
     new LongOpt("generate-subdirs",      LongOpt.NO_ARGUMENT,       null, 'g'),
     new LongOpt("help",                  LongOpt.NO_ARGUMENT,       null, 'h'),
+    new LongOpt("generate-interceptors", LongOpt.NO_ARGUMENT,       null, 'i'),
+    new LongOpt("language-subdir",       LongOpt.NO_ARGUMENT,       null, 'l'),
+    new LongOpt("make-prefix",           LongOpt.REQUIRED_ARGUMENT, null, 'm'),
     new LongOpt("output-directory",      LongOpt.REQUIRED_ARGUMENT, null, 'o'), 
-    new LongOpt("vpath",                 LongOpt.REQUIRED_ARGUMENT, null, 'V'), 
     new LongOpt("parse-check",           LongOpt.NO_ARGUMENT,       null, 'p'),
-    new LongOpt("remote",                LongOpt.REQUIRED_ARGUMENT, null, 'r'), 
     new LongOpt("repository-path",       LongOpt.REQUIRED_ARGUMENT, null, 'R'), 
     new LongOpt("server",                LongOpt.REQUIRED_ARGUMENT, null, 's'), 
     new LongOpt("text",                  LongOpt.REQUIRED_ARGUMENT, null, 't'), 
-    new LongOpt("xml",                   LongOpt.NO_ARGUMENT,       null, 'x'),
+    new LongOpt("hide-glue",             LongOpt.NO_ARGUMENT,       null, 'u'),
     new LongOpt("version",               LongOpt.NO_ARGUMENT,       null, 'v'),
-    new LongOpt("exclude",               LongOpt.REQUIRED_ARGUMENT, null, 'e'),
+    new LongOpt("vpath",                 LongOpt.REQUIRED_ARGUMENT, null, 'V'), 
+    new LongOpt("cxx-ior-exception",     LongOpt.NO_ARGUMENT,       null, 'x'),
     new LongOpt("generate-sidl-stdlib",  LongOpt.NO_ARGUMENT,       null, 5), 
     new LongOpt("no-default-repository", LongOpt.NO_ARGUMENT,       null, 6), 
     new LongOpt("suppress-timestamp",    LongOpt.NO_ARGUMENT,       null, 7), 
     new LongOpt("comment-local-only",    LongOpt.NO_ARGUMENT,       null, 8),
-    new LongOpt("hide-glue",             LongOpt.NO_ARGUMENT,       null, 'u'),
-    new LongOpt("exclude-external",      LongOpt.NO_ARGUMENT,       null, 'E'),    
-    new LongOpt("language-subdir",       LongOpt.NO_ARGUMENT,       null, 'l'),
+    new LongOpt("short-file-names",      LongOpt.NO_ARGUMENT,       null, 9),
   };
 
   /**
@@ -117,14 +115,12 @@ public class UserOptions
    */
   private static UserOptions s_my_instance = null;
   
-
   /**
    * Since this is a singleton class, the constructor is protected.
    */
   protected UserOptions() {
   }
 
-
   /**
    * Return the singleton instance of the user options.  
    * If the user options instance has not yet been created, 
@@ -137,7 +133,6 @@ public class UserOptions
     return s_my_instance;
   }
 
-
   /** 
    * Extract options from the command line.
    *
@@ -150,7 +145,7 @@ public class UserOptions
                                       String             args[] ) 
   {
     Getopt commandlineOptions = 
-      new Getopt("babel", args, "Eghpvxluo:c:r:R:s:t:e:", d_long_opts);
+      new Getopt("babel", args, "Eghilpuvxa:o:c:R:s:t:e:m:", d_long_opts);
 
     int numRequired    = 0;
     boolean helpSet    = false;
@@ -171,8 +166,8 @@ public class UserOptions
          * sure it appears in the help file.  Processing should continue
          * even if it does get passed in as an argument.
          */
-        System.err.println("Babel: Warning: no-default-repository is a babel driver"
-                           + " script option only.");
+        System.err.println("Babel: Warning: no-default-repository is a babel "
+              + "driver script option only.");
         break;
       case 7:
         theConfig.setSuppressTimestamps(true);
@@ -180,10 +175,16 @@ public class UserOptions
       case 8:
         theConfig.setCommentLocalOnly(true);
         break;
+      case 9:
+        theConfig.setShortFileNames(true);
+        break;
       case 'p' :
         theConfig.setParseCheckOnly(true);
         numRequired = numRequired + 1;
         break;
+      case 'm': // make prefix
+        theConfig.setMakePrefix(commandlineOptions.getOptarg());
+        break;
       case 'c' :
         String clang = commandlineOptions.getOptarg().toLowerCase();
         if ( validClientLanguage(clang) ) {
@@ -192,24 +193,9 @@ public class UserOptions
           numRequired = numRequired + 1;
         } else {
           System.err.println("Babel: Error: Unsupported language (" + clang 
-                             + ") specified for client option.");
+                + ") specified for client option.");
         }
         break;
-      case 'r' :
-        String rlang = commandlineOptions.getOptarg().toLowerCase();
-        if ( validRemoteLanguage(rlang) ) {
-          theConfig.setGenerateRemote(true);
-          theConfig.setTargetLanguage(rlang);
-          //ToDo...Uncomment the following once supported.
-          //numRequired = numRequired + 1;
-        } else {
-          System.err.println("Babel: Error: Unsupported language (" + rlang 
-                             + ") specified for remote option.");
-        }
-        //ToDo...Remove the following once supported.
-        System.err.println("\nBabel: *** Sorry. Remote invocation is not yet supported"
-                           + ". ***\n");
-        break;
       case 's' :
         String slang = commandlineOptions.getOptarg().toLowerCase();
         if ( validServerLanguage(slang) ) {
@@ -218,7 +204,7 @@ public class UserOptions
           numRequired = numRequired + 1;
         } else {
           System.err.println("Babel: Error: Unsupported language (" + slang 
-                             + ") specified for server option.");
+                + ") specified for server option.");
         }
         break;
       case 't' :
@@ -229,13 +215,9 @@ public class UserOptions
           numRequired = numRequired + 1;
         } else {
           System.err.println("Babel: Error: Unsupported language representation"
-                             + " (" + tlang + ") specified for text option.");
+                + " (" + tlang + ") specified for text option.");
         }
         break;
-      case 'x' :		// Deprecated!
-        theConfig.setGenerateXML(true);
-        numRequired = numRequired + 1;
-        break;
       case 'h':
         helpSet     = true;
         numRequired = numRequired + 1;
@@ -245,7 +227,35 @@ public class UserOptions
         numRequired = numRequired + 1;
         break;
       case 'R':
-        theConfig.addToRepositoryPath(commandlineOptions.getOptarg());
+        String shortname = commandlineOptions.getOptarg();
+        boolean absolutePath = false;
+        java.io.File[] roots = java.io.File.listRoots();
+
+        //directory ends with sperator? 
+        if(!shortname.endsWith(""+java.io.File.separatorChar)) 
+           shortname = shortname + java.io.File.separatorChar;
+        //Figure out if we have an absolute path by checking that the path
+        //begins with a root.  ( '/' in Unix, "C:\" or "D:\" in Windows)
+        for(int i = 0; i < roots.length; ++i) {
+          if(roots[i].toString().compareTo(shortname.substring(0,
+                                          roots[i].toString().length())) == 0) {
+            absolutePath=true;
+          }
+        }
+
+        if(absolutePath) { //If it's an absolute path, good
+          theConfig.addToRepositoryPath(shortname);
+        } else {// if the path is realitive, make it absolute
+          java.io.File file = new java.io.File(".");
+          try {
+          theConfig.addToRepositoryPath(file.getCanonicalPath()
+                                      + java.io.File.separatorChar + shortname);
+          } catch (java.io.IOException e) {
+            System.err.println("Unable to get path to current directory! Any "
+                  + "XML output may contain an incorrect path!");
+          }
+        }
+        //theConfig.addToRepositoryPath(commandlineOptions.getOptarg());
         break;
       case 'o' :
         theConfig.setOutputDirectory(commandlineOptions.getOptarg());
@@ -262,9 +272,12 @@ public class UserOptions
             (new RegexMatch(commandlineOptions.getOptarg()));
         }
         catch(RegexUnsupportedException rue) {
-          System.err.println("Babel: The --exclude=<regex> | -e <regex> feature requires a Java runtime environment with the");
-          System.err.println("1.4 library or higher for regular expression support.");
-          System.err.println("You must use a more recent version of Java to use this feature.");
+          System.err.println("Babel: The --exclude=<regex> | -e <regex> feature"
+                + " requires a Java runtime environment with the");
+          System.err.println("1.4 library or higher for regular expression "
+                + "support.");
+          System.err.println("You must use a more recent version of Java to use"
+                + " this feature.");
         }
         catch(BadRegexException bre) {
           System.err.println("Babel: " + bre.getMessage());
@@ -278,7 +291,23 @@ public class UserOptions
         break;
       case 'E':
         theConfig.setExcludeExternal(true);
-				break;
+        break;
+      case 'a':
+        String alvl = commandlineOptions.getOptarg();
+        if (validAssertionLevel(alvl)) {
+          theConfig.setGenerateAssertions(alvl);
+        } else {
+          System.err.println("Babel: Error: Unsupported assertion checking "
+                + "level (" + alvl 
+                + ") specified for generate assertion option.");
+        }
+        break;
+      case 'i':
+        theConfig.setGenerateInterceptors(true);
+        break;
+      case 'x':
+        theConfig.setCxxCheckNullIOR(true);
+        break;
       case ':':
         // the option requires an argument
         break;
@@ -286,12 +315,11 @@ public class UserOptions
         // the option is not valid
         break;      
       default :
-        System.err.println("Babel: Error: Should never have reached default in gov."
-                           + "llnl.babel.UserOptions.");
+        System.err.println("Babel: Error: Should never have reached default in "
+              + "gov.llnl.babel.UserOptions.");
         break;	
       }
     }
-
     /*
      * If no options were encountered and there's nothing else on the line, 
      * then force displaying the help.
@@ -302,7 +330,6 @@ public class UserOptions
       numRequired = 1;
     }
 
-
     /*
      * If building the Python client, then allow generation of the SIDL
      * Standard library to ensure it happens for only one Python client.
@@ -314,8 +341,8 @@ public class UserOptions
     if (  theConfig.generateClient() 
        && theConfig.getTargetLanguage().equals("python")
        && theConfig.generateStdlib()  ) {
-      System.err.println("Babel: Warning:  Allowing concurrent request for python "
-                         + "client and SIDL standard library generation.");
+      System.err.println("Babel: Warning:  Allowing concurrent request for "
+            + "python client and SIDL standard library generation.");
       numRequired = numRequired - 1;
     }
 
@@ -325,26 +352,19 @@ public class UserOptions
      */
     if (numRequired > 1) 
     {
-      System.err.println("Babel: Error: Only one of the following options or its "
-                         + "equivalent is allowed:");
+      System.err.println("Babel: Error: Only one of the following options or "
+            + "its equivalent is allowed:");
       System.err.println("         --help, --version, --client, --server, "
-//ToDo...Replace current list with current commented once remote supported.
-//                         + "--remote, --text,");
-                         + "--text,");
+            + "--text,");
       System.err.println("         --parse-check, or --generate-sidl-stdlib.");
-    }
-    else if (numRequired < 1) {
-      System.err.println("Babel: Error: You must enter one of the following options "
-                         + "or its equivalent along with");
+    } else if (numRequired < 1) {
+      System.err.println("Babel: Error: You must enter one of the following "
+            + "options or its equivalent along with");
       System.err.println("       any associated argument:");
       System.err.println("         --help, --version, --client, --server, "
-//ToDo...Replace current list with current commented once remote supported.
-//                         + "--remote, --text,");
-                         + "--text,");
+            + "--text,");
       System.err.println("         --parse-check, or --generate-sidl-stdlib.");
-    } 
-    else
-    {
+    } else {
       /*
        * Finalize any temporary options or ensure option consistency as
        * needed.
@@ -383,7 +403,6 @@ public class UserOptions
     return returnIndex;
   }
 
-
   /**
    * Print the command line options and contact information.
    */
@@ -392,123 +411,147 @@ public class UserOptions
     System.out.println("Usage:  babel [ -h | --help ]");
     System.out.println("   or   babel [ -v | --version ]");
     System.out.println("   or   babel option(s) sidlfilename1 ... "
-                       + "sidlfilenameN");
+          + "sidlfilenameN");
     System.out.println();
     System.out.println("where help, version, and option(s) are:");
     System.out.println(" -h       | --help          Display usage information "
-                       + "and exit.");
+          + "and exit.");
     System.out.println(" -v       | --version       Display version and exit.");
     System.out.println(" -p       | --parse-check   Parse the sidl file but do "
-                       + "not generate code.");
-    System.out.println(" -x       | --xml           Generate only SIDL XML ("
-                       + "deprecated; use -tXML).");
+          + "not generate code.");
     System.out.println(" -c<lang> | --client=<lang> Generate only client code "
-                       + "in specified language");
-    System.out.println("                              (C | C++ | F77 | F90 |"
-                       + " Java | Python).");
-//ToDo...Uncomment the following once remote is supported.
-//    System.out.println(" -r<lang> | --remote=<lang> Generate only remote "
-//                       + "client code in specified");
-//    System.out.println("                              language ().");
+          + "in specified language");
+    System.out.println("                            (C | C++ | UC++ | F77 "
+          + "| F90 | Java | Python).");
+    System.out.println(" -r | --with-rmi            Generate remote "
+          + "client code in specified");
+    System.out.println("                            language ().");
     System.out.println(" -s<lang> | --server=<lang> Generate server (and "
-                       + "client) code in specified");
-    System.out.println("                              language (C | C++ | F77 "
-                       + "| F90 | Java | Python).");
+          + "client) code in specified");
+    System.out.println("                            language (C | C++ | F77 "
+          + "| F90 | Java | Python).");
     System.out.println(" -t<form> | --text=<form>   Generate text in specified"
-                       + " form (XML | SIDL), where");
-    System.out.println("                              XML updates the "
-                       + "repository.");
+          + " form (XML | SIDL), where");
+    System.out.println("                            XML updates the "
+          + "repository.");
     System.out.println(" -o<dir>  | --output-directory=<dir>");
     System.out.println("                            Set Babel output directory "
-                       + "(\'.\' default).");
+          + "(\'.\' default).");
     System.out.println(" -R<path> | --repository-path=<path>");
     System.out.println("                            Set semicolon-separated URL"
-                       + " list used to resolve");
-    System.out.println("                              symbols.");
+          + " list used to resolve");
+    System.out.println("                            symbols.");
     System.out.println(" -e<regex> | --exclude=<regex>");
-    System.out.println("                            " +
-                       "Symbols matching the regular expression");
-    System.out.println("                            " +
-                       "are excluded from code generation.");
-    System.out.println("                            " +
-                       "Requires JRE 1.4 virtual machine or higher.");
-    System.out.println("                            matching package hierarchy.");
+    System.out.println("                            " 
+          + "Symbols matching the regular expression");
+    System.out.println("                            " 
+          + "are excluded from code generation.");
+    System.out.println("                            " 
+          + "Requires JRE 1.4 virtual machine or higher.");
+    System.out.println("                            matching package "
+          + "hierarchy.");
     System.out.println(" -E        | --exclude-external");
-    System.out.println("                            " +
-                       "Code is generated only for the symbol hierarchies rooted");
-    System.out.println("                            " +
-                       "at the symbols specified on the command line.");
-    System.out.println(" -V<path>  | --vpath=<path> Prepend alternative search path for reading Impl");
-    System.out.println("                            splicer blocks.  Does not affect where Impls are");
+    System.out.println("                            "
+          + "Code is generated only for the symbol hierarchies ");
+    System.out.println("                            "
+          + "rooted at the symbols specified on the command line.");
+    System.out.println(" -V<path>  | --vpath=<path> Prepend alternative search "
+          + "path for reading Impl");
+    System.out.println("                            splicer blocks.  Does not "
+          + "affect where Impls are");
     System.out.println("                            generated.");
-    System.out.println("                            NOTE: --vpath=. is a no-op for autoconf/automake");
-    System.out.println("                            If you really want current directory, use"); 
-    System.out.println("                            another argument like `pwd` or even ./.");
+    System.out.println("                            NOTE: --vpath=. is a no-op "
+          + "for autoconf/automake");
+    System.out.println("                            If you really want current "
+          + "directory, use"); 
+    System.out.println("                            another argument like `pwd`"
+          + " or even ./.");
+    System.out.println(" -a<lvl>  | --generate-assertions=<lvl>");
+    System.out.println("                            Generate SIDL-specified "
+          + "assertion checking");
+    System.out.println("                            code.");
+    System.out.println("                            NOTE: 0=None, 1=Minimal sta"
+          + "s, 2=Full stats.");
+    System.out.println(" -i       | --generate-interceptors");
+    System.out.println("                            Generate method "
+          + "interceptors in the form of pre-");
+    System.out.println("                            and post-method hooks.");
     System.out.println(" -g       | --generate-subdirs");
-    System.out.println("                            Postpend Java-style package subdirs");
-    System.out.println("                            to paths specified by -o and -V.");
+    System.out.println("                            Postpend Java-style package"
+          + " subdirs");
+    System.out.println("                            to paths specified by -o "
+          + "and -V.");
     System.out.println(" -l       | --language-subdir");
-    System.out.println("                            Generate code in a language-specific");
+    System.out.println("                            Generate code in a language"
+          + "-specific");
     System.out.println("                            subdirectory.");
-    System.out.println(" -u       | --hide-glue");
-    System.out.println("                            Put \"glue\" (not modifiable by user) generated");
-    System.out.println("                            code in a glue/ subdirectory.");
+    System.out.println(" -u       | --hide-glue     Put \"glue\" (not "
+          + "modifiable by user) generated");
+    System.out.println("                            code in a glue/ "
+          + "subdirectory.");
     System.out.println(" --no-default-repository    Prohibit use of default to "
-                       + "resolve symbols.");
+          + "resolve symbols.");
     System.out.println(" --suppress-timestamp       Suppress timestamps in "
-                       + "generated files.");
+          + "generated files.");
     System.out.println(" --generate-sidl-stdlib     Regenerate only the SIDL "
-                       + "standard library.");
-    System.out.println
-      (" --comment-local-only       Only comment locally defined methods in stub.");
+          + "standard library.");
+    System.out.println(" --comment-local-only       Only comment locally "
+          + "defined methods in stub.");
+    System.out.println(" --short-file-names         Use in conjunction with -g "
+          + "to generate file names");
+    System.out.println("                            that don't include package "
+          + "names.");
+    System.out.println(" --cxx-ior-exception        Include IOR pointer "
+          + "checking in C++ Stubs, throws");
+    System.out.println("                            a NullIORException if IOR "
+          + "is Null.");
     System.out.println();
     System.out.println("Note:  One of the following options or its "
-                       + "equivalent must be specified:");
+          + "equivalent must be specified:");
     System.out.println("         --help, --version, --client, --server, "
-                       + "--xml, --text, --parse-check, or");
+          + "--text, --parse-check, or");
     System.out.println("         --generate-sidl-stdlib.");
     System.out.println();
     System.out.println("If you have any suggestions or questions not answered "
-                       + "by the documenation,");
+          + "by the documenation,");
     System.out.println("please send email to components@llnl.gov.");
   }
   
-  
   /**
    * Check if the specified client-side language is supported.
    */
-  private boolean validClientLanguage( String lang ) 
-  { 
+  private boolean validClientLanguage( String lang ) { 
     CodeGenerationFactory factory = CodeGenerationFactory.getInstance();
     return (factory.getCodeGenerator(lang, "stub") != null);
   }
 
-
   /**
-   * Check if the specified remote client-side language is supported.
+   * Check if the specified assertion checking level is supported.
    */
-  private static boolean validRemoteLanguage( String lang ) 
-  { 
-    boolean isValid = false;
-    return isValid; 
+  private boolean validAssertionLevel( String alvl ) { 
+    boolean is;
+    if (  (alvl.equals(BabelConfiguration.NO_CHECKING        )) 
+       || (alvl.equals(BabelConfiguration.MIN_CHECKING_STATS )) 
+       || (alvl.equals(BabelConfiguration.FULL_CHECKING_STATS)) ) {
+      is = true;
+    } else {
+      is = false;
+    }
+    return is;
   }
 
-
   /**
    * Check if the specified server-side language is supported.
    */
-  private static boolean validServerLanguage( String lang ) 
-  { 
+  private static boolean validServerLanguage( String lang ) { 
     CodeGenerationFactory factory = CodeGenerationFactory.getInstance();
     return (factory.getCodeGenerator(lang, "skel") != null);
   }
 
-
   /**
    * Check if the specified textual representation is supported.
    */
-  private boolean validTextRepresentation( String lang ) 
-  { 
+  private boolean validTextRepresentation( String lang ) { 
     boolean isValid = false;
     if ( lang.equals("xml") ) {
       isValid = true;
@@ -519,7 +562,6 @@ public class UserOptions
     return isValid;
   }
 
-
   /**
    * Check if the user options indicate processing can continue.
    */
@@ -527,7 +569,6 @@ public class UserOptions
     return d_can_proceed; 
   }
 
-
   /*********************************************************************
    * Used for testing purposes only!
    */
@@ -546,9 +587,10 @@ public class UserOptions
     }
 
     System.out.println("\n\n...Printing resulting settings...");
+    System.out.println("Generate Assertions  = " + config.getAssertionLevel());
+    System.out.println("Generate Interceptors= " 
+      + config.generateInterceptors());
     System.out.println("Generate Client      = " + config.generateClient());
-//ToDo...Uncomment the following once remote supported.
-//    System.out.println("Generate Remote      = " + config.generateRemote());
     System.out.println("Generate Server      = " + config.generateServer());
     System.out.println("Generate Text        = " + config.generateText());
     System.out.println("Generate SIDL Stdlib = " + config.generateStdlib());
