@@ -1,9 +1,8 @@
 //
 // File:        ArrayMethods.java
 // Package:     gov.llnl.babel.backend
-// Release:     $Name: release-0-8-8 $
-// Revision:    @(#) $Revision: 1.9 $
-// Date:        $Date: 2003/09/05 15:44:50 $
+// Revision:    @(#) $Revision: 4434 $
+// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
 // Description: 
 // 
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -31,13 +30,11 @@
 // Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 package gov.llnl.babel.backend.c;
-import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.backend.IOR;
-import gov.llnl.babel.BabelConfiguration;
-import gov.llnl.babel.symbols.Version;
 import gov.llnl.babel.backend.writers.LanguageWriter;
-import java.util.List;
-import java.util.ArrayList;
+import gov.llnl.babel.BabelConfiguration;
+import gov.llnl.babel.symbols.SymbolID;
+import gov.llnl.babel.symbols.Version;
 
 public class ArrayMethods {
   static private final int s_maxArray = BabelConfiguration.getMaximumArray();
@@ -56,35 +53,26 @@ public class ArrayMethods {
 
   private SymbolID d_implID;
 
-  public ArrayMethods(SymbolID id,
-                      boolean isEnum) {
+  public ArrayMethods(SymbolID id, boolean isEnum) {
     d_id = id;
     d_isEnum = isEnum;
-    d_implArrayType = isEnum 
-      ? "struct SIDL_int__array *" 
-      : "struct SIDL_interface__array *";
-    d_implDataType = isEnum 
-      ? "int32_t" 
-      : "struct SIDL_BaseInterface__object *";
-    d_initDataType = isEnum
-      ? "int32_t"
-      : C.getObjectName(id);
+    d_implArrayType = isEnum ? "struct sidl_int__array *" 
+                             : "struct sidl_interface__array *";
+    d_implDataType = isEnum ? "int32_t" : "struct sidl_BaseInterface__object *";
+    d_initDataType = isEnum ? "int32_t" : C.getObjectName(id);
     if (isEnum) {
-      d_implID = new SymbolID("SIDL.int", new Version());
-    }
-    else {
-      d_implID = new SymbolID("SIDL.interface", new Version());
+      d_implID = new SymbolID("sidl.int", new Version());
+    } else {
+      d_implID = new SymbolID("sidl.interface", new Version());
     }
     d_comments = !BabelConfiguration.getInstance().getCommentLocalOnly();
   }
 
-  private String getTypeName()
-  {
+  private String getTypeName() {
     return d_isEnum ? C.getEnumName(d_id) : C.getObjectName(d_id);
   }
 
-  public static String generateCreateName(SymbolID id, String shortType)
-  {
+  public static String generateCreateName(SymbolID id, String shortType) {
     return C.getSymbolName(id) + "__array_create" + shortType;
   }
 
@@ -96,7 +84,8 @@ public class ArrayMethods {
     String methodName = generateCreateName(d_id, shortType);
     if (d_comments){
       lw.beginBlockComment(true);
-      lw.println("Create a contiguous array of the given dimension with specified");
+      lw.println("Create a contiguous array of the given dimension with "
+        + "specified");
       lw.println("index bounds in " + longType + "-major order. This array");
       lw.println("owns and manages its data.");
       if (!d_isEnum){
@@ -106,25 +95,24 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println(IOR.getArrayName(d_id) + "*");
-    lw.println(methodName + "(int32_t        dimen,");
-    lw.printSpaces(methodName.length()+1);
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println("int32_t       dimen,");
     lw.println("const int32_t lower[],");
-    lw.printSpaces(methodName.length()+1);
     lw.println("const int32_t upper[])" + terminator);
+    lw.backTab();
   }
 
-  public static String generateOneDName(SymbolID id)
-  {
+  public static String generateOneDName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_create1d";
   }
 
-  private void generateOneDSig(LanguageWriter lw,
-                               String terminator)
-  {
+  private void generateOneDSig(LanguageWriter lw, String terminator) {
     String methodName = generateOneDName(d_id);
     if (d_comments){ 
       lw.beginBlockComment(true);
-      lw.println("Create a contiguous one-dimensional array with a lower index");
+      lw.println("Create a contiguous one-dimensional array with a lower "
+        + "index");
       lw.println("of 0 and an upper index of len-1. This array");
       lw.println("owns and manages its data.");
       if (!d_isEnum){
@@ -137,45 +125,44 @@ public class ArrayMethods {
     lw.println(methodName + "(int32_t len)" + terminator);
   }
   
-  public static String generateOneDInitName(SymbolID id)
-  {
+  public static String generateOneDInitName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_create1dInit";
   }
 
-  private void generateOneDInitSig(LanguageWriter lw,
-                                   String terminator)
-  {
+  private void generateOneDInitSig(LanguageWriter lw, String terminator) {
     String methodName = generateOneDInitName(d_id);
     if (d_comments){ 
       lw.beginBlockComment(true);
       lw.println("Create a dense one-dimensional vector with a lower");
-      lw.println("index of 0 and an upper index of len-1. The initial data for this");
-      lw.println("new array is copied from data. This will increment the reference");
+      lw.println("index of 0 and an upper index of len-1. The initial data for "
+        + "this");
+      lw.println("new array is copied from data. This will increment the "
+        + "reference");
       lw.println("count of each non-NULL object/interface reference in data.");
       lw.println();
       lw.println("This array owns and manages its data.");
       lw.endBlockComment(true);
     }
     lw.println(IOR.getArrayName(d_id) + "*");
-    lw.println(methodName + "(int32_t len, "
-               +  d_initDataType + "* data)" + terminator);
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println("int32_t len, ");
+    lw.println(d_initDataType + "* data)" + terminator);
+    lw.backTab();
   }
   
-  public static String generateTwoDName(SymbolID id, String shortType)
-  {
+  public static String generateTwoDName(SymbolID id, String shortType) {
     return IOR.getSymbolName(id) + "__array_create2d" + shortType;
   }
 
-  private void generateTwoDSig(LanguageWriter lw,
-                               String shortType,
-                               String longType, 
-                               String terminator)
+  private void generateTwoDSig(LanguageWriter lw, String shortType,
+                               String longType, String terminator)
   {
     String methodName = generateTwoDName(d_id, shortType);
     if (d_comments) {
       lw.beginBlockComment(true);
-      lw.println("Create a contiguous two-dimensional array in " + longType +
-                 "-major");
+      lw.println("Create a contiguous two-dimensional array in " + longType 
+        + "-major");
       lw.println("order with a lower index of (0,0) and an upper index of");
       lw.println("(m-1,n-1). This array owns and manages its data.");
       if (!d_isEnum){
@@ -188,20 +175,16 @@ public class ArrayMethods {
     lw.println(methodName + "(int32_t m, int32_t n)" + terminator);
   }
 
-  public static String generateBorrowName(SymbolID id)
-  {
+  public static String generateBorrowName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_borrow";
   }
 
-  private void generateBorrowSig(LanguageWriter lw,
-                                 String terminator)
-  {
+  private void generateBorrowSig(LanguageWriter lw, String terminator) {
     String methodName = generateBorrowName(d_id);
     String firstType;
     if (d_isEnum) {
       firstType = "int32_t *";
-    }
-    else {
+    } else {
       firstType = C.getObjectName(d_id) + "*";
     }
     if (d_comments) {
@@ -211,8 +194,7 @@ public class ArrayMethods {
       lw.println("data being borrowed.");
       if (d_isEnum){
         lw.println("The borrowed data must be a pointer to int32_t.");
-      }
-      else {
+      } else {
         lw.println("Any time an element in the borrowed array is replaced");
         lw.println("via a set call, deleteRef will be called on the");
         lw.println("value being replaced if it is not NULL.");
@@ -220,22 +202,21 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println(IOR.getArrayName(d_id) + "*");
-    lw.println(methodName + "(" + firstType + "firstElement,");
-    lw.printSpaces(methodName.length()+1);
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println(firstType + " firstElement,");
     lw.println("int32_t       dimen,");
     lw.println("const int32_t lower[],");
     lw.println("const int32_t upper[],");
     lw.println("const int32_t stride[])" + terminator);
+    lw.backTab();
   }
   
-  public static String generateSmartCopyName(SymbolID id)
-  {
+  public static String generateSmartCopyName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_smartCopy";
   }
 
-  private void generateSmartCopySig(LanguageWriter lw,
-                                    String terminator)
-  {
+  private void generateSmartCopySig(LanguageWriter lw, String terminator) {
     String methodName = generateSmartCopyName(d_id);
     String firstType;
     if (d_comments) {
@@ -251,18 +232,17 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println(IOR.getArrayName(d_id) + "*");
-    lw.println(methodName + "(" + IOR.getArrayName(d_id) + " *array)" +
-               terminator);
+    lw.println(methodName + "(" );
+    lw.tab();
+    lw.println(IOR.getArrayName(d_id) + " *array)" + terminator);
+    lw.backTab();
   }
   
-  public static String generateDelRefName(SymbolID id)
-  {
+  public static String generateDelRefName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_deleteRef";
   }
   
-  private void generateDelRefSig(LanguageWriter lw,
-                                 String terminator)
-  {
+  private void generateDelRefSig(LanguageWriter lw, String terminator) {
     if (d_comments) {
       lw.beginBlockComment(true);
       lw.println("Decrement the array's internal reference count by one.");
@@ -274,44 +254,40 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println("void");
-    lw.println(generateDelRefName(d_id) + "(" +
-               IOR.getArrayName(d_id) + "* array)" + terminator);
+    lw.println(generateDelRefName(d_id) + "(");
+    lw.tab();
+    lw.println(IOR.getArrayName(d_id) + "* array)" + terminator);
+    lw.backTab();
     
   }
 
-  public static String generateAddRefName(SymbolID id)
-  {
+  public static String generateAddRefName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_addRef";
   }
   
-  private void generateAddRefSig(LanguageWriter lw,
-                                 String terminator)
-  {
+  private void generateAddRefSig(LanguageWriter lw, String terminator) {
     if (d_comments){ 
       lw.beginBlockComment(true);
       lw.println("Increment the array's internal reference count by one.");
       lw.endBlockComment(true);
     }
     lw.println("void");
-    lw.println(generateAddRefName(d_id) + "(" +
-               IOR.getArrayName(d_id) + "* array)" + terminator);
+    lw.println(generateAddRefName(d_id) + "(");
+    lw.tab();
+    lw.println(IOR.getArrayName(d_id) + "* array)" + terminator);
+    lw.backTab();
     
   }
 
-  public static String generateGetName(SymbolID id)
-  {
+  public static String generateGetName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_get";
   }
 
-  public static String generateGetName(SymbolID id, int num)
-  {
+  public static String generateGetName(SymbolID id, int num) {
     return generateGetName(id) + num;
   }
 
-  private void generateGetSig(LanguageWriter lw,
-                              int num,
-                              String terminator)
-  {
+  private void generateGetSig(LanguageWriter lw, int num, String terminator) {
     String methodName = generateGetName(d_id, num);
     String arrayName = IOR.getArrayName(d_id) + "*";
     if (d_comments) {
@@ -332,17 +308,17 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println(getTypeName());
-    lw.println(methodName + "(const " + arrayName + " array,");
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println("const " + arrayName + " array,");
     for(int i = 1; i <= num; ++i){
-      lw.printSpaces(methodName.length()+1);
       lw.println("const int32_t i" + i + 
                  ((i < num) ? "," : (")" + terminator)));
     }
+    lw.backTab();
   }
 
-  private void generateGetSig(LanguageWriter lw,
-                              String terminator)
-  {
+  private void generateGetSig(LanguageWriter lw, String terminator) {
     String methodName = generateGetName(d_id);
     String arrayName = IOR.getArrayName(d_id) + "*";
     if (d_comments) {
@@ -353,25 +329,22 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println(getTypeName());
-    lw.println(methodName + "(const " + arrayName + " array,");
-    lw.printSpaces(methodName.length()+1);
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println("const " + arrayName + " array,");
     lw.println("const int32_t indices[])" + terminator);
+    lw.backTab();
   }
 
-  public static String generateSetName(SymbolID id)
-  {
+  public static String generateSetName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_set";
   }
 
-  public static String generateSetName(SymbolID id, int num)
-  {
+  public static String generateSetName(SymbolID id, int num) {
     return generateSetName(id) + num;
   }
 
-  private void generateSetSig(LanguageWriter lw,
-                              int num,
-                              String terminator)
-  {
+  private void generateSetSig(LanguageWriter lw, int num, String terminator) {
     String methodName = generateSetName(d_id, num);
     String arrayName = IOR.getArrayName(d_id) + "*";
     if (d_comments) {
@@ -392,18 +365,17 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println("void");
-    lw.println(methodName + "(" + arrayName + " array,");
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println(arrayName + " array,");
     for(int i = 1; i <= num; ++i){
-      lw.printSpaces(methodName.length()+1);
       lw.println("const int32_t i" + i + ",");
     }
-    lw.printSpaces(methodName.length()+1);
     lw.println(getTypeName() + " const value)" + terminator);
+    lw.backTab();
   }
 
-  private void generateSetSig(LanguageWriter lw,
-                              String terminator)
-  {
+  private void generateSetSig(LanguageWriter lw, String terminator) {
     String methodName = generateSetName(d_id);
     String arrayName = IOR.getArrayName(d_id) + "*";
     if (d_comments) {
@@ -414,21 +386,19 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println("void");
-    lw.println(methodName + "(" + arrayName + " array,");
-    lw.printSpaces(methodName.length()+1);
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println(arrayName + " array,");
     lw.println("const int32_t indices[],");
-    lw.printSpaces(methodName.length()+1);
     lw.println(getTypeName() + " const value)" + terminator);
+    lw.backTab();
   }
 
-  public static String generateDimenName(SymbolID id)
-  {
+  public static String generateDimenName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_dimen";
   }
 
-  private void generateDimenSig(LanguageWriter lw,
-                                String terminator)
-  {
+  private void generateDimenSig(LanguageWriter lw, String terminator) {
     String methodName = generateDimenName(d_id);
     String arrayName = IOR.getArrayName(d_id) + "*";
     if (d_comments) {
@@ -438,21 +408,18 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println("int32_t");
-    lw.println(methodName + "(const " + arrayName + " array)" +
-               terminator);
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println("const " + arrayName + " array)" + terminator);
+    lw.backTab();
   }
 
-  public static String generateBoundName(SymbolID id,
-                                             String direction)
-  {
+  public static String generateBoundName(SymbolID id, String direction) {
     return IOR.getSymbolName(id) + "__array_" + direction;
   }
 
-  private void generateBoundSig(LanguageWriter lw,
-                                String direction,
-                                String desc,
-                                String bad,
-                                String terminator)
+  private void generateBoundSig(LanguageWriter lw, String direction,
+                                String desc, String bad, String terminator)
   {
     String methodName = generateBoundName(d_id, direction);
     String arrayName = IOR.getArrayName(d_id) + "*";
@@ -464,21 +431,20 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println("int32_t");
-    lw.println(methodName + "(const " + arrayName + " array,");
-    lw.printSpaces(methodName.length()+1);
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println("const " + arrayName + " array,");
     lw.println("const int32_t ind)" + terminator);
+    lw.backTab();
   }
 
-  public static String generateOrderName(SymbolID id,
-                                         String order)
-  {
+  public static String generateOrderName(SymbolID id, String order) {
     return IOR.getSymbolName(id) + "__array_is"
       + order.substring(0,1).toUpperCase()
       + order.substring(1) + "Order";
   }
 
-  private void generateOrderSig(LanguageWriter lw,
-                                String order,
+  private void generateOrderSig(LanguageWriter lw, String order,
                                 String terminator)
   {
     String methodName = generateOrderName(d_id, order);
@@ -491,17 +457,17 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println("int");
-    lw.println(methodName + "(const " + arrayName + " array)" +
-               terminator);
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println("const " + arrayName + " array)" + terminator);
+    lw.backTab();
   }
 
-  public static String generateCopyName(SymbolID id)
-  {
+  public static String generateCopyName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_copy";
   }
 
-  private void generateCopySig(LanguageWriter lw, String terminator)
-  {
+  private void generateCopySig(LanguageWriter lw, String terminator) {
     String methodName = generateCopyName(d_id);
     String arrayName = IOR.getArrayName(d_id) + "*";
     if (d_comments) {
@@ -529,18 +495,18 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println("void");
-    lw.println(methodName + "(const " + arrayName + " src,");
-    lw.printSpaces(methodName.length() + 7);
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println("const " + arrayName + " src,");
     lw.println(arrayName + " dest)" + terminator);
+    lw.backTab();
   }
 
-  public static String generateSliceName(SymbolID id)
-  {
+  public static String generateSliceName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_slice";
   }
 
-  private void generateSliceSig(LanguageWriter lw, String terminator)
-  {
+  private void generateSliceSig(LanguageWriter lw, String terminator) {
     String methodName = generateSliceName(d_id);
     String arrayName = IOR.getArrayName(d_id) + "*";
     if (d_comments) {
@@ -608,26 +574,22 @@ public class ArrayMethods {
       lw.endBlockComment(true);
     }
     lw.println(arrayName);
-    lw.println(methodName + "(" + arrayName + " src,");
-    lw.printSpaces(methodName.length() + 7);
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println(arrayName + " src,");
     lw.println("int32_t        dimen,");
-    lw.printSpaces(methodName.length() + 7);
     lw.println("const int32_t  numElem[],");
-    lw.printSpaces(methodName.length() + 7);
     lw.println("const int32_t  *srcStart,");
-    lw.printSpaces(methodName.length() + 7);
     lw.println("const int32_t  *srcStride,");
-    lw.printSpaces(methodName.length() + 7);
     lw.println("const int32_t  *newStart)" + terminator);
+    lw.backTab();
   }
 
-  public static String generateEnsureName(SymbolID id)
-  {
+  public static String generateEnsureName(SymbolID id) {
     return IOR.getSymbolName(id) + "__array_ensure";
   }
 
-  private void generateEnsureSig(LanguageWriter lw, String terminator)
-  {
+  private void generateEnsureSig(LanguageWriter lw, String terminator) {
     String methodName = generateEnsureName(d_id);
     String arrayName = IOR.getArrayName(d_id) + "*";
     if (d_comments) {
@@ -644,27 +606,28 @@ public class ArrayMethods {
       lw.println("array.");
       lw.println();
       lw.println("The ordering parameter should be one of the constants");
-      lw.println("defined in enum SIDL_array_ordering");
-      lw.println("(e.g. SIDL_general_order, SIDL_column_major_order, or");
-      lw.println("SIDL_row_major_order). If you specify");
-      lw.println("SIDL_general_order, this routine will only check the");
-      lw.println("dimension because any matrix is SIDL_general_order.");
+      lw.println("defined in enum sidl_array_ordering");
+      lw.println("(e.g. sidl_general_order, sidl_column_major_order, or");
+      lw.println("sidl_row_major_order). If you specify");
+      lw.println("sidl_general_order, this routine will only check the");
+      lw.println("dimension because any matrix is sidl_general_order.");
       lw.println();
       lw.println("The caller assumes ownership of the returned reference");
       lw.println("unless it's NULL.");
       lw.endBlockComment(true);
     }
     lw.println(arrayName);
-    lw.println(methodName + "(" + arrayName + " src,");
-    lw.printSpaces(methodName.length() + 1);
+    lw.println(methodName + "(");
+    lw.tab();
+    lw.println(arrayName + " src,");
     lw.println("int32_t dimen,");
-    lw.printSpaces(methodName.length() + 1);
     lw.println("int     ordering)" + terminator);
+    lw.backTab();
   }
 
-  public void generateHeader(LanguageWriter lw)
-  {
+  public void generateHeader(LanguageWriter lw) {
     int i;
+
     generateCreateSig(lw, "Col", "column", ";");
     lw.println();
 
@@ -720,6 +683,9 @@ public class ArrayMethods {
     generateBoundSig(lw, "upper", "upper bound", "-1", ";");
     lw.println();
 
+    generateBoundSig(lw, "length", "length", "-1", ";");
+    lw.println();
+
     generateBoundSig(lw, "stride", "stride", "0", ";");
     lw.println();
 
@@ -739,178 +705,161 @@ public class ArrayMethods {
     lw.println();
   }
 
-  private void generateCreate(LanguageWriter lw,
-                              String shortType,
+  private void generateCreate(LanguageWriter lw, String shortType,
                               String longType)
   {
     generateCreateSig(lw, shortType, longType, "");
     lw.println("{");
-    lw.increaseTabLevel();
-    lw.println("return (" + IOR.getArrayName(d_id) + "*)" +
-               generateCreateName(d_implID, shortType) +
-               "(dimen, lower, upper);");
-    lw.decreaseTabLevel();
+    lw.tab();
+    lw.println("return (" + IOR.getArrayName(d_id) + "*)" 
+       + generateCreateName(d_implID, shortType) + "(dimen, lower, upper);");
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateOneD(LanguageWriter lw)
-  {
+  private void generateOneD(LanguageWriter lw) {
     generateOneDSig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
-    lw.println("return (" + IOR.getArrayName(d_id) + "*)" +
-               generateOneDName(d_implID) + "(len);");
-    lw.decreaseTabLevel();
+    lw.tab();
+    lw.println("return (" + IOR.getArrayName(d_id) + "*)" 
+      + generateOneDName(d_implID) + "(len);");
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateOneDInit(LanguageWriter lw)
-  {
+  private void generateOneDInit(LanguageWriter lw) {
     generateOneDInitSig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
-    lw.println("return (" + IOR.getArrayName(d_id) + "*)" +
-               generateOneDInitName(d_implID) + "(len, (" +
-               d_implDataType + "*)data);");
-    lw.decreaseTabLevel();
+    lw.tab();
+    lw.println("return (" + IOR.getArrayName(d_id) + "*)" 
+      + generateOneDInitName(d_implID) + "(len, (" + d_implDataType 
+      + "*)data);");
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateTwoD(LanguageWriter lw,
-                            String shortType,
+  private void generateTwoD(LanguageWriter lw, String shortType,
                             String longType)
   {
     generateTwoDSig(lw, shortType, longType, "");
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("return (" + IOR.getArrayName(d_id) + "*)" +
                generateTwoDName(d_implID, shortType) + "(m, n);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateBorrow(LanguageWriter lw)
-  {
+  private void generateBorrow(LanguageWriter lw) {
     generateBorrowSig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("return (" + IOR.getArrayName(d_id) + "*)" +
                generateBorrowName(d_implID) + "(");
-    lw.increaseTabLevel();
+    lw.tab();
     if (!d_isEnum) {
-      lw.println("(struct SIDL_BaseInterface__object **)");
+      lw.println("(struct sidl_BaseInterface__object **)");
     }
     lw.println("firstElement, dimen, lower, upper, stride);");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
   
-  private void generateSmartCopy(LanguageWriter lw)
-  {
+  private void generateSmartCopy(LanguageWriter lw) {
     generateSmartCopySig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("return (" + IOR.getArrayName(d_id) + "*)");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(generateSmartCopyName(d_implID) + 
                "((" + d_implArrayType + ")array);");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
   
-  private void generateDelRef(LanguageWriter lw)
-  {
+  private void generateDelRef(LanguageWriter lw) {
     generateDelRefSig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
-    lw.println(generateDelRefName(d_implID) + "((" +
-               d_implArrayType + ")array);");
-    lw.decreaseTabLevel();
+    lw.tab();
+    lw.println(generateDelRefName(d_implID) + "((" + d_implArrayType 
+      + ")array);");
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateAddRef(LanguageWriter lw)
-  {
+  private void generateAddRef(LanguageWriter lw) {
     generateAddRefSig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
-    lw.println(generateAddRefName(d_implID) + "((" +
-               d_implArrayType + ")array);");
-    lw.decreaseTabLevel();
+    lw.tab();
+    lw.println(generateAddRefName(d_implID) + "((" + d_implArrayType 
+      + ")array);");
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateGet(LanguageWriter lw,
-                           int num)
-  {
+  private void generateGet(LanguageWriter lw, int num) {
     generateGetSig(lw, num, "");
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("return (" + getTypeName() + ")");
-    lw.increaseTabLevel();
-    lw.println(generateGetName(d_implID, num) + "((const " +
-               d_implArrayType + ")array");
+    lw.tab();
+    lw.println(generateGetName(d_implID, num) + "((const " + d_implArrayType 
+      + ")array");
     for(int i = 1 ; i <= num; ++i){
       lw.print(", i" + i);
     }
     lw.println(");");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateGet(LanguageWriter lw)
-  {
+  private void generateGet(LanguageWriter lw) {
     generateGetSig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("return (" + getTypeName() + ")");
-    lw.increaseTabLevel();
-    lw.println(generateGetName(d_implID) + "((const " +
-               d_implArrayType + ")array, indices);");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.tab();
+    lw.println(generateGetName(d_implID) + "((const " + d_implArrayType 
+      + ")array, indices);");
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateSet(LanguageWriter lw,
-                           int num)
-  {
+  private void generateSet(LanguageWriter lw, int num) {
     generateSetSig(lw, num, "");
     lw.println("{");
-    lw.increaseTabLevel();
-    lw.println(generateSetName(d_implID, num) + "((" +
-               d_implArrayType + ")array");
+    lw.tab();
+    lw.println(generateSetName(d_implID, num) + "((" + d_implArrayType 
+      + ")array");
     for(int i = 1 ; i <= num; ++i){
       lw.print(", i" + i);
     }
     lw.println(", (" + d_implDataType + ")value);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateSet(LanguageWriter lw)
-  {
+  private void generateSet(LanguageWriter lw) {
     generateSetSig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
-    lw.println(generateSetName(d_implID) + "((" +
-               d_implArrayType + ")array, indices, (" +
-               d_implDataType + ")value);");
-    lw.decreaseTabLevel();
+    lw.tab();
+    lw.println(generateSetName(d_implID) + "((" + d_implArrayType 
+      + ")array, indices, (" + d_implDataType + ")value);");
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateDimen(LanguageWriter lw)
-  {
+  private void generateDimen(LanguageWriter lw) {
     generateDimenSig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
-    lw.println("return " + generateDimenName(d_implID) + "((" +
-               d_implArrayType + ")array);");
-    lw.decreaseTabLevel();
+    lw.tab();
+    lw.println("return " + generateDimenName(d_implID) + "((" + d_implArrayType
+      + ")array);");
+    lw.backTab();
     lw.println("}");
   }
 
@@ -921,70 +870,63 @@ public class ArrayMethods {
   {
     generateBoundSig(lw, direction, desc, bad, "");
     lw.println("{");
-    lw.increaseTabLevel();
-    lw.println("return " + generateBoundName(d_implID, direction) +
-               "((" + d_implArrayType + ")array, ind);");
-    lw.decreaseTabLevel();
+    lw.tab();
+    lw.println("return " + generateBoundName(d_implID, direction) + "((" 
+      + d_implArrayType + ")array, ind);");
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateOrder(LanguageWriter lw,
-                             String order)
-  {
+  private void generateOrder(LanguageWriter lw, String order) {
     generateOrderSig(lw, order, "");
     lw.println("{");
-    lw.increaseTabLevel();
-    lw.println("return " + generateOrderName(d_implID, order) +
-               "((" + d_implArrayType + ")array);");
-    lw.decreaseTabLevel();
+    lw.tab();
+    lw.println("return " + generateOrderName(d_implID, order) + "((" 
+      + d_implArrayType + ")array);");
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateCopy(LanguageWriter lw)
-  {
+  private void generateCopy(LanguageWriter lw) {
     generateCopySig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
-    lw.println(generateCopyName(d_implID) +
-               "((const " + d_implArrayType + ")src,");
+    lw.tab();
+    lw.println(generateCopyName(d_implID) + "((const " + d_implArrayType 
+      + ")src,");
     lw.printSpaces(generateCopyName(d_implID).length() + 1);
     lw.println("(" + d_implArrayType + ")dest);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateSlice(LanguageWriter lw)
-  {
+  private void generateSlice(LanguageWriter lw) {
     generateSliceSig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("return (" + IOR.getArrayName(d_id) + "*)");
-    lw.increaseTabLevel();
-    lw.println(generateSliceName(d_implID) +
-               "((" + d_implArrayType + ")src,");
+    lw.tab();
+    lw.println(generateSliceName(d_implID) + "((" + d_implArrayType + ")src,");
     lw.printSpaces(generateSliceName(d_implID).length() + 1);
     lw.println("dimen, numElem, srcStart, srcStride, newStart);");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
 
-  private void generateEnsure(LanguageWriter lw)
-  {
+  private void generateEnsure(LanguageWriter lw) {
     generateEnsureSig(lw, "");
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("return (" + IOR.getArrayName(d_id) + "*)");
-    lw.increaseTabLevel();
-    lw.println(generateEnsureName(d_implID) +
-               "((" + d_implArrayType + ")src, dimen, ordering);");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.tab();
+    lw.println(generateEnsureName(d_implID) + "((" + d_implArrayType 
+      + ")src, dimen, ordering);");
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
 
-  public void generateStub(LanguageWriter lw)
-  {
+  public void generateStub(LanguageWriter lw) {
     int i;
     generateCreate(lw, "Col", "column");
     lw.println();
@@ -1041,6 +983,9 @@ public class ArrayMethods {
     generateBound(lw, "upper", "upper bound", "-1");
     lw.println();
 
+    generateBound(lw, "length", "length", "-1");
+    lw.println();
+
     generateBound(lw, "stride", "stride", "0");
     lw.println();
 
