@@ -1,8 +1,7 @@
 //
 // File:        ImplHeader.java
 // Package:     gov.llnl.babel.backend.c
-// Release:     $Name: release-0-8-8 $
-// Revision:    @(#) $Id: ImplHeader.java,v 1.34 2003/02/04 23:49:08 epperly Exp $
+// Revision:    @(#) $Id: ImplHeader.java 4462 2005-03-23 19:29:24Z leek2 $
 // Description: generate C implementation header to a pretty writer stream
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -39,7 +38,6 @@ import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.Utilities;
 import gov.llnl.babel.backend.c.C;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
-import gov.llnl.babel.backend.writers.LanguageWriter;
 import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.Class;
 import gov.llnl.babel.symbols.Extendable;
@@ -51,7 +49,6 @@ import gov.llnl.babel.symbols.Type;
 import java.util.Collection;
 import java.util.HashSet;
 import java.util.Iterator;
-import java.util.List;
 import java.util.Set;
 
 /**
@@ -62,6 +59,7 @@ import java.util.Set;
  * writer stream is not closed by this object.
  */
 public class ImplHeader {
+
   /**
    * The writer is the output stream for the new file. The content of the
    * new file is writtern to this writer.
@@ -107,10 +105,9 @@ public class ImplHeader {
    *    violations of the data type invariants.
    *
    */
-  public static void generateCode(Class cls,
-                                  LanguageWriterForC writer,
+  public static void generateCode(Class cls, LanguageWriterForC writer,
                                   CodeSplicer splicer)
-     throws CodeGenerationException
+    throws CodeGenerationException
   {
     ImplHeader header = new ImplHeader(writer, splicer);
     header.generateCode(cls);
@@ -124,8 +121,8 @@ public class ImplHeader {
   private void writePrologue(Class cls) {
     final SymbolID id = cls.getSymbolID();
     String header = C.getImplHeaderFile(id);
-    d_writer.writeBanner(cls, header, true, 
-      CodeConstants.C_DESC_IMPL_PREFIX + id.getFullName());
+    d_writer.writeBanner(cls, header, true, CodeConstants.C_DESC_IMPL_PREFIX 
+      + id.getFullName());
     d_writer.openHeaderGuard(header);
   }
 
@@ -134,11 +131,13 @@ public class ImplHeader {
    *
    * @param cls  the <code>Class</code> whose header will be written.
    */
-  private void writeIncludeSection(Class cls,
-                                   Collection localRefs) {
+  private void writeIncludeSection(Class cls, Collection localRefs) {
     final SymbolID id = cls.getSymbolID();
     String includes = id.getFullName() + "." + "_includes";
-    d_writer.generateInclude("SIDL_header.h", true);
+    d_writer.generateInclude("sidl_header.h", true);
+    if(cls.hasOverwrittenMethods()) {
+      d_writer.generateInclude(IOR.getHeaderFile(id), true);
+    }
 
     Iterator i = localRefs.iterator();
     while (i.hasNext()) {
@@ -148,14 +147,7 @@ public class ImplHeader {
     i = null;
 
     d_writer.println();
-    if (d_splicer.hasSymbol(includes)) {
-      d_splicer.outputSymbolEdits(includes, d_writer.getPrintWriter());
-    }
-    else{
-      d_writer.writeCommentLine(CodeSplicer.getBeginString(includes));
-      d_writer.writeCommentLine("Put additional include files here...");
-      d_writer.writeCommentLine(CodeSplicer.getEndString(includes));
-    }
+    d_splicer.splice(includes, d_writer, "include files");
     d_writer.println();
   }
 
@@ -177,9 +169,9 @@ public class ImplHeader {
     final SymbolID id = cls.getSymbolID();
     d_writer.println("extern " + C.getDataName(id) + "*");
     d_writer.println(C.getDataGetName(id) + "(");
-    d_writer.increaseTabLevel();
+    d_writer.tab();
     d_writer.println(C.getObjectName(id)+ ");");
-    d_writer.decreaseTabLevel();
+    d_writer.backTab();
     d_writer.println();
   }
 
@@ -192,10 +184,10 @@ public class ImplHeader {
     final SymbolID id = cls.getSymbolID();
     d_writer.println("extern void");
     d_writer.println(C.getDataSetName(id) + "(");
-    d_writer.increaseTabLevel();
+    d_writer.tab();
     d_writer.println(C.getObjectName(id) + ",");
     d_writer.println(C.getDataName(id) + "*);");
-    d_writer.decreaseTabLevel();
+    d_writer.backTab();
     d_writer.println();
   }
 
@@ -211,64 +203,121 @@ public class ImplHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private void writeIORWrapperDecl(Class cls,
-                                   String funcName,
-                                   Method iorMethod)
+  private void writeIORWrapperDecl(Class cls, String funcName, Method iorMethod)
     throws CodeGenerationException
   {
     final SymbolID id = cls.getSymbolID();
-    Iterator args = Utilities.extendArgs(id, iorMethod).iterator();
+    Iterator args = Utilities.extendArgs(id, iorMethod, false).iterator();
     d_writer.println("extern void");
     d_writer.print(funcName);
     d_writer.println("(");
-    d_writer.increaseTabLevel();
+    d_writer.tab();
     while (args.hasNext()) {
       Argument a = (Argument)args.next();
-      d_writer.print(C.getArgumentString(a));
+      d_writer.print(getArgumentString(a));
       if (args.hasNext()) {
         d_writer.print(", ");
       }
     }
     d_writer.println(");");
-    d_writer.decreaseTabLevel();
+    d_writer.backTab();
     d_writer.println();
   }
 
+   /**
+    * Generate an argument string for the specified sidl argument.
+    * The formal argument name is not included.
+    *
+    * WARNING:  In general we do not want the object pointer version
+    *           of the argument's type so the IOR source code has
+    *           been changed to use the C version when that version
+    *           is needed leaving this one for all other cases (especially
+    *           back ends that rely on it).
+    */
+   private static String getArgumentString(Argument arg)
+     throws CodeGenerationException
+   {
+     return IOR.getArgumentString(arg, false, false, false);
+   }
+
+  /**
+   * Write the method signature with arguments indented one level.  The 
+   * caller is expected to decrease the indentation one level after
+   * terminating the signature with either a simple println() or a
+   * println(";") as needed.
+   *
+   * @param lw        the target language writer 
+   * @param meth      the method 
+   * @param suffix    the suffix to attach to the name, typically for
+   *                  the pre- and post-method hooks.
+   * @param self      the self variable, if appropriate
+   * @param addExtern TRUE if the signature is to include the "extern" 
+   *                  modifier (as in the header file case).
+   * @param isIfc     TRUE if the method belongs to an interface
+   * @param id        the name of the symbol owning the method
+   * @exception gov.llnl.babel.backend.CodeGenerationException
+   *    this is a catch all exception. It can be caused by I/O trouble or
+   *    violations of the data type invariants.
+   */
+  public static void writeMethodSignature(LanguageWriterForC lw, Method meth,
+                                          String suffix, String self,
+                                          boolean addExtern, boolean isIfc, 
+                                          boolean addType, SymbolID id)
+    throws CodeGenerationException
+  {
+    boolean isOrig        = suffix.equals("");
+    Type    returnType    = meth.getReturnType();
+    String  returnTypeStr = getReturnString(returnType);
+    if (addExtern) {
+      lw.print("extern ");
+    }
+    if (isOrig) {
+      lw.println(returnTypeStr);
+    } else {
+      lw.println("void");
+    }
+    lw.print(C.getMethodImplName(id, meth.getLongMethodName()) + suffix);
+    lw.println("(");
+    lw.tab();
+    C.generateArgumentList(lw, self, isIfc, id, meth, true, addType, false, 
+                           isOrig, suffix.equals(IOR.GENERIC_POST_SUFFIX), true,
+                           true);
+    lw.print(")");
+  }
+
+   /**
+    * Generate a return string for the specified SIDL type.  Most
+    * of the SIDL return strings are listed in the static structures defined
+    * at the start of this class.  Symbol types and array types require
+    * special processing.
+    */
+   private static String getReturnString(Type type)
+     throws CodeGenerationException
+   {
+     return IOR.getReturnString(type, false, false);
+   }
+
   /**
    * Write a method declaration for a generic implementation method.
    *
    * @param lw       the language writer to write to
+   * @param isIfc     TRUE if the method belongs to an interface
    * @param m        the method description
+   * @param suffix   the suffix to attach to the name, typically for
+   *                 the pre- and post-method hooks.
    * @param id       the name of the symbol owning the method
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private static void writeMethodDecl(LanguageWriter lw,
-                                     Method m,
-                                     SymbolID id)
+  private static void writeMethodDecl(LanguageWriterForC lw, boolean isIfc, 
+                                      Method m, String suffix, SymbolID id)
     throws CodeGenerationException
   {
-    Iterator arguments = Utilities.extendArgs(id, m).iterator();
-    lw.print("extern ");
-    lw.println(C.getReturnString(m.getReturnType()));
-    lw.print(C.getMethodImplName(id, m.getLongMethodName()));
-    lw.println("(");
-    if (arguments.hasNext()){ 
-      lw.increaseTabLevel();
-      while (arguments.hasNext()) {
-        Argument arg = (Argument)arguments.next();
-        lw.print(C.getArgumentString(arg));
-        if (arguments.hasNext()) {
-          lw.println(",");
-        }
-      }
-      lw.println(");");
-      lw.decreaseTabLevel();
-    }
-    else {
-      lw.print("void);");
-    }
+    writeMethodSignature(lw, m, suffix, Utilities.s_self, true, isIfc, true, 
+                         id);
+    lw.println(";");
+    lw.backTab();
     lw.println();
   }
 
@@ -285,8 +334,7 @@ public class ImplHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  public static void writeMethodDecls(LanguageWriter lw,
-                                      Class cls,
+  public static void writeMethodDecls(LanguageWriterForC lw, Class cls,
                                       boolean isStatic)
     throws CodeGenerationException
   {
@@ -300,7 +348,13 @@ public class ImplHeader {
     while (i.hasNext()) {
       Method m = (Method)i.next();
       if (!m.isAbstract()) {
-        writeMethodDecl(lw, m, id);
+        if (IOR.supportInterceptors(cls)) {
+          writeMethodDecl(lw, false, m, IOR.GENERIC_PRE_SUFFIX, id);
+          writeMethodDecl(lw, false, m, "", id);
+          writeMethodDecl(lw, false, m, IOR.GENERIC_POST_SUFFIX, id);
+        } else {
+          writeMethodDecl(lw, false, m, "", id);
+        }
       }
     }
   }
@@ -315,13 +369,31 @@ public class ImplHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  public static void writeBuiltinDecls(LanguageWriter lw,
-                                       Class cls)
+  public static void writeBuiltinDecls(LanguageWriterForC lw, Class cls)
     throws CodeGenerationException
   {
     final SymbolID id = cls.getSymbolID();
-    writeMethodDecl(lw,IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id), id);
-    writeMethodDecl(lw,IOR.getBuiltinMethod(IOR.DESTRUCTOR, id), id);
+    if(cls.hasOverwrittenMethods()) {
+      //function from the Impl file that sets the superEPV
+      lw.println("extern void " + C.getObjectName(id) + "__superEPV(");
+      lw.println("struct " + C.getObjectName(cls.getParentClass().getSymbolID())
+        + "__epv*);");
+      lw.println();
+    }
+    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.LOAD, id), "", 
+                    id);
+    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CONSTRUCTOR, id), "", 
+                    id);
+    writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.DESTRUCTOR, id), "", 
+                    id);
+    if (IOR.supportAssertions(cls)) {
+      if (cls.hasStaticMethod(true)) {
+        writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CHECK_ERROR, id, 
+                        true), "", id);
+      }
+      writeMethodDecl(lw, false, IOR.getBuiltinMethod(IOR.CHECK_ERROR, id), "",
+                      id);
+    }
   }
 
   public Collection getLocalReferences(Extendable ext) {
@@ -334,7 +406,7 @@ public class ImplHeader {
       refs.addAll(m.getSymbolReferences());
       if (!(hasThrows || m.getThrows().isEmpty())) {
         Symbol sym =
-          d_symtab.lookupSymbol(BabelConfiguration.getBaseException());
+          d_symtab.lookupSymbol(BabelConfiguration.getBaseExceptionInterface());
         if (sym != null) {
           hasThrows = true;
           refs.add(sym.getSymbolID());
@@ -357,21 +429,13 @@ public class ImplHeader {
     d_writer.writeComment("Private data for class " + name, false);
 
     d_writer.println(C.getDataName(id) +" {");
-    d_writer.increaseTabLevel();
+    d_writer.tab();
 
     String data = cls.getFullName() + "." + "_data";
-    if (d_splicer.hasSymbol(data)) {
-      d_splicer.outputSymbolEdits(data, d_writer.getPrintWriter());
-    } else {
-      d_writer.writeCommentLine(CodeSplicer.getBeginString(data));
-      d_writer.writeCommentLine("Put private data members here...");
-      d_writer.println(
-        "int ignore; " +
-        "/* dummy to force non-empty struct; remove if you add data */");
-      d_writer.writeCommentLine(CodeSplicer.getEndString(data));
-    }
+    d_splicer.splice(data, d_writer, "private data members", "int ignore; " 
+      + "/* dummy to force non-empty struct; remove if you add data */");
 
-    d_writer.decreaseTabLevel();
+    d_writer.backTab();
     d_writer.println("};");
     d_writer.println();
   }
@@ -386,9 +450,7 @@ public class ImplHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  public void generateCode(Class cls)
-    throws CodeGenerationException
-  {
+  public void generateCode(Class cls) throws CodeGenerationException {
     final SymbolID id = cls.getSymbolID();
     Collection localRefs = getLocalReferences(cls);
     writePrologue(cls);
@@ -396,9 +458,8 @@ public class ImplHeader {
     writeForwardStructDecls(cls, localRefs);
     writePrivateDataType(cls);
     d_writer.openCxxExtern();
-    d_writer.writeComment(
-      "Access functions for class private data and built-in methods",
-                          false);
+    d_writer.writeComment("Access functions for class private data and built-in"
+      + " methods", false);
     writeGetPrivateDataDecl(cls);
     writeSetPrivateDataDecl(cls);
     if (cls.getParentClass() == null) {
@@ -415,6 +476,7 @@ public class ImplHeader {
      */
     if (d_splicer.hasUnusedSymbolEdits()) {
       d_writer.println();
+      d_writer.printlnUnformatted("#error File has unused splicer blocks.");
       d_writer.beginBlockComment(true);
       d_writer.println(CodeConstants.C_BEGIN_UNREFERENCED_METHODS);
       d_writer.println(CodeConstants.C_UNREFERENCED_COMMENT1);
