@@ -1,8 +1,7 @@
 //
 // File:        StubHeader.java
 // Package:     gov.llnl.babel.backend.c
-// Release:     $Name: release-0-8-8 $
-// Revision:    @(#) $Id: StubHeader.java,v 1.28 2003/09/18 14:46:41 epperly Exp $
+// Revision:    @(#) $Id: StubHeader.java 4449 2005-03-21 18:00:39Z leek2 $
 // Description: write C client (i.e., stub) header to a pretty writer stream
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -45,6 +44,8 @@ import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.Package;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
+import gov.llnl.babel.symbols.Type;
+import gov.llnl.babel.symbols.Class;
 import gov.llnl.babel.BabelConfiguration;
 import java.util.Collection;
 import java.util.HashSet;
@@ -61,7 +62,20 @@ import java.util.Set;
  */
 public class StubHeader {
   private final static String SIDL_EXCEPTION =
-    BabelConfiguration.getBaseException();
+    BabelConfiguration.getBaseExceptionClass();
+  private final static String SIDL_EXCEPTION_INTERFACE =
+    BabelConfiguration.getBaseExceptionInterface();
+
+  public final static String s_epv       = IOR.getEPVVar(IOR.PUBLIC_EPV);
+  public final static String s_self      = Utilities.s_self;
+  public final static String s_sepv_func = "_getSEPV()";
+
+  /**
+   * Indices associated with the special, stub-only built-in methods.
+   */
+  public final static int DUMP_STATS       = 0;
+  public final static int SET_CHECKING     = 1;
+  public final static int SET_INTERCEPTORS = 2;
 
   private LanguageWriterForC d_writer;
 
@@ -83,8 +97,7 @@ public class StubHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  public static void generateCode(Symbol             symbol,
-                                  LanguageWriterForC writer)
+  public static void generateCode(Symbol symbol, LanguageWriterForC writer)
     throws CodeGenerationException
   {
     StubHeader header = new StubHeader(writer);
@@ -118,10 +131,11 @@ public class StubHeader {
    *    this is a catch all exception. It can be caused by I/O trouble
    *    or violations of the data type invariants.
    */
-  public void generateCode(Symbol symbol)
-    throws CodeGenerationException
-  {
+  public void generateCode(Symbol symbol) throws CodeGenerationException {
     if (symbol != null) {
+      if ( symbol.getSymbolType() == Symbol.PACKAGE ) { 
+	  d_writer.skipIncludeGuard(); 
+      }
       generatePrologue(symbol);
 
       switch (symbol.getSymbolType()) {
@@ -210,8 +224,7 @@ public class StubHeader {
    *    this is a catch all exception. It can be caused by I/O trouble
    *    or violations of the data type invariants.
    */
-  private void generateExtendable(Extendable ext)
-    throws CodeGenerationException
+  private void generateExtendable(Extendable ext) throws CodeGenerationException
   {
     SymbolID id = ext.getSymbolID();
 
@@ -280,7 +293,6 @@ public class StubHeader {
    * table, then throw a code generation exception.
    */
   private void generateIncludes(Extendable ext) throws CodeGenerationException {
-
     /*
      * Create the set of include symbol identifiers.
      */
@@ -293,6 +305,8 @@ public class StubHeader {
       if (!method.getThrows().isEmpty()) {
         Symbol symbol = Utilities.lookupSymbol(SIDL_EXCEPTION);
         includes.add(symbol.getSymbolID());
+        symbol = Utilities.lookupSymbol(SIDL_EXCEPTION_INTERFACE);
+        includes.add(symbol.getSymbolID());
       }
     }
 
@@ -300,7 +314,7 @@ public class StubHeader {
      * We should always include the base header file.
      */
     d_writer.writeComment("Includes for all header dependencies.", false);
-    d_writer.generateInclude("SIDL_header.h", true);
+    d_writer.generateInclude("sidl_header.h", true);
 
     /*
      * Remove this symbol from the dependency set and iterate over symbols if
@@ -315,13 +329,42 @@ public class StubHeader {
       }
       d_writer.println();
     }
+    d_writer.generateInclude(C.getHeaderFile(
+              Utilities.lookupSymbol("sidl.io.Serializer").getSymbolID()), 
+              true);
+    d_writer.generateInclude(C.getHeaderFile(
+              Utilities.lookupSymbol("sidl.io.Deserializer").getSymbolID()), 
+              true);
   }
 
   /**
-   * Generate method prototypes for the methods in the interface or class.
+   * Generate prototypes for the specified built-in method.
    */
+  private void generateBuiltinPrototypes(SymbolID id, int stubType, 
+                                         boolean doStatic) 
+    throws CodeGenerationException 
+  {
+    d_writer.writeComment(getBuiltinComment(stubType, doStatic), true);
+    generateBuiltinSignature(d_writer, stubType, id, doStatic, ";");
+    d_writer.println();
+  }
+
+  /**
+   * Generate prototypes for the specified assertion method(s).
+   */
+  private void generateAssertionPrototypes(SymbolID id, boolean doStatic) 
+    throws CodeGenerationException 
+  {
+    generateBuiltinPrototypes(id, SET_CHECKING, doStatic);
+    generateBuiltinPrototypes(id, DUMP_STATS, doStatic);
+  }
+ 
+   /**
+    * Generate method prototypes for the methods in the interface or class.
+    */
   private void generateMethodPrototypes(Extendable ext)
-    throws CodeGenerationException {
+    throws CodeGenerationException 
+  {
     /*
      * Generate the name for this entry point vector as well as a pointer
      * to "self" for this structure.  For classes, self will be the object
@@ -330,22 +373,51 @@ public class StubHeader {
     SymbolID id   = ext.getSymbolID();
 
     /*
-     * Output the constructor if this class or interface is not abstract.
+     * Output the normal and remote constructors if this extendable is a class.
      */
     if (!ext.isAbstract()) {
       d_writer.writeComment("Constructor function for the class.", true);
-      d_writer.println(C.getSymbolName(id));
+      d_writer.println(C.getSymbolObjectPtr(id));
       d_writer.println(C.getFullMethodName(id, "_create") + "(void);");
       d_writer.println();
+
+      d_writer.writeComment("RMI constructor function for the class.", true);
+      d_writer.println(C.getSymbolName(id));
+
+      d_writer.println(C.getFullMethodName(id, "_createRemote") 
+        + "(const char *, sidl_BaseInterface *_ex);");
+
+      d_writer.println();
+
+    }
+
+    /*
+     * Output the assertion and/or interceptor method(s) if appropriate.
+     */
+    boolean hasStatic = ext.hasStaticMethod(true);
+    if (!ext.isAbstract() && !ext.isInterface()) {
+      if (IOR.supportAssertions(ext)) {
+        if (hasStatic) {
+          generateAssertionPrototypes(id, true);
+        }
+        generateAssertionPrototypes(id, false);
+      }
+      if (IOR.supportInterceptors(ext)) {
+        if (hasStatic) {
+          generateBuiltinPrototypes(id, SET_INTERCEPTORS, true);
+        }
+        generateBuiltinPrototypes(id, SET_INTERCEPTORS, false);
+      }
     }
 
     /*
      * Output each of the method prototypes from the interface or class.
      */
+    boolean isIfc = ext.isInterface();
     Collection methods = ext.getMethods(true);
     for (Iterator m = methods.iterator(); m.hasNext(); ) {
       Method method = (Method) m.next();
-      generateMethodSignature(id, method, 
+      generateMethodSignature(id, isIfc, method, 
                               d_comment_all || ext.isLocal(method));
       d_writer.println();
     }
@@ -353,24 +425,33 @@ public class StubHeader {
     /*
      * Output the cast methods for the class or interface.
      */
-    d_writer.writeComment(
-      "Cast method for interface and class type conversions.", true);
-    d_writer.println(C.getSymbolName(id));
+    d_writer.writeComment("Cast method for interface and class type "
+      + "conversions.", true);
+    d_writer.println(C.getSymbolObjectPtr(id));
     d_writer.println(C.getFullMethodName(id, "_cast") + "(");
-    d_writer.increaseTabLevel();
+    d_writer.tab();
     d_writer.println("void* obj);");
-    d_writer.decreaseTabLevel();
+    d_writer.backTab();
     d_writer.println();
 
-    d_writer.writeComment(
-      "String cast method for interface and class type conversions.", true);
+    d_writer.writeComment("String cast method for interface and class type "
+      + "conversions.", true);
     d_writer.println("void*");
     d_writer.println(C.getFullMethodName(id, "_cast2") + "(");
-    d_writer.increaseTabLevel();
+    d_writer.tab();
     d_writer.println("void* obj,");
     d_writer.println("const char* type);");
-    d_writer.decreaseTabLevel();
+    d_writer.backTab();
     d_writer.println();
+    
+    /*
+     * Output the exec methods for the class or interface.
+     */
+    Method m_exec = C.getExecMethod();
+    generateMethodSignature(id, isIfc, m_exec, true);
+
+    m_exec = C.getSExecMethod();
+    generateMethodSignature(id, isIfc, m_exec, true);
   }
 
   /**
@@ -379,6 +460,9 @@ public class StubHeader {
    * @param id the <code>SymbolID</code> of the <code>Extendable</code> whose
    *   header is being written.
    *
+   * @param isInterface TRUE if the <code>Extendable</code> is an interface,
+   *                    otherwise, use FALSE.
+   * 
    * @param method the <code>Method</code> whose signature is being output.
    * 
    * @param docComment iff <code>true</code> this will print a document
@@ -390,51 +474,125 @@ public class StubHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private void generateMethodSignature(SymbolID id,
-                                       Method   method,
-                                       boolean  docComment)
+  private void generateMethodSignature(SymbolID id, boolean isInterface,
+                                       Method method, boolean docComment)
     throws CodeGenerationException
   {
     if (docComment) {
       d_writer.writeComment(method, true);
     }
-    d_writer.println(C.getReturnString(method.getReturnType()));
+    d_writer.println(getReturnString(method.getReturnType()));
     d_writer.print(C.getFullMethodName(id, method));
     d_writer.println("(");
-    d_writer.increaseTabLevel();
-    generateArgumentList(id, method);
+    d_writer.tab();
+    C.generateArgumentList(d_writer, s_self, isInterface, id, method, true, 
+                           true, false, true, false, true, true);
     d_writer.println(");");
-    d_writer.decreaseTabLevel();
+    d_writer.backTab();
+  }
+
+   /**
+    * Generate a return string for the specified SIDL type.  Most
+    * of the SIDL return strings are listed in the static structures defined
+    * at the start of this class.  Symbol types and array types require
+    * special processing.
+    */
+   private static String getReturnString(Type type)
+     throws CodeGenerationException
+   {
+     return IOR.getReturnString(type, false, true);
+   }
+
+
+  /**
+   * Return the comment description associated with the specified
+   * built-in stub.
+   */
+  public static String getBuiltinComment(int type, boolean doStatic) {
+    String cmt  = null;
+    String desc = doStatic ? "static " : "";
+    switch (type) {
+      case SET_CHECKING:
+        cmt = "Method to set the " + desc + "assertion checking level.";
+        break;
+      case DUMP_STATS:
+        cmt = "Method to dump " + desc +  "assertion checking statistics.";
+        break;
+      case SET_INTERCEPTORS:
+        cmt = "Method to enable/disable " + desc +  "interceptor execution.";
+        break;
+    }
+    return cmt;
   }
 
   /**
-   * Generate the argument list, excluding the enclosing parentheses.
-   *
-   * @param id the <code>SymbolID</code> of the <code>Extendable</code> whose
-   *   header is being written.
-   *
-   * @param method the <code>Method</code> whose signature is being output.
-   *
-   * @exception gov.llnl.babel.backend.CodeGenerationException
-   *    this is a catch all exception. It can be caused by I/O trouble or
-   *    violations of the data type invariants.
+   * Generate the specified stub-only built-in method signature.
    */
-  private void generateArgumentList(SymbolID id,
-                                    Method   method)
-    throws CodeGenerationException
+  public static void generateBuiltinSignature(LanguageWriterForC lw, int type, 
+                                              SymbolID id, boolean doStatic,
+                                              String terminator)
   {
-    Iterator a = Utilities.extendArgs(id, method).iterator();
-    if (a.hasNext()) {
-      while (a.hasNext()) {
-        Argument arg = (Argument) a.next();
-        d_writer.print(C.getArgumentWithFormal(arg));
-        if (a.hasNext()) {
-          d_writer.println(",");
-        }
-      }
+    String suffix = doStatic ? IOR.s_static_suffix : "";
+    String basename = null;
+    switch (type) {
+      case SET_CHECKING:
+        basename = "_set_checking";
+        break;
+      case DUMP_STATS:
+        basename = "_dump_stats";
+        break;
+      case SET_INTERCEPTORS:
+        basename = "_set_interceptors";
+        break;
     }
-    else {
-      d_writer.print("void");
+    lw.println("void");
+    lw.println(C.getFullMethodName(id, basename + suffix) + "(");
+    lw.tab();
+    if (!doStatic) {
+      lw.println(C.getFullSelfDecl(id) + ",");
     }
+    switch (type) {
+      case SET_CHECKING:
+        lw.println("int32_t level,");
+        lw.println("double  rate,");
+        lw.println("int32_t resetCounters)" + terminator);
+        break;
+      case DUMP_STATS:
+        lw.println("const char* filename)"+ terminator);
+        break;
+      case SET_INTERCEPTORS:
+        lw.println("int32_t on)" + terminator);
+        break;
+    }
+    lw.backTab();
+  }
+
+  /**
+   * Return the comma-separated list of arguments associated with the 
+   * specified built-in method.  The entries must match the names and
+   * ordering found (above) in generateStubBuiltinSignature().
+   */
+  public static String getBuiltinArgList(int type) {
+    String args = null;
+    switch (type) {
+      case SET_CHECKING:
+        args = "level, rate, resetCounters";
+        break;
+      case DUMP_STATS:
+        args = "filename";
+        break;
+      case SET_INTERCEPTORS:
+        args = "on";
+        break;
+    }
+    return args;
+  }
+
+  /**
+   * Return a string that dereferences the specified IOR function pointer.
+   */
+  public static String getDerefFunctionPtr(String baseName, boolean doStatic) {
+    String base = doStatic ? s_sepv_func : "*" + s_self + "->" + s_epv;
+    return "(" + base + "->" + IOR.getVectorEntry(baseName) + ")";
   }
 }
