@@ -2,9 +2,8 @@
 // File:        ArrayModule.java
 // Package:     gov.llnl.babel.backend.fortran
 // Copyright:   (c) 2003 The Regents of the University of California
-// Release:     $Name: release-0-8-8 $
-// Revision:    @(#) $Revision: 1.5 $
-// Date:        $Date: 2003/09/03 15:08:42 $
+// Revision:    @(#) $Revision: 4434 $
+// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
 // Description: Create a F90 module to provide stubs for arrays
 // 
 // Copyright (c) 2003, The Regents of the University of Calfornia.
@@ -32,18 +31,17 @@
 // Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 package gov.llnl.babel.backend.fortran;
+
 import gov.llnl.babel.BabelConfiguration;
 import gov.llnl.babel.backend.CodeGenerationException;
-import gov.llnl.babel.backend.IOR;
-import gov.llnl.babel.backend.mangler.NameMangler;
+import gov.llnl.babel.backend.fortran.AbbrevHeader;
 import gov.llnl.babel.backend.mangler.FortranMangler;
+import gov.llnl.babel.backend.mangler.NameMangler;
 import gov.llnl.babel.backend.writers.LanguageWriter;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
 import gov.llnl.babel.symbols.Version;
 import java.io.UnsupportedEncodingException;
-import java.util.ArrayList;
-import java.util.List;
 import java.security.NoSuchAlgorithmException;
 
 /**
@@ -51,6 +49,11 @@ import java.security.NoSuchAlgorithmException;
  * classes and interfaces.
  */
 public class ArrayModule {
+  private final int GENERIC = -1;
+  private final int BASETYPE = 0;
+  private final int BASEINTERFACE = 1;
+  private final int USERDEFINED = 2;
+
   static private final int s_maxArray = BabelConfiguration.getMaximumArray();
 
   private NameMangler d_mang;
@@ -69,14 +72,19 @@ public class ArrayModule {
 
   private boolean d_is_base_type;
 
+  private boolean d_is_generic_array;
+
+  private int d_type;
+  private int d_casttype;
+
   private static final String[] s_constant_methods = {
-    "addRef",
     "copy",
     "createCol",
     "createRow",
+    "ensure",
+    "addRef",
     "deleteRef",
     "dimen",
-    "ensure",
     "isColumnOrder",
     "isRowOrder",
     "is_null",
@@ -85,9 +93,28 @@ public class ArrayModule {
     "set_null",
     "smartCopy",
     "stride",
-    "upper"
+    "upper",
+    "length"
   };
 
+  private static final String[] s_generic_methods = {
+    "addRef",
+    "deleteRef",
+    "dimen",
+    "type",
+    "isColumnOrder",
+    "isRowOrder",
+    "is_null",
+    "lower",
+    "not_null",
+    "set_null",
+    "smartCopy",
+    "stride",
+    "upper",
+    "length"
+  };
+
+
   /**
    * The name of the type use to hold values going in/out.
    */
@@ -102,14 +129,37 @@ public class ArrayModule {
 
   private String getPrivateName(String name, int dim)
   {
+    if(dim == 0) {
+      return name + d_privateSuffix;
+    }
     return name + dim + d_privateSuffix;
   }
 
+  /**
+   * The function returns an int that determines what sort of cast functions
+   * should be written out.  There are 3 possibilities:
+   * if d_is_base_type is true, we need to be able to cast to and from
+   * generic arrays.
+   * if this is a base interface, we also need to be able to cast to and
+   * from generic arrays
+   * However, if niether of those are true, we should only be able cast to
+   * generic arrays, not from them.
+   */
+  private int castType() {
+    if(d_is_base_type)
+      return BASETYPE;
+    else if (d_id.getFullName().compareTo(BabelConfiguration.getBaseInterface()) == 0)
+      return BASEINTERFACE;
+    else
+      return USERDEFINED;
+  }
+
   private void initImplID(int type)
     throws CodeGenerationException
   {
     d_access = true;
     d_borrow = true;
+    d_is_generic_array = false;
     switch (type) {
     case Type.VOID:
       throw new CodeGenerationException("Cannot create an array for a void");
@@ -125,7 +175,7 @@ public class ArrayModule {
       d_is_base_type = false;
       d_access = false;
       d_borrow = false;
-      d_implID = new SymbolID("SIDL.int", new Version());
+      d_implID = new SymbolID("sidl.int", new Version());
       break;
     case Type.CHAR:
     case Type.OPAQUE:
@@ -136,7 +186,7 @@ public class ArrayModule {
       // fall through intended!
     default:
       d_is_base_type = true;
-      d_implID = new SymbolID("SIDL." + (new Type(type)).getTypeString(),
+      d_implID = new SymbolID("sidl." + (new Type(type)).getTypeString(),
                               new Version());
       break;
     }
@@ -149,7 +199,7 @@ public class ArrayModule {
    *
    * @param id    the array module should be created for this
    *              id. This can be a real id or a fake id (for
-   *              SIDL.double and the like).
+   *              sidl.double and the like).
    * @param type  this should be a constant from 
    *              {@link gov.llnl.babel.symbols.Type}. It indicates
    *              the type of id.
@@ -166,6 +216,7 @@ public class ArrayModule {
       boolean isEnum = (Type.ENUM == type);
       d_id = id;
       d_lw = lw;
+      d_type = type;
       d_mang = new FortranMangler(AbbrevHeader.MAXNAME,
                                   AbbrevHeader.MAXUNMANGLED);
       if (type <= Type.STRING) {
@@ -176,6 +227,40 @@ public class ArrayModule {
       }
       d_intType = Fortran.getReturnString(new Type(Type.INT));
       initImplID(type);
+      d_casttype = castType();
+    }
+    catch (NoSuchAlgorithmException nsae) {
+      throw new CodeGenerationException(nsae.getMessage());
+    }
+  }
+
+    
+  /**
+   * SPECIAL CONSTRUCTOR for creating the GENERIC ARRAY MODULE.
+   * DO NOT CALL FOR ANY OTHER REASON!
+   *
+   * @param lw    here is where the file is created.
+   * @exception gov.llnl.babel.backend.CodeGenerationException
+   *  this means the arguments were incorrect
+   */
+  public ArrayModule(LanguageWriter lw)
+    throws CodeGenerationException
+  {
+    try {
+      d_id = new SymbolID("sidl.array", new Version());
+      d_lw = lw;
+      d_type = -1;
+      d_mang = new FortranMangler(AbbrevHeader.MAXNAME,
+                                  AbbrevHeader.MAXUNMANGLED);
+      d_implDataType = null;
+      d_intType = Fortran.getReturnString(new Type(Type.INT));
+      // Instead of calling initImplID(type), we do this directly.
+      d_access = false;
+      d_borrow = false;
+      d_is_base_type = false;
+      d_is_generic_array = true;
+      d_implID = new SymbolID("sidl", new Version());
+      d_casttype = GENERIC;
     }
     catch (NoSuchAlgorithmException nsae) {
       throw new CodeGenerationException(nsae.getMessage());
@@ -186,11 +271,11 @@ public class ArrayModule {
   {
     d_lw.println("logical function " + getPrivateName("is_null", dim) +
                  "(array)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(in) :: array");
     d_lw.println(getPrivateName("is_null", dim) +
                  " = (array%d_array .eq. 0)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end function " + getPrivateName("is_null", dim));
     d_lw.println();
   }
@@ -199,11 +284,11 @@ public class ArrayModule {
   {
     d_lw.println("logical function " + getPrivateName("not_null", dim) +
                  "(array)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(in) :: array");
     d_lw.println(getPrivateName("not_null", dim) +
                  " = (array%d_array .ne. 0)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end function " + getPrivateName("not_null", dim));
     d_lw.println();
   }
@@ -212,13 +297,13 @@ public class ArrayModule {
   {
     d_lw.println("subroutine " + getPrivateName("set_null", dim) +
                  "(array)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(out) :: array");
     d_lw.println("array%d_array = 0");
     if (d_access) {
       d_lw.println("nullify(array%d_data)");
     }
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName("set_null", dim));
     d_lw.println();
   }
@@ -242,14 +327,14 @@ public class ArrayModule {
     d_lw.endBlockComment(false);
     d_lw.println("subroutine " + getPrivateName(createFunc, dim) +
                  "(lower, upper, array)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(d_intType + ", dimension(" + dim + "), intent(in) :: lower");
     d_lw.println(d_intType +
                  ", dimension(" + dim + "), intent(in) :: upper");
     d_lw.println(getImplArrayType(dim) + ", intent(out) :: array");
     d_lw.println("external " + function);
     d_lw.println("call " + function + "(" + dim + ", lower, upper, array)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName(createFunc, dim));
     d_lw.println();
   }
@@ -262,12 +347,12 @@ public class ArrayModule {
       (d_implID.getFullName(), createFunc, "_m");
     d_lw.println("subroutine " + getPrivateName(createFunc, 1) +
                  "(len, array)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(d_intType + ", intent(in) :: len");
     d_lw.println(getImplArrayType(1) + ", intent(out) :: array");
     d_lw.println("external " + function);
     d_lw.println("call " + function + "(len, array)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName(createFunc, 1));
     d_lw.println();
   }
@@ -280,12 +365,12 @@ public class ArrayModule {
       (d_implID.getFullName(), createFunc, "_m");
     d_lw.println("subroutine " + getPrivateName(createFunc, 2) +
                  "(m, n, array)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(d_intType + ", intent(in) :: m, n");
     d_lw.println(getImplArrayType(2) + ", intent(out) :: array");
     d_lw.println("external " + function);
     d_lw.println("call " + function + "(m, n, array)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName(createFunc, 2));
     d_lw.println();
   }
@@ -310,12 +395,12 @@ public class ArrayModule {
       (d_implID.getFullName(), func, "_m");
     d_lw.println("subroutine " + getPrivateName(func, dim) +
                  "(src, dest)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(in) :: src");
     d_lw.println(getImplArrayType(dim) + ", intent(in) :: dest");
     d_lw.println("external " + function);
     d_lw.println("call " + function + "(src, dest)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName(func, dim));
     d_lw.println();
   }
@@ -327,14 +412,14 @@ public class ArrayModule {
       (d_implID.getFullName(), func, "_m");
     d_lw.println("subroutine " + getPrivateName(func, dim) +
                  "(src, dest)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(in) :: src");
     d_lw.println(getImplArrayType(dim) + ", intent(out) :: dest");
     d_lw.println("integer(selected_int_kind(9)) :: dim");
     d_lw.println("external " + function);
     d_lw.println("dim = " + dim);
     d_lw.println("call " + function + "(src, " + dim +", dest)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName(func, dim));
     d_lw.println();
   }
@@ -353,14 +438,14 @@ public class ArrayModule {
       (d_implID.getFullName(), "ensure", "_m");
     d_lw.println("subroutine " + getPrivateName("ensure", dim) +
                  "(src, dim, ordering, result)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(in)  :: src");
     d_lw.println(getImplArrayType(dim) + ", intent(out) :: result");
     d_lw.println(d_intType + ", intent(in) :: dim, ordering");
     d_lw.println("external " + function);
     d_lw.println("call " + function + 
                  "(src, " + dim + ", ordering, result)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName("ensure", dim));
     d_lw.println();
   }
@@ -373,7 +458,7 @@ public class ArrayModule {
         (d_implID.getFullName(), "slice", "_m");
       d_lw.println("subroutine " + getPrivateName("slice" + j, dim) +
                    "(src, numElem, srcStart, srcStride, newLower, result)");
-      d_lw.increaseTabLevel();
+      d_lw.tab();
       d_lw.println(getImplArrayType(dim) + ", intent(in)  :: src");
       d_lw.println(d_intType + ", dimension(" + dim + "), intent(in) :: numElem");
       d_lw.println
@@ -384,7 +469,7 @@ public class ArrayModule {
       d_lw.println("external " + function);
       d_lw.println("call " + function + 
                    "(src, " + j + ", numElem, srcStart, srcStride, newLower, result)");
-      d_lw.decreaseTabLevel();
+      d_lw.backTab();
       d_lw.println("end subroutine " + getPrivateName("slice" + j, dim));
       d_lw.println();
     }
@@ -407,17 +492,17 @@ public class ArrayModule {
   private void writeSliceInterface()
   {
     d_lw.println("private :: &");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     slicePrivateNames();
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println();
     d_lw.println("interface slice");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println("module procedure &");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     slicePrivateNames();
-    d_lw.decreaseTabLevel();
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
+    d_lw.backTab();
     d_lw.println("end interface");
     d_lw.println();
   }
@@ -429,16 +514,118 @@ public class ArrayModule {
       (d_implID.getFullName(), func, "_m");
     d_lw.println(returnType + " function  " + getPrivateName(func, dim) +
                  "(array)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(in)  :: array");
     d_lw.println("external " + function);
     d_lw.println("call " + function + 
                  "(array, " + getPrivateName(func, dim) + ")");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end function " + getPrivateName(func, dim));
     d_lw.println();
   }
 
+  private void generateCasts()
+    throws UnsupportedEncodingException
+  {
+    //Actually, this should never be called on a generic array at all.
+    //Double check
+    if(d_casttype >= BASETYPE) {
+      //Functions that cast TO a generic array
+      for(int dim = 1; dim <= s_maxArray; ++dim) {
+        generateCastToGeneric(d_type, dim); 
+      }      
+      d_lw.println();
+
+      //USER Defined class array cannot be cast to from a generic array.
+      if(d_casttype != USERDEFINED) {
+        //Functions that cast FROM a generic array
+        for(int dim = 1; dim <= s_maxArray; ++dim) {
+          generateCastFromGeneric(d_type, dim);
+        }
+        d_lw.println();
+      }
+    }
+  }
+
+  //Doesn't exist for non-base types! (except BaseInterface)
+  private void generateCastFromGeneric(int type, int dim)
+    throws UnsupportedEncodingException
+  {
+    String typeString = "sidl_"+BabelConfiguration.arrayType(d_type);
+    String funcname = d_mang.shortName("castGenericTo"+typeString+dim+"d",d_privateSuffix);
+    String oldTypeName = typeString+"_"+dim+"d";
+    String extfunc = d_mang.shortArrayName(typeString, "cast", "_m");  
+
+    d_lw.println("subroutine " + funcname + "(oldType, newType)");
+    d_lw.tab();    
+    //d_lw.println("use " + "sidl_"+BabelConfiguration.arrayType(type)+"_array");
+    d_lw.println("type(sidl__array), intent(in) :: oldType");
+    d_lw.println("type("+oldTypeName+"), intent(out) :: newType");
+    
+
+    //There is no BaseInterface array cast, so do that manually
+    if(d_casttype == BASEINTERFACE) {
+      d_lw.println("if(sidl__array_dimen_m(oldType%d_array) .ne. "+dim+" .or. sidl__array_type_m(oldType%d_array) .ne. "+type+") then");
+      d_lw.tab();
+      d_lw.println("newType%d_array=0");
+      d_lw.backTab();
+      d_lw.println("else");
+      d_lw.tab();
+      d_lw.println("newType%d_array = oldType%d_array");
+      d_lw.backTab();
+      d_lw.println("end if"); 
+    } else { //A Base type (int, bool, etc.) has a cast function, use it.
+      d_lw.println("external " + extfunc);
+      d_lw.println("call " + extfunc + "(oldType%d_array, "+dim+", newType)");
+    }
+
+    d_lw.backTab();
+    d_lw.println("end subroutine " + funcname);
+    d_lw.println();
+
+  }
+
+  private void generateCastToGeneric(int type, int dim)
+    throws UnsupportedEncodingException
+  {
+    String typeString = null;
+    if(d_casttype == USERDEFINED)  //Not a base type or sidl.BaseInterface
+      typeString = d_id.getFullName().replace('.','_');
+    else
+      typeString = "sidl_"+BabelConfiguration.arrayType(d_type);
+    
+    String funcname = d_mang.shortName("cast" + typeString + dim+"dToGeneric", d_privateSuffix);
+    String newTypeName = typeString+"_"+dim+"d";
+    //String extfunc = d_mang.shortArrayName("sidl", "cast", "_m");  
+
+    d_lw.println("subroutine " + funcname + "(oldType, newType)");
+    d_lw.tab();
+    d_lw.println("type(sidl__array), intent(out) :: newType");
+    d_lw.println("type("+newTypeName+"), intent(in) :: oldType");
+
+    //There is no BaseInterface array cast, so do that manually
+    //if(d_casttype == BASEINTERFACE) {
+    //  d_lw.println("if(dimen_p(oldType) .ne. +"dim"+ .or. type_p(oldType) .ne. "+type+") then");
+    //  d_lw.tab();
+    //  d_lw.println("set_null(newType)");
+    // d_lw.backTab();
+    // d_lw.println("else");
+    // d_lw.tab();
+
+    //Anything can get cast to a generic array.  That's no big deal.
+    d_lw.println("newType%d_array = oldType%d_array");
+    //  d_lw.backTab();
+    //d_lw.println("end if");
+    //} else {
+    //d_lw.println("external " + extfunc);
+    //d_lw.println("call " + extfunc + "(oldType%d_array, newType)");
+    // }
+    
+    d_lw.backTab();
+    d_lw.println("end subroutine " + funcname);  
+    d_lw.println();
+  }
+
   private void generateOrderingTests(int dim)
     throws UnsupportedEncodingException
   {
@@ -453,13 +640,13 @@ public class ArrayModule {
       (d_implID.getFullName(), func, "_m");
     d_lw.println(d_intType + " function  " + getPrivateName(func,dim) +
                  "(array, index)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(in)  :: array");
     d_lw.println(d_intType + ", intent(in) :: index");
     d_lw.println("external " + function);
     d_lw.println("call " + function + 
                  "(array, index, " + getPrivateName(func, dim) + ")");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end function " + getPrivateName(func, dim));
     d_lw.println();
   }
@@ -468,9 +655,11 @@ public class ArrayModule {
     throws UnsupportedEncodingException
   {
     generateQuery("dimen", d_intType, dim);
+    //generateQuery("type", d_intType, dim);
     generateIndexedQuery("stride", dim);
     generateIndexedQuery("lower", dim);
     generateIndexedQuery("upper", dim);
+    generateIndexedQuery("length", dim);
   }
 
   private void generateGenericGet(int dim)
@@ -480,14 +669,14 @@ public class ArrayModule {
       (d_implID.getFullName(), "get", "_m");
     d_lw.println("subroutine " + getPrivateName("getg", dim) +
                  "(array, index, value)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(in)  :: array");
     d_lw.println(d_intType + ", intent(in), dimension(" + dim + ") :: index");
     d_lw.println(d_implDataType + ", intent(out) :: value");
     d_lw.println("external " + function);
     d_lw.println("call " + function + 
                  "(array, index, value)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName("getg", dim));
     d_lw.println();
   }
@@ -501,13 +690,13 @@ public class ArrayModule {
       (d_implID.getFullName(), func + dim, "_m");
     d_lw.println("subroutine " + getPrivateName(func, dim) +
                  "(array, &");
-    d_lw.increaseTabLevel();
-    d_lw.increaseTabLevel();
+    d_lw.tab();
+    d_lw.tab();
     for(i = 1; i <= dim; ++i){
       d_lw.println("i" + i + ", &");
     }
     d_lw.println("value)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println(getImplArrayType(dim) + ", intent(in)  :: array");
     for(i = 1; i <= dim; ++i) {
       d_lw.println(d_intType + ", intent(in) :: i" + i);
@@ -515,13 +704,13 @@ public class ArrayModule {
     d_lw.println(d_implDataType + ", intent(out) :: value");
     d_lw.println("external " + function);
     d_lw.println("call " + function + "(array, &");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     for(i = 1; i <= dim; ++i) {
       d_lw.println("i" + i + ", &");
     }
     d_lw.println("value)");
-    d_lw.decreaseTabLevel();
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName(func,dim));
     d_lw.println();
   }
@@ -533,7 +722,7 @@ public class ArrayModule {
       (d_implID.getFullName(), "set", "_m");
     d_lw.println("subroutine " + getPrivateName("setg", dim) +
                  "(array, index, value)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(in)  :: array");
     d_lw.println(d_intType + ", intent(in), dimension(" + dim +
                  ") :: index");
@@ -541,7 +730,7 @@ public class ArrayModule {
     d_lw.println("external " + function);
     d_lw.println("call " + function + 
                  "(array, index, value)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName("setg",dim));
     d_lw.println();
   }
@@ -555,13 +744,13 @@ public class ArrayModule {
       (d_implID.getFullName(), func + dim, "_m");
     d_lw.println("subroutine " + getPrivateName(func, dim)+
                  "(array, &");
-    d_lw.increaseTabLevel();
-    d_lw.increaseTabLevel();
+    d_lw.tab();
+    d_lw.tab();
     for(i = 1; i <= dim; ++i){
       d_lw.println("i" + i + ", &");
     }
     d_lw.println("value)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println(getImplArrayType(dim) + ", intent(in)  :: array");
     for(i = 1; i <= dim; ++i) {
       d_lw.println(d_intType + ", intent(in) :: i" + i);
@@ -570,13 +759,13 @@ public class ArrayModule {
     d_lw.println("external " + function);
     d_lw.println("call " + function + 
                  "(array, &");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     for(i = 1; i <= dim; ++i) {
       d_lw.println("i" + i + ", &");
     }
     d_lw.println("value)");
-    d_lw.decreaseTabLevel();
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName(func, dim));
     d_lw.println();
   }
@@ -594,14 +783,18 @@ public class ArrayModule {
   {
     int i;
     d_lw.println("private :: &");
-    d_lw.increaseTabLevel();
-    d_lw.print(getPrivateName(method, 1));
-    for(i = 2; i <= s_maxArray; ++i ){
-      d_lw.println(", &");
-      d_lw.print(getPrivateName(method, i));
+    d_lw.tab();
+    if(d_is_generic_array) {
+      d_lw.print(getPrivateName(method, 0));
+    } else {
+      d_lw.print(getPrivateName(method, 1));
+      for(i = 2; i <= s_maxArray; ++i ){
+        d_lw.println(", &");
+        d_lw.print(getPrivateName(method, i));
+      }
     }
     d_lw.println();
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println();
   }
 
@@ -612,9 +805,9 @@ public class ArrayModule {
     writePrivate(generic);
     writePrivate(method);
     d_lw.println("interface " + method);
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println("module procedure &");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     for(i = 1; i <= s_maxArray; ++i) {
       d_lw.println(getPrivateName(generic, i) + ", &");
       d_lw.print(getPrivateName(method, i));
@@ -622,9 +815,9 @@ public class ArrayModule {
         d_lw.println(", &");
       }
     }
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println();
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end interface");
     d_lw.println();
   }
@@ -634,29 +827,119 @@ public class ArrayModule {
     d_lw.println("private :: " + getPrivateName(func, dim));
     d_lw.println();
     d_lw.println("interface " + func);
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println("module procedure " + getPrivateName(func, dim));
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end interface");
     d_lw.println();
   }
 
+  private void writeGenericCastInterfaces()
+    throws UnsupportedEncodingException
+  {
+   
+    String typeString = null;
+    if(d_casttype == USERDEFINED)  //Not a base type or sidl.BaseInterface
+      typeString = d_id.getFullName().replace('.','_');
+    else
+      typeString = "sidl_"+BabelConfiguration.arrayType(d_type);
+
+    d_lw.println();
+    d_lw.println("private :: &");
+    d_lw.tab(); 
+    //Functions that cast FROM a generic array 
+    for(int dim = 1; dim <= s_maxArray; ++dim) {
+      d_lw.print(d_mang.shortName("cast" + typeString +dim+"dToGeneric",
+                                  d_privateSuffix));
+      if(dim < s_maxArray) //So the last declaration doesn't end with a &
+        d_lw.println(", &"); 
+      else
+        d_lw.println();
+    }
+    
+    if(d_casttype != USERDEFINED) {
+      d_lw.println();
+      d_lw.println("private :: &");
+      d_lw.tab(); 
+      //Functions that cast TO a generic array
+      for(int dim = 1; dim <= s_maxArray; ++dim) {
+        d_lw.print(d_mang.shortName("castGenericTo" + typeString+dim+"d",
+                                    d_privateSuffix));
+        if(dim < s_maxArray) //So the last declaration doesn't end with a &
+          d_lw.println(", &"); 
+        else
+          d_lw.println();
+      }
+    }
+    d_lw.backTab();
+    d_lw.backTab();
+
+    //Interface declaration
+
+    d_lw.println("interface cast");
+    d_lw.tab();
+
+    d_lw.println("module procedure &");
+    d_lw.tab(); 
+    //Functions that cast FROM a generic array
+    for(int dim = 1; dim <= s_maxArray; ++dim) {
+      d_lw.print(d_mang.shortName("cast" + typeString+dim+"dToGeneric",
+                                  d_privateSuffix));
+      if(dim < s_maxArray) //So the last declaration doesn't end with a &
+        d_lw.println(", &"); 
+      else
+        d_lw.println(); 
+    }
+
+    
+    if(d_casttype != USERDEFINED) {
+      d_lw.backTab();
+      d_lw.println("module procedure &");
+      d_lw.tab();
+      //Functions that cast TO a generic array
+      for(int dim = 1; dim <= s_maxArray; ++dim) {
+        d_lw.print(d_mang.shortName("castGenericTo" + typeString+dim+"d",
+                                    d_privateSuffix));
+        if(dim < s_maxArray) //So the last declaration doesn't end with a &
+          d_lw.println(", &"); 
+        else
+          d_lw.println();
+      }
+    }
+
+    d_lw.backTab();
+    d_lw.backTab();
+    d_lw.println("end interface");
+    d_lw.println();
+  }
+
+  private void generateUseStatements() {
+    for(int type = 1; type <= BabelConfiguration.sidl_interface_array; ++type) {
+      d_lw.println("use sidl_" + BabelConfiguration.arrayType(type) + "_array"); 
+    }
+  }
+
   private void writeGenericInterface(String func)
   {
     int i;
     writePrivate(func);
     d_lw.println("interface " + func);
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println("module procedure &");
-    d_lw.increaseTabLevel();
-    d_lw.print(getPrivateName(func, 1));
-    for(i = 2; i <= s_maxArray; ++i) {
-      d_lw.println(", &");
-      d_lw.print(getPrivateName(func, i));
+    d_lw.tab();
+ 
+    if(d_is_generic_array) {
+      d_lw.print(getPrivateName(func, 0));
+    } else {
+      d_lw.print(getPrivateName(func, 1));
+      for(i = 2; i <= s_maxArray; ++i) {
+        d_lw.println(", &");
+        d_lw.print(getPrivateName(func, i));
+      }
     }
     d_lw.println();
-    d_lw.decreaseTabLevel();
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
+    d_lw.backTab();
     d_lw.println("end interface");
     d_lw.println();
   }
@@ -668,7 +951,7 @@ public class ArrayModule {
       (d_implID.getFullName(), "access", "_m");
     d_lw.println("subroutine " + getPrivateName("access", dim) +
                  "(array, ref, low, up, str, index)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println("implicit none");
     d_lw.println(getImplArrayType(dim) + ", intent(in) :: array");
     d_lw.println(d_implDataType + ", dimension(1), intent(in) :: ref");
@@ -677,7 +960,7 @@ public class ArrayModule {
     d_lw.println("external " + function);
     d_lw.println("call " + function +
                  "(array, ref, low, up, str, index)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName("access", dim));
   }
 
@@ -689,17 +972,18 @@ public class ArrayModule {
       (d_implID.getFullName(), func, "_m");
     d_lw.println("subroutine " + getPrivateName(func, dim) +
                  "(first, low, up, astr, result)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(d_implDataType + ", intent(in) :: first");
     d_lw.println(d_intType + ", dimension(" + dim +"), intent(in) :: low, up, astr");
     d_lw.println(getImplArrayType(dim) + ", intent(out) :: result");
     d_lw.println("external " + function);
     d_lw.println("call " + function + "(first, " + dim + ", low, up, astr, result)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName(func, dim));
     d_lw.println();
   }
 
+
   private void generateDeclarations()
     throws UnsupportedEncodingException
   {
@@ -708,21 +992,31 @@ public class ArrayModule {
       TypeModule.writeArrayType(d_lw, d_id, 
                                 (d_access ? d_implDataType : null));
     }
-    for(i = 0; i < s_constant_methods.length ; ++i){
-      writeGenericInterface(s_constant_methods[i]);
-    }
-    writeSingleInterface("create1d",1);
-    writeSingleInterface("create2dRow", 2);
-    writeSingleInterface("create2dCol", 2);
+    if(d_is_generic_array) {  //The methods available in generic arrays.
+      for(i = 0; i < s_generic_methods.length ; ++i){
+        writeGenericInterface(s_generic_methods[i]);
+      }
+ 
+    }else {
+
+      for(i = 0; i < s_constant_methods.length ; ++i){
+        writeGenericInterface(s_constant_methods[i]);
+      }      
+      writeSingleInterface("create1d",1);
+      writeSingleInterface("create2dRow", 2);
+      writeSingleInterface("create2dCol", 2);
+      
+      writeSliceInterface();
+      writeInterface("get");
+      writeInterface("set");
     
-    writeSliceInterface();
-    writeInterface("get");
-    writeInterface("set");
-    if (d_access) {
-      writeGenericInterface("access");
-    }
-    if (d_borrow) {
-      writeGenericInterface("borrow");
+      writeGenericCastInterfaces();
+      if (d_access) {
+        writeGenericInterface("access");
+      }
+      if (d_borrow) {
+        writeGenericInterface("borrow");
+      }
     }
   }
 
@@ -733,12 +1027,12 @@ public class ArrayModule {
       (d_implID.getFullName(), func, "_m");
     d_lw.println("subroutine  " + getPrivateName(func, dim) +
                  "(array)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println(getImplArrayType(dim) + ", intent(in)  :: array");
     d_lw.println("external " + function);
     d_lw.println("call " + function + 
                  "(array)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine " + getPrivateName(func, dim));
     d_lw.println();
   }
@@ -750,55 +1044,83 @@ public class ArrayModule {
     generateRef("deleteRef", dim);
   }
 
+  private void generateGenericArrayTypes() {
+    
+    d_lw.println();
+    d_lw.println("type sidl__array");
+    d_lw.tab();
+    d_lw.println("sequence");
+    d_lw.println("integer (selected_int_kind(18)) :: d_array");
+    d_lw.backTab();
+    d_lw.println("end type sidl__array");
+    d_lw.println();
+
+  }
+
   private void generateImplementations()
     throws UnsupportedEncodingException
   {
-    for(int i = 1; i <= s_maxArray; ++i) {
-      generateCreates(i );
-      generateCopies(i);
-      generateEnsure(i);
-      generateSlice(i);
-      generateOrderingTests(i);
-      generateSizeQueries(i);
-      generateGetSet(i);
-      generateRefCounting(i);
-      generateNullRelated(i);
-      if (d_access) {
-        generateAccess(i);
-      }
-      if (d_borrow) {
-        generateBorrow(i);
+    if(d_is_generic_array) {
+      generateSmartCopy("smartCopy", 0);
+      generateOrderingTests(0);
+      generateSizeQueries(0);
+      generateQuery("type", d_intType, 0);
+      generateRefCounting(0);
+      generateNullRelated(0);
+
+    } else {
+      for(int i = 1; i <= s_maxArray; ++i) {
+        generateCreates(i );
+        generateCopy("copy", i);
+        generateEnsure(i);
+        generateSlice(i);
+        generateGetSet(i);
+
+        generateSmartCopy("smartCopy", i);
+        generateOrderingTests(i);
+        generateSizeQueries(i);
+        generateRefCounting(i);
+        generateNullRelated(i);
+        if (d_access) {
+          generateAccess(i);
+        }
+        if (d_borrow) {
+          generateBorrow(i);
+        }
       }
+      generateCasts();  //have their own interfal for loop
     }
   }
-
+  
   public void generateStub()
     throws CodeGenerationException
   {
     try {
-      if (!d_is_base_type) {
-        d_lw.printlnUnformatted("#include \"" +
-                                Fortran.getStubNameFile(d_id) +
-                                "\"");
+      d_lw.println();
+      if (d_is_generic_array) {
+        d_lw.printlnUnformatted("#include \"sidl_array_fAbbrev.h\"");      
       }
       d_lw.println("module " + Fortran.getArrayModule(d_id));
-      d_lw.increaseTabLevel();
-      if (!d_is_base_type) {
-        d_lw.println("use " + Fortran.getTypeModule(d_id));
+      d_lw.tab();
+      if (!d_is_generic_array) { 
+        if (!d_is_base_type) {
+          d_lw.println("use " + Fortran.getTypeModule(d_id));
+        }
       }
+      d_lw.println("use sidl_array_type"); //Everyone needs the generic module!
       d_lw.println();
       
       generateDeclarations();
 
       d_lw.println();
-      d_lw.decreaseTabLevel();
+      d_lw.backTab();
       d_lw.println("contains");
-      d_lw.increaseTabLevel();
+      d_lw.tab();
       d_lw.println();
       d_lw.println();
 
       generateImplementations();
-      d_lw.decreaseTabLevel();
+      d_lw.backTab();
       d_lw.println("end module " +  Fortran.getArrayModule(d_id));
     }
     catch (UnsupportedEncodingException uee) {
