@@ -1,8 +1,7 @@
 //
 // File:        ArrayMethods.java
 // Package:     gov.llnl.babel.backend.fortran
-// Release:     $Name: release-0-8-8 $
-// Revision:    @(#) $Revision: 1.5 $
+// Revision:    @(#) $Revision: 4434 $
 // Description: 
 //
 // Copyright (c) 2000-2002, The Regents of the University of Calfornia.
@@ -38,8 +37,6 @@ import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.BabelConfiguration;
 import gov.llnl.babel.symbols.Version;
 import gov.llnl.babel.backend.writers.LanguageWriter;
-import java.util.List;
-import java.util.ArrayList;
 
 public class FortArrayMethods {
   static private final int s_maxArray = BabelConfiguration.getMaximumArray();
@@ -59,16 +56,16 @@ public class FortArrayMethods {
     d_id = id;
     d_isEnum = isEnum;
     d_implArrayType = isEnum 
-      ? "struct SIDL_int__array *" 
-      : "struct SIDL_interface__array *";
+      ? "struct sidl_int__array *" 
+      : "struct sidl_interface__array *";
     d_implDataType = isEnum 
       ? "(int32_t)" 
-      : "(struct SIDL_BaseInterface__object *)";
+      : "(struct sidl_BaseInterface__object *)";
     if (isEnum) {
-      d_implID = new SymbolID("SIDL.int", new Version());
+      d_implID = new SymbolID("sidl.int", new Version());
     }
     else {
-      d_implID = new SymbolID("SIDL.interface", new Version());
+      d_implID = new SymbolID("sidl.interface", new Version());
     }
   }
 
@@ -100,12 +97,12 @@ public class FortArrayMethods {
     String methodName = generateCreateName(d_id, shortType);
     lw.println("void");
     writeMethodName(lw,methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int32_t *dimen,");
     lw.println(" int32_t lower[],");
     lw.println(" int32_t upper[],");
     lw.println(" int64_t *result)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateCreate(LanguageWriter lw,
@@ -114,13 +111,13 @@ public class FortArrayMethods {
   {
     generateCreateSig(lw, shortType, longType);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("*result = (ptrdiff_t)");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateCreateName(d_implID, shortType) +
                "(*dimen, lower, upper);");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -135,19 +132,19 @@ public class FortArrayMethods {
     String methodName = generateOneDName(d_id);
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int32_t *len, int64_t *result)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
   
   private void generateOneD(LanguageWriter lw)
   {
     generateOneDSig(lw);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("*result = (ptrdiff_t)" +
                ArrayMethods.generateOneDName(d_implID) + "(*len);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -164,9 +161,9 @@ public class FortArrayMethods {
     String methodName = generateTwoDName(d_id, shortType);
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int32_t *m, int32_t *n, int64_t *result)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateTwoD(LanguageWriter lw,
@@ -175,11 +172,11 @@ public class FortArrayMethods {
   {
     generateTwoDSig(lw, shortType, longType);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("*result = (ptrdiff_t)" +
                ArrayMethods.generateTwoDName(d_implID, shortType) + 
                "(*m, *n);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -193,19 +190,19 @@ public class FortArrayMethods {
   {
     lw.println("void");
     writeMethodName(lw,generateDelRefName(d_id));
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *array)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateDelRef(LanguageWriter lw)
   {
     generateDelRefSig(lw);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateDelRefName(d_implID) + "((" +
                d_implArrayType + ")(ptrdiff_t)*array);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -218,19 +215,19 @@ public class FortArrayMethods {
   {
     lw.println("void");
     writeMethodName(lw, generateAddRefName(d_id));
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *array)" );
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateAddRef(LanguageWriter lw)
   {
     generateAddRefSig(lw);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateAddRefName(d_implID) + "((" +
                d_implArrayType + ")(ptrdiff_t)*array);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -252,13 +249,13 @@ public class FortArrayMethods {
     String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *array, ");
     for(int i = 1; i <= num; ++i){
       lw.println(" int32_t *i" + i + ", ");
     }
     lw.println(" int64_t *result)" );
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateGet(LanguageWriter lw,
@@ -266,17 +263,17 @@ public class FortArrayMethods {
   {
     generateGetSig(lw, num);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("*result = (ptrdiff_t)");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateGetName(d_implID, num) + "((const " +
                d_implArrayType + ")(ptrdiff_t)*array");
     for(int i = 1 ; i <= num; ++i){
       lw.print(", *i" + i);
     }
     lw.println(");");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -286,24 +283,24 @@ public class FortArrayMethods {
     String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *array,");
     lw.println(" int32_t indices[],");
     lw.println(" int64_t *result)" );
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateGet(LanguageWriter lw)
   {
     generateGetSig(lw);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("*result = (ptrdiff_t)");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateGetName(d_implID) + "((const " +
                d_implArrayType + ")(ptrdiff_t)*array, indices);");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -325,13 +322,13 @@ public class FortArrayMethods {
     String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *array,");
     for(int i = 1; i <= num; ++i){
       lw.println(" int32_t *i" + i + ",");
     }
     lw.println(" int64_t *value)" );
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateSet(LanguageWriter lw,
@@ -339,14 +336,14 @@ public class FortArrayMethods {
   {
     generateSetSig(lw, num);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateSetName(d_implID, num) + "((" +
                d_implArrayType + ")(ptrdiff_t)*array");
     for(int i = 1 ; i <= num; ++i){
       lw.print(", *i" + i);
     }
     lw.println(", " + d_implDataType + "(ptrdiff_t)*value);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -356,22 +353,22 @@ public class FortArrayMethods {
     String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *array,");
     lw.println("int32_t indices[],");
     lw.println("int64_t *value)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateSet(LanguageWriter lw)
   {
     generateSetSig(lw);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateSetName(d_implID) + "((" +
                d_implArrayType + ")(ptrdiff_t)*array, indices, " +
                d_implDataType + "(ptrdiff_t)*value);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -386,22 +383,22 @@ public class FortArrayMethods {
     String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *array, int32_t *result)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateDimen(LanguageWriter lw)
   {
     generateDimenSig(lw);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("*result =");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateDimenName(d_implID) + "((" +
                d_implArrayType + ")(ptrdiff_t)*array);");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -421,11 +418,11 @@ public class FortArrayMethods {
     String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *array,");
     lw.println(" int32_t *ind,");
     lw.println(" int32_t *result)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateBound(LanguageWriter lw,
@@ -435,13 +432,13 @@ public class FortArrayMethods {
   {
     generateBoundSig(lw, direction, desc, bad);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("*result = ");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateBoundName(d_implID, direction) +
                "((" + d_implArrayType + ")(ptrdiff_t)*array, *ind);");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -460,10 +457,10 @@ public class FortArrayMethods {
     String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *array,");
     lw.println(" SIDL_F" + Fortran.getFortranVersion() + "_Bool *result)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateOrder(LanguageWriter lw,
@@ -471,11 +468,11 @@ public class FortArrayMethods {
   {
     generateOrderSig(lw, order);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("*result = " + 
                ArrayMethods.generateOrderName(d_implID, order) +
                "((" + d_implArrayType + ")(ptrdiff_t)*array);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -490,22 +487,22 @@ public class FortArrayMethods {
     String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *src,");
     lw.println(" int64_t *dest)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateCopy(LanguageWriter lw)
   {
     generateCopySig(lw);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateCopyName(d_implID) +
                "((const " + d_implArrayType + ")(ptrdiff_t)*src,");
     lw.printSpaces(ArrayMethods.generateCopyName(d_implID).length() + 1);
     lw.println("(" + d_implArrayType + ")(ptrdiff_t)*dest);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -520,7 +517,7 @@ public class FortArrayMethods {
     String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *src,");
     lw.println(" int32_t *dimen,");
     lw.println(" int32_t numElem[],");
@@ -528,23 +525,23 @@ public class FortArrayMethods {
     lw.println(" int32_t srcStride[],");
     lw.println(" int32_t newStart[],");
     lw.println(" int64_t *result)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateSlice(LanguageWriter lw)
   {
     generateSliceSig(lw);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("*result = (ptrdiff_t)");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateSliceName(d_implID) +
                "((" + d_implArrayType + ")(ptrdiff_t)*src,");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("*dimen, numElem, srcStart, srcStride, newStart);");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.backTab();
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -560,19 +557,19 @@ public class FortArrayMethods {
     String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *src)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateSmartCopy(LanguageWriter lw)
   {
     generateSmartCopySig(lw);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateSmartCopyName(d_implID) +
                "((" + d_implArrayType + ")(ptrdiff_t)*src);");
-    lw.decreaseTabLevel();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -587,26 +584,26 @@ public class FortArrayMethods {
     String arrayName = IOR.getArrayName(d_id) + "*";
     lw.println("void");
     writeMethodName(lw, methodName);
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("(int64_t *src,");
     lw.println(" int32_t *dimen,");
     lw.println(" int     *ordering,");
     lw.println(" int64_t *result)");
-    lw.decreaseTabLevel();
+    lw.backTab();
   }
 
   private void generateEnsure(LanguageWriter lw)
   {
     generateEnsureSig(lw);
     lw.println("{");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println("*result = (ptrdiff_t)");
-    lw.increaseTabLevel();
+    lw.tab();
     lw.println(ArrayMethods.generateEnsureName(d_implID) +
                "((" + d_implArrayType + ")(ptrdiff_t)*src,");
     lw.println("*dimen, *ordering);");
-    lw.decreaseTabLevel();
-    lw.decreaseTabLevel();
+    lw.backTab();
+    lw.backTab();
     lw.println("}");
   }
 
@@ -659,6 +656,9 @@ public class FortArrayMethods {
     generateBound(lw, "upper", "upper bound", "-1");
     lw.println();
 
+    generateBound(lw, "length", "length", "-1");
+    lw.println();
+
     generateBound(lw, "stride", "stride", "0");
     lw.println();
 
