@@ -1,8 +1,7 @@
 //
 // File:        Fortran.java
 // Package:     gov.llnl.babel.backend.fortran
-// Release:     $Name: release-0-8-8 $
-// Revision:    @(#) $Revision: 1.14 $
+// Revision:    @(#) $Revision: 4434 $
 // Description: Collection of static methods for the FORTRAN binding
 //
 // Copyright (c) 2000-2002, The Regents of the University of Calfornia.
@@ -35,14 +34,21 @@ package gov.llnl.babel.backend.fortran;
 import gov.llnl.babel.BabelConfiguration;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
+import gov.llnl.babel.backend.IOR;
 import gov.llnl.babel.backend.mangler.NameMangler;
+import gov.llnl.babel.symbols.Argument;
+import gov.llnl.babel.symbols.Comment;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
 import gov.llnl.babel.symbols.Version;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Iterator;
+import java.util.List;
 
 /**
- * Provide a collection of static methods to provide the mapping of SIDL
+ * Provide a collection of static methods to provide the mapping of sidl
  * concepts into FORTRAN.  This class provides the mapping of symbol names
  * to FORTRAN symbols, the mapping of the wrapper code to a set of files,
  * and the mapping of types.
@@ -73,8 +79,7 @@ public class Fortran implements CodeConstants
     "integer*8",                // class
     "integer*8",                // interface
     "void",                     // package
-    "integer*8",                // symbol
-    "integer*8"                 // array
+    "integer*8"                 // symbol
   };
 
   private final static String s_f90_types[] = {
@@ -94,7 +99,7 @@ public class Fortran implements CodeConstants
   
   /**
    * This array stores the name of the C type that corresponds to the FORTRAN
-   * type for a particular SIDL type.  For example, an opaque maps into
+   * type for a particular sidl type.  For example, an opaque maps into
    * <code>integer*8</code> in FORTRAN.  When the <code>integer*8</code>
    * comes into C, it is a <code>int64_t</code>.
    */
@@ -102,9 +107,9 @@ public class Fortran implements CodeConstants
     "void",                     // void
     "SIDL_F77_Bool",            // bool
     "char",                     // character
-    "struct SIDL_dcomplex",     // dcomplex
+    "struct sidl_dcomplex",     // dcomplex
     "double",                   // double
-    "struct SIDL_fcomplex",     // fcomplex
+    "struct sidl_fcomplex",     // fcomplex
     "float",                    // float
     "int32_t",                  // int
     "int64_t",                  // long
@@ -119,14 +124,7 @@ public class Fortran implements CodeConstants
   };
   private final static int    s_Fortran_Bool = 1;
   private final static String s_F90_Bool = "SIDL_F90_Bool";
-  private final static String s_F90_Array = "struct SIDL_fortran_array";
-
-  /**
-   * Indicator of the version of FORTRAN being generated.  True means
-   * F90; otherwise, F77 assumed.
-   */
-  private final static boolean s_isF90 = 
-      BabelConfiguration.getInstance().getTargetLanguage().equals("f90");
+  private final static String s_F90_Array = "struct sidl_fortran_array";
 
 
   /**
@@ -146,14 +144,39 @@ public class Fortran implements CodeConstants
    * with underline characters.
    */
   public static String getSymbolName(SymbolID id) {
+      return id.getFullName().replace('.', '_');
+  }
+
+  /**
+   * Convert a symbol name into string with the pieces of the symbol joined
+   * together with underline characters. Generate short file names 
+   * if the options for generating code in package-dependent subdirectories
+   * and excluding external symbols are enabled (--generate-subdirs and
+   * --exclude-external).
+   *
+   * @param id  the symbol id to convert.
+   * @return a string representation of the symbol with periods replaced
+   * with underline characters.
+   */
+  public static String getSymbolNameForFile(SymbolID id) {
     BabelConfiguration s_babel_config = BabelConfiguration.getInstance();
-    if (s_babel_config.makePackageSubdirs() && s_babel_config.excludeExternal()) {
+		return id.getFullName().replace('.', '_');
+  }
+	
+	/** 
+	 * Same as above method, but use an extra argument for determining 
+	 * when the file whose name is to be generated corresponds to an impl.
+	 */
+  public static String getSymbolNameForFile(SymbolID id, boolean isImpl) {
+    BabelConfiguration s_babel_config = BabelConfiguration.getInstance();
+    if (s_babel_config.makePackageSubdirs() && s_babel_config.getShortFileNames()) {
       return id.getShortName().replace('.','_');
     } else {
       return id.getFullName().replace('.', '_');
     }
   }
 
+
   /**
    * Return the name of the stub file for a particular symbol.
    * The stub file is a C file that receives calls from FORTRAN client.
@@ -163,11 +186,11 @@ public class Fortran implements CodeConstants
    *         the FORTRAN stubs.
    */
   public static String getStubFile(SymbolID id) {
-    return getSymbolName(id) + "_fStub.c";
+    return getSymbolNameForFile(id) + "_fStub.c";
   }
 
   public static String getStubNameFile(SymbolID id) {
-    return getSymbolName(id) + "_fAbbrev.h";
+    return getSymbolNameForFile(id) + "_fAbbrev.h";
   }
 
   /**
@@ -180,7 +203,7 @@ public class Fortran implements CodeConstants
    *         of the FORTRAN stubs.
    */
   public static String getStubDocFile(SymbolID id) {
-    return getSymbolName(id) + ".fif";
+    return getSymbolNameForFile(id) + ".fif";
   }
 
   /**
@@ -194,7 +217,7 @@ public class Fortran implements CodeConstants
    *         of the FORTRAN stubs.
    */
   public static String getModuleFile(SymbolID id) {
-    return getSymbolName(id) + "." + CodeConstants.C_F90_IMPL_EXTENSION;
+    return getSymbolNameForFile(id) + "." + CodeConstants.C_F90_IMPL_EXTENSION;
   }
 
   /**
@@ -207,7 +230,7 @@ public class Fortran implements CodeConstants
    *         of the FORTRAN stubs.
    */
   public static String getTypeFile(SymbolID id) {
-    return getSymbolName(id) + "_type." + CodeConstants.C_F90_IMPL_EXTENSION;
+    return getSymbolNameForFile(id) + "_type." + CodeConstants.C_F90_IMPL_EXTENSION;
   }
 
   public static String getTypeModule(SymbolID id) {
@@ -219,7 +242,7 @@ public class Fortran implements CodeConstants
   }
 
   public static String getArrayFile(SymbolID id) {
-    return getSymbolName(id) + "_array." + CodeConstants.C_F90_IMPL_EXTENSION;
+    return getSymbolNameForFile(id) + "_array." + CodeConstants.C_F90_IMPL_EXTENSION;
   }
 
   public static String getTypeName(SymbolID id) {
@@ -227,6 +250,9 @@ public class Fortran implements CodeConstants
   }
 
   public static String getArrayName(SymbolID id, int dim) {
+    if(dim == 0) {//If dim == 0, we'll assume it's a generic array
+      return "sidl__array";
+    }
     return getSymbolName(id) + "_" + dim + "d";
   }
 
@@ -234,17 +260,64 @@ public class Fortran implements CodeConstants
     return getSymbolName(id);
   }
 
+  /**
+   * Reorder an argument list to guarantee that all raw arrays occur at the
+   * end of the argument list. The relative position of non-raw arrays
+   * arguments is unchanged by this function.
+   *
+   * @param args  the incoming list of arguments
+   * @return  the incoming list has been reordered such that all raw arrays
+   *          appear at the end.
+   */
+  public static List reorderArguments(List args)
+  {
+    ArrayList result = new ArrayList(args.size());
+    ArrayList deferred = new ArrayList(args.size());
+    Iterator i = args.iterator();
+    while (i.hasNext()) {
+      Argument a = (Argument)i.next();
+      if (a.getType().isRarray()) {
+        deferred.add(a);
+      }
+      else {
+        result.add(a);
+      }
+    }
+    result.addAll(deferred);
+    return result;
+  }
+
+  static public String arrayIndices(Collection indices)
+  {
+    StringBuffer result = new StringBuffer();
+    Iterator i = indices.iterator();
+    result.append('(');
+    while (i.hasNext()) {
+      result.append("0:");
+      result.append(i.next().toString());
+      result.append("-1");
+      if (i.hasNext()) {
+        result.append(", ");
+      }
+    }
+    result.append(')');
+    return result.toString();
+  }
+
   public static boolean hasDirectAccess(Type t)
   {
     if (t.getDetailedType() == Type.ARRAY) {
-      switch(t.getArrayType().getDetailedType()) {
-      case Type.DCOMPLEX:
-      case Type.DOUBLE:
-      case Type.FCOMPLEX:
-      case Type.FLOAT:
-      case Type.INT:
-      case Type.LONG:
-        return true;
+      final Type arrayType = t.getArrayType();
+      if (null != arrayType) {
+        switch(arrayType.getDetailedType()) {
+        case Type.DCOMPLEX:
+        case Type.DOUBLE:
+        case Type.FCOMPLEX:
+        case Type.FLOAT:
+        case Type.INT:
+        case Type.LONG:
+          return true;
+        }
       }
     }
     return false;
@@ -260,33 +333,27 @@ public class Fortran implements CodeConstants
    *         the FORTRAN stubs.
    */
   public static String getEnumStubFile(SymbolID id) {
-    return getSymbolName(id) + ".inc";
+    return getSymbolNameForFile(id) + ".inc";
   }
 
   public static String getEnsureArray(Type arrayType) {
     switch(arrayType.getDetailedType()) {
     case Type.INTERFACE:
     case Type.CLASS:
-      return "SIDL_interface__array_ensure((struct SIDL_interface__array *)";
+      return "sidl_interface__array_ensure((struct sidl_interface__array *)";
     case Type.ENUM:
-      return "SIDL_int__array_ensure((struct SIDL_int__array *)";
+      return "sidl_int__array_ensure((struct sidl_int__array *)";
     default:
-      return "SIDL_" + arrayType.getTypeString() + "__array_ensure(";
+      return "sidl_" + arrayType.getTypeString() + "__array_ensure(";
     }
   }
 
+  public static String getInitArray(Type arrayType) {
+    return "sidl_" + arrayType.getTypeString() + "__array_init(";
+  }
+
   public static String getDelRefArray(Type arrayType) {
-    switch(arrayType.getDetailedType()) {
-    case Type.INTERFACE:
-    case Type.CLASS:
-      return "SIDL_interface__array_deleteRef((struct SIDL_interface__array *)";
-    case Type.ENUM:
-      return "SIDL_int__array_deleteRef((struct SIDL_int__array *)";
-    default:
-      return "SIDL_" + arrayType.getTypeString() +
-        "__array_deleteRef((struct SIDL_" +
-        arrayType.getTypeString() + "__array *)";
-    }
+    return "sidl__array_deleteRef((struct sidl__array *)";
   }
 
   /**
@@ -299,7 +366,7 @@ public class Fortran implements CodeConstants
    *         the FORTRAN enum array stubs.
    */
   public static String getEnumStubImpl(SymbolID id) {
-    return getSymbolName(id) + "_fStub.c";
+    return getSymbolNameForFile(id) + "_fStub.c";
   }
 
   /**
@@ -308,7 +375,7 @@ public class Fortran implements CodeConstants
    * the glue between the IOR and a FORTRAN.
    */
   public static String getSkelFile(SymbolID id) {
-    return getSymbolName(id) + "_fSkel.c";
+    return getSymbolNameForFile(id) + "_fSkel.c";
   }
 
   /**
@@ -317,7 +384,7 @@ public class Fortran implements CodeConstants
    * @return the FORTRAN file extension.
    */
   public static String getImplExtension() {
-    if (s_isF90) {
+    if (isFortran90()) {
       return CodeConstants.C_F90_IMPL_EXTENSION;
     } else {
       return CodeConstants.C_F77_IMPL_EXTENSION;
@@ -326,30 +393,30 @@ public class Fortran implements CodeConstants
 
   /**
    * Return the name of the file that hold the implementation of the
-   * FORTRAN SIDL object. The BABEL system generates the subroutine
+   * FORTRAN sidl object. The BABEL system generates the subroutine
    * declarations, but the user needs to fill in the subroutine bodies.
    *
    * @param id the name of the symbol
-   * @return the filename for the SIDL object implementation.
+   * @return the filename for the sidl object implementation.
    */
   public static String getImplFile(SymbolID id) {
-    return getSymbolName(id) + "_Impl." + getImplExtension();
+    return getSymbolNameForFile(id, true) + "_Impl." + getImplExtension();
   }
 
   /**
    * Return the name of the file that hold the derived types for
-   * implementation of the FORTRAN SIDL object.
+   * implementation of the FORTRAN sidl object.
    *
    * @param id the name of the symbol
-   * @return the filename for the SIDL impl module file.
+   * @return the filename for the sidl impl module file.
    */
   public static String getImplModuleFile(SymbolID id) {
-    return getSymbolName(id) + "_Mod." + getImplExtension();
+    return getSymbolNameForFile(id, true) + "_Mod." + getImplExtension();
   }
 
   /**
    * Get the function name that should be used for the FORTRAN stubs to the
-   * SIDL object methods.  This method does not take into account the
+   * sidl object methods.  This method does not take into account the
    * compiler specific issues (i.e. whether the symbol should be all upper
    * or lower case or whether it has underscores appended); it provides
    * the starting name with potentially mixed case.
@@ -363,6 +430,39 @@ public class Fortran implements CodeConstants
            + getMethodSuffix();
   }
 
+  /**
+   * Get the function name that should be used for the FORTRAN stubs to the
+   * sidl object methods.  This method does not take into account the
+   * compiler specific issues (i.e. whether the symbol should be all upper
+   * or lower case or whether it has underscores appended); it provides
+   * the starting name with potentially mixed case.
+   *
+   * @param  id     the name of the symbol who has the method.
+   * @param  method information about the method to be named.
+   * @return the name of the function to be used in the FORTRAN stub.
+   */
+  public static String getAltStubName(SymbolID id, Method method) {
+    return getSymbolName(id) + "_" + method.getLongMethodName() 
+           + getAltSuffix();
+  }
+
+  /**
+   * Get the function name that should be used for the FORTRAN super method
+   * in the skels (availible in this Impls) This method does not take into account the
+   * compiler specific issues (i.e. whether the symbol should be all upper
+   * or lower case or whether it has underscores appended); it provides
+   * the starting name with potentially mixed case.
+   *
+   * @param  id     the name of the symbol who has the method.
+   * @param  method information about the method to be named.
+   * @return the name of the function to be used in the FORTRAN stub.
+   */
+  public static String getMethodSuperName(SymbolID id, Method method) {
+    return getSymbolName(id) + "_" + "super_" + method.getLongMethodName() 
+           + getMethodSuffix();
+  }
+
+
   /**
    * Return the appropriate version of Fortran (i.e., 77 or 90).
    *
@@ -370,7 +470,7 @@ public class Fortran implements CodeConstants
    */
   public static int getFortranVersion() {
     int fversion;
-    if (s_isF90) {
+    if (isFortran90()) {
       fversion = CodeConstants.C_F90_VERSION;
     } else {
       fversion = CodeConstants.C_F77_VERSION;
@@ -384,20 +484,29 @@ public class Fortran implements CodeConstants
    * @return the FORTRAN method name suffix.
    */
   public static String getMethodSuffix() {
-    if (s_isF90) {
+    if (isFortran90()) {
       return CodeConstants.C_F90_METHOD_SUFFIX;
     } else {
       return CodeConstants.C_F77_METHOD_SUFFIX;
     }
   }
 
+  /**
+   * Return the appropriate FORTRAN alternative stub name.
+   *
+   * @return the FORTRAN method name suffix.
+   */
+  public static String getAltSuffix() {
+    return CodeConstants.C_F90_ALT_SUFFIX;
+  }
+
   /**
    * Return the appropriate FORTRAN impl method name suffix based.
    *
    * @return the FORTRAN impl method name suffix.
    */
   public static String getImplMethodSuffix() {
-    if (s_isF90) {
+    if (isFortran90()) {
       return CodeConstants.C_F90_IMPL_METHOD_SUFFIX;
     } else {
       return CodeConstants.C_F77_IMPL_METHOD_SUFFIX;
@@ -478,6 +587,13 @@ public class Fortran implements CodeConstants
     return getSymbolName(id) + "__array_upper" + getMethodSuffix();
   }
 
+  /**
+   * Generate the name of the array length access function.
+   */
+  public static String getArrayLength(SymbolID id) {
+    return getSymbolName(id) + "__array_length" + getMethodSuffix();
+  }
+
   /**
    * Return the function name that should be used for the FORTRAN subroutine
    * that implements a particular method.  The returned string leaves
@@ -507,6 +623,35 @@ public class Fortran implements CodeConstants
     }
   }
 
+  /**
+   * Return the function name that should be used for the FORTRAN subroutine
+   * that implements a particular SUPER method.  The returned string leaves
+   * the cast of the components of the name unchanged.  If <code>id</code>
+   * and the method name have mixed case, this method returns a mixed case
+   * string.
+   *
+   * @param  id     the name of the symbol who has the method.
+   * @param  method the information about the method.
+   * @param  mang   the name mangler being used to address long names.
+   * @return the name of the FORTRAN subroutine that implements this method.
+   * @exception gov.llnl.babel.backend.CodeGenerationException
+   *    the name mangler is broken.
+   */
+  public static String getMethodSuperImplName(SymbolID id,
+                                         Method method,
+                                         NameMangler mang)
+    throws CodeGenerationException
+  {
+    try {
+      return mang.shortName(id.getFullName(), 
+                            "super_"+method.getLongMethodName(),
+                            getImplMethodSuffix());
+    }
+    catch (java.io.UnsupportedEncodingException uee) {
+      throw new CodeGenerationException(uee.getMessage());
+    }
+  }
+
   /**
    * Return the C function name that should be used for the skeleton
    * method.  This is a C function that is called by the IOR.
@@ -521,13 +666,13 @@ public class Fortran implements CodeConstants
 
   /**
    * Return the C type corresponding to the FORTRAN type corresponding to
-   * a particular SIDL type.  This is the type that a C subroutine would
+   * a particular sidl type.  This is the type that a C subroutine would
    * need to pass to FORTRAN, or the type that a C subroutine could expect
    * to receive from a FORTRAN caller.
    *
-   * @parm type  the SIDL type description.
+   * @parm type  the sidl type description.
    * @return the C type corresponding to the FORTRAN type corresponding
-   *         to the SIDL type.
+   *         to the sidl type.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    the type is unsupported.
    */
@@ -536,7 +681,7 @@ public class Fortran implements CodeConstants
   {
     final int t = type.getDetailedType();
     if (t >= 0 && t < s_cTypeMatchingFortran.length) {
-      if (s_isF90) {
+      if (isFortran90()) {
         if (t == s_Fortran_Bool) return s_F90_Bool;
         if (t == Type.ARRAY) return s_F90_Array;
       }
@@ -546,30 +691,59 @@ public class Fortran implements CodeConstants
   }
 
   /**
-   * Return the SIDL Fortran prefix for macros and types.
+   * Return the sidl Fortran prefix for macros and types.
    *
-   * @return the SIDL Fortran prefix
+   * @return the sidl Fortran prefix
    */
   public static String getFortranPrefix() {
     return "SIDL_F" + getFortranVersion();
   }
 
+  public static boolean needsAbbrev()
+  {
+    return ! "f77".
+      equals(BabelConfiguration.getInstance().getTargetLanguage());
+  }
+
   public static boolean isFortran90()
   {
-    return s_isF90;
+    return "f90".equals(BabelConfiguration.getInstance().getTargetLanguage());
+  
   }
 
   public static String arrayIOR()
   {
-    return s_isF90 ? ".d_ior" : "";
+    return isFortran90() ? ".d_ior" : "";
+  }
+
+  public static Method createCast(SymbolID id)
+  {
+    Method m = new Method();
+    m.setMethodName("_cast");
+    m.setDefinitionModifier(Method.STATIC);
+    String[] s = new String[1];
+    s[0] = "Cast method for interface and type conversions.";
+    m.setComment(new Comment(s));
+    m.setReturnType(new Type(id));
+    Argument a = new
+      Argument(false, Argument.IN, new Type(Type.OPAQUE), "ref");
+    m.addArgument(a);
+    return m;
+  }
+
+  public static  Method createCastTwo(SymbolID id) throws CodeGenerationException
+  {
+    Method m = IOR.getBuiltinMethod(IOR.CAST, id).cloneMethod();
+    m.setMethodName("_cast2");
+    return m;
   }
 
   /**
    * Return the FORTRAN type declaration corresponding to <code>type</code>,
-   * a particular SIDL type.
+   * a particular sidl type.
    *
-   * @param  type  a SIDL type description.
-   * @return the FORTRAN type used to some something of the given SIDL
+   * @param  type  a sidl type description.
+   * @return the FORTRAN type used to some something of the given sidl
    * type.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    the type is unsupported.
@@ -578,26 +752,35 @@ public class Fortran implements CodeConstants
     throws CodeGenerationException
   {
     final int t = type.getDetailedType();
-    if (s_isF90) {
+    if (isFortran90()) {
       if (t >= 0 && t < s_f90_types.length) {
         return s_f90_types[t];
-      }
-      else {
+      } else {
         switch(t) {
         case Type.INTERFACE:
         case Type.CLASS:
           return "type(" + getTypeName(type.getSymbolID()) + ")";
         case Type.ARRAY:
           Type arrayType = type.getArrayType();
-          if (arrayType.getDetailedType() <= Type.STRING) {
-            return "type(" +
-              getArrayName(new SymbolID("SIDL." + arrayType.getTypeString(),
-                                        new Version()),
-                           type.getArrayDimension()) + ")";
+          if (null != arrayType) {
+            if (type.isRarray()) {
+              return getReturnString(arrayType);
+            }
+            else{
+              if (arrayType.getDetailedType() <= Type.STRING) {
+                return "type(" +
+                  getArrayName(new SymbolID("sidl." + arrayType.getTypeString(),
+                                            new Version()),
+                               type.getArrayDimension()) + ")";
+              }
+              else {
+                return "type(" + getArrayName(arrayType.getSymbolID(),
+                                              type.getArrayDimension()) + ")";
+              }
+            }
           }
           else {
-            return "type(" + getArrayName(arrayType.getSymbolID(),
-                                          type.getArrayDimension()) + ")";
+            return "type(sidl__array)";
           }
         default:
           throw new CodeGenerationException("Unknown supported " + t);
@@ -607,6 +790,9 @@ public class Fortran implements CodeConstants
       if (t >= 0 && t < s_f77_types.length) {
         return s_f77_types[t];
       }
+      else if (type.isArray()) {
+        return type.isRarray() ? getReturnString(type.getArrayType()) : "integer*8";
+      }
       throw new CodeGenerationException("Unknown supported " + t);
     }
   }
