@@ -1,8 +1,7 @@
 //
 // File:        ModuleSource.java
 // Package:     gov.llnl.babel.backend.fortran
-// Release:     $Name: release-0-8-8 $
-// Revision:    @(#) $Revision: 1.10 $
+// Revision:    @(#) $Revision: 4434 $
 // Description: Generate a module file for FORTRAN 90 clients
 //
 // Copyright (c) 2002-2003, The Regents of the University of Calfornia.
@@ -32,16 +31,17 @@
 
 package gov.llnl.babel.backend.fortran;
 
-import gov.llnl.babel.BabelConfiguration;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
-import gov.llnl.babel.backend.IOR;
+import gov.llnl.babel.backend.CodeSplicer;
 import gov.llnl.babel.backend.fortran.Fortran;
 import gov.llnl.babel.backend.fortran.StubSource;
 import gov.llnl.babel.backend.writers.LanguageWriterForFortran;
+import gov.llnl.babel.backend.mangler.NameMangler;
 import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.Comment;
 import gov.llnl.babel.symbols.Extendable;
+import gov.llnl.babel.symbols.Class;
 import gov.llnl.babel.symbols.Enumeration;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.Symbol;
@@ -52,14 +52,16 @@ import gov.llnl.babel.symbols.Version;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
 
 /**
  * This class provides the ability to write a FORTRAN 90 module for
- * a SIDL class/interface. 
+ * a sidl class/interface. 
  */
 public class ModuleSource {
   private static final int s_CASTS_PER_LINE = 7;
@@ -67,8 +69,13 @@ public class ModuleSource {
   /**
    * Indicator of version of FORTRAN being generated.
    */
-  private final static boolean d_isF90 = "f90".
-    equals(BabelConfiguration.getInstance().getTargetLanguage());
+  private final static boolean d_isF90 = Fortran.isFortran90();
+
+  /**
+   * If Fortran90 is being generated, we need to keep track of 
+   * what explicit method interfaces have been already generated.
+   */
+  private static HashSet interfacesWritten = null;
 
   /**
    * This is the output device.
@@ -92,6 +99,41 @@ public class ModuleSource {
     d_lw = writer;
   }
 
+  /**
+   * This is a convenience utility function specifically for the generation
+   * of super "Stub" functions in the Impl files. 
+   * The output stream is not closed on exit.  A code
+   * generation exception is thrown if an error is detected.
+   *
+   * @param writer the output writer to which the stub source will
+   *               be written. This will not be closed.
+   *
+   * @param cls The class in which these supers are to be generated      
+   *
+   * @exception gov.llnl.babel.backend.CodeGenerationException
+   *    this is a catch all exception. It can be caused by I/O trouble or
+   *    violations of the data type invariants.
+   */
+  public static void generateSupers(Class             cls,
+                                    LanguageWriterForFortran writer,
+                                    CodeSplicer              splicer,
+                                    NameMangler mang)
+    throws CodeGenerationException
+  {
+    ModuleSource source = new ModuleSource(writer);
+    source.generateSupers(cls, splicer, mang);
+  }  
+
+  public void generateSupers(Class cls, CodeSplicer splicer, NameMangler mang)
+    throws CodeGenerationException
+  {
+    Collection methods = cls.getOverwrittenClassMethods();
+    for (Iterator m = methods.iterator(); m.hasNext(); ) {
+      Method method = (Method) m.next();
+      extendAndGenerateSuper(cls, method, splicer, mang);
+    }
+  }  
+
 
   /**
    * Generate the argument list for the subroutine, including the
@@ -112,9 +154,115 @@ public class ModuleSource {
     d_lw.println(")");
   }
 
+  /**
+   * Write out the declaration of an argument.
+   */
+  private void declareArgument(Argument a,
+                               boolean showIntent)
+    throws CodeGenerationException
+  {
+    Type argType = a.getType();
+    d_lw.writeCommentLine(a.getArgumentString());
+    d_lw.print(Fortran.getReturnString(argType));
+    if (showIntent) {
+      d_lw.print(" " + s_intent_spec[a.getMode()]);
+    }
+    if (argType.isRarray()) {
+      final int dimen = argType.getArrayDimension();
+      d_lw.print(", dimension(:");
+      for(int j = 1; j < dimen; ++j) {
+        d_lw.print(", :");
+      }
+      d_lw.print(")");
+    }
+    d_lw.println(" :: " + a.getFormalName());
+  }
+
+
   /**
    * Write the FORTRAN 90 module for a subroutine that corresponds to a 
-   * SIDL class/interface method. This writes method signatures and declares 
+   * sidl class/interface method. This writes method signatures and declares 
+   * the types of the arguments.
+   * 
+   * @param m     the method whose module method is to be written.
+   * @param id    the name of the class that owns this method.
+   * @param suffix string to add to method name
+   * @param convertRarray <code>true</code> means that rarray arguments
+   *                      should be converted to normal arrays before
+   *                      writing them out.
+   * @exception   gov.llnl.babel.backend.CodeGenerationException
+   *   a catch all exception to indicate problems in the code generation
+   *   phase.
+   */
+  private void writeMethodStub(Method   m,
+                               SymbolID id,
+                               String suffix,
+                               boolean convertRarray)
+    throws CodeGenerationException
+  {
+
+    List extendedArgs = StubSource.extendArgs(id, m, false);
+    Map indexMap;
+    final String methodName = m.getLongMethodName() + suffix;
+    final String stubName = 
+      (convertRarray ? Fortran.getAltStubName(id, m) 
+       : Fortran.getMethodStubName(id, m));
+    d_lw.print("recursive subroutine " + methodName);
+    if (convertRarray) {
+      extendedArgs = StubSource.convertRarrayToArray(extendedArgs);
+      indexMap = new HashMap(0);
+    }
+    else {
+      indexMap = m.getRarrayInfo();
+    }
+    printArgs(extendedArgs);
+
+    d_lw.tab();
+    d_lw.println("implicit none");
+
+    Iterator i = extendedArgs.iterator();
+    while (i.hasNext()) {
+      Argument a = (Argument)i.next();
+      declareArgument(a, true);
+    }
+    // declare array indices
+    i = indexMap.keySet().iterator();
+    while (i.hasNext()) {
+      String indexVar = (String)i.next();
+      Collection choice = (Collection)indexMap.get(indexVar);
+      if (choice.size() > 0) {
+        Method.RarrayInfo info = (Method.RarrayInfo)choice.iterator().next();
+        declareArgument(info.index, false);
+      }
+    }
+    d_lw.println();
+    d_lw.println("external " + stubName);
+    i = indexMap.keySet().iterator();
+    while (i.hasNext()) {
+      String indexVar = (String)i.next();
+      Collection choice = (Collection)indexMap.get(indexVar);
+      if (choice.size() > 0) {
+        Method.RarrayInfo info = (Method.RarrayInfo)choice.iterator().next();
+        d_lw.println(indexVar + " = size(" +
+                     info.rarray.getFormalName() + ", " +
+                     (info.dim+1) + ')');
+      }
+    }
+    d_lw.print("call " + stubName);
+    if (convertRarray) {
+      printArgs(extendedArgs);
+    }
+    else {
+      printArgs(StubSource.extendArgs(id, m, true));
+    }
+    d_lw.backTab();
+    d_lw.println();
+    d_lw.println("end subroutine " + methodName);
+  }
+
+  /**
+   * Write the FORTRAN 90 module for a subroutine that corresponds to a 
+   * sidl class/interface method. This writes method signatures and declares 
    * the types of the arguments.
    * 
    * @param m     the method whose module method is to be written.
@@ -129,13 +277,42 @@ public class ModuleSource {
                                  SymbolID id)
     throws CodeGenerationException
   {
-    List extendedArgs = StubSource.extendArgs(id, m);
-    final String methodName = m.getLongMethodName() + "_s";
-    final String stubName = Fortran.getMethodStubName(id, m);
+    if (m.hasRarray()) {
+      writeMethodStub(m, id, "_1s", true);
+      d_lw.println();
+      writeMethodStub(m, id, "_2s", false);
+    }
+    else {
+      writeMethodStub(m, id, "_s", false);
+    }
+  }
+  
+  /**
+   * Generate the extended argument list for a method and then generate the
+   * stub code for the method.
+   * 
+   * @param m           the method to be generated.
+   * @param id          the name of the class/interface who owns the method.
+   * @param isInterface is the method owned by a class or interface.
+   * @exception gov.llnl.babel.backend.CodeGenerationException
+   *  a catch all exception for problems in the code generation phase.
+   */
+  private void extendAndGenerateSuper(Class cls, 
+                                      Method   m,
+                                      CodeSplicer              splicer,
+                                      NameMangler mang)
+    throws CodeGenerationException
+  {
+    SymbolID id = cls.getSymbolID();
+    List extendedArgs = StubSource.extendArgs(id, m, false);
+    final String methodName = Fortran.getMethodSuperImplName(id,m,mang);//"super_"+ m.getLongMethodName() + "_s";
+    final String stubName = Fortran.getMethodSuperName(id, m);
     d_lw.print("recursive subroutine " + methodName);
     printArgs(extendedArgs);
 
-    d_lw.increaseTabLevel();
+    ImplSource.useStatementsForSupers(m,id,d_lw, splicer);
+
+    d_lw.tab();
     d_lw.println("implicit none");
 
     Iterator i = extendedArgs.iterator();
@@ -151,12 +328,15 @@ public class ModuleSource {
     d_lw.print("call " + stubName);
     printArgs(extendedArgs);
 
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println();
     d_lw.println("end subroutine " + methodName);
+    d_lw.println();
   }
 
 
+
+
   /**
    * Generate a create method object.
    */
@@ -232,10 +412,15 @@ public class ModuleSource {
   {
     if (t.getDetailedType() == Type.ARRAY) {
       final Type arrayType = t.getArrayType();
-      final int detailedType = arrayType.getDetailedType();
-      if ((detailedType > Type.VOID) && (detailedType <= Type.STRING)) {
-        result.add(new SymbolID("SIDL." + arrayType.getTypeString(),
-                                new Version()));
+      if (null != arrayType) {
+        final int detailedType = arrayType.getDetailedType();
+        if ((detailedType > Type.VOID) && (detailedType <= Type.STRING)) {
+          result.add(new SymbolID("sidl." + arrayType.getTypeString(),
+                                  new Version()));
+        }
+      }
+      else {
+        result.add(new SymbolID("sidl.array", new Version()));
       }
     }
   }
@@ -286,13 +471,13 @@ public class ModuleSource {
                             int      num)
   {
     String castMethod = Fortran.getMethodStubName
-      (newType, StubDoc.createCast(newType));
+      (newType, Fortran.createCast(newType));
     d_lw.beginBlockComment(false);
     d_lw.println("Static function to cast from " + oldType.getFullName());
     d_lw.println("to " + newType.getFullName() + ".");
     d_lw.endBlockComment(false);
     d_lw.println("subroutine cast_" + num + "(oldType, newType)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println("implicit none");
     d_lw.println("type(" + Fortran.getTypeName(oldType) + 
                  "), intent(in) :: oldType");
@@ -301,7 +486,7 @@ public class ModuleSource {
     d_lw.println("external " + castMethod);
     d_lw.println();
     d_lw.println("call " + castMethod + "(oldType, newType)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine cast_" + num);
     d_lw.println();
   }
@@ -333,7 +518,7 @@ public class ModuleSource {
           && (i < (numCasts - 1))) {
         d_lw.println(", &");
         if (numCasts == 7) {
-          d_lw.increaseTabLevel();
+          d_lw.tab();
         }
       }
       else {
@@ -351,17 +536,17 @@ public class ModuleSource {
       writeCastList(numCasts);
       d_lw.println();
       if (numCasts > 7) {
-        d_lw.decreaseTabLevel();
+        d_lw.backTab();
       }
       d_lw.println("interface cast");
-      d_lw.increaseTabLevel();
+      d_lw.tab();
       d_lw.print("module procedure ");
       writeCastList(numCasts);
       d_lw.println();
       if (numCasts > 7) {
-        d_lw.decreaseTabLevel();
+        d_lw.backTab();
       }
-      d_lw.decreaseTabLevel();
+      d_lw.backTab();
       d_lw.println("end interface");
     }
   }
@@ -369,56 +554,115 @@ public class ModuleSource {
   private void generateNullSubroutines(Extendable ext)
   {
     d_lw.println("logical function is_null_s(ext)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println("type(" + Fortran.getTypeName(ext.getSymbolID()) +
                  "), intent(in) :: ext");
     d_lw.println("is_null_s = (ext%d_ior .eq. 0)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end function is_null_s");
     d_lw.println();
     d_lw.println("logical function not_null_s(ext)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println("type(" + Fortran.getTypeName(ext.getSymbolID()) +
                  "), intent(in) :: ext");
     d_lw.println("not_null_s = (ext%d_ior .ne. 0)");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end function not_null_s");
     d_lw.println();
     d_lw.println("subroutine set_null_s(ext)");
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println("type(" + Fortran.getTypeName(ext.getSymbolID()) +
                  "), intent(out) :: ext");
     d_lw.println("ext%d_ior = 0");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end subroutine set_null_s");
     d_lw.println();
   }
-  
+  //write the interface for generated (non-user) methods
   private void writeMethodInterface(String methodName)
   {
     d_lw.println("private :: " + methodName + "_s");
     d_lw.println("interface " + methodName);
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println("module procedure " + methodName + "_s");
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end interface");
     d_lw.println();
   }
 
-  private void writeMethodInterfaces(Collection methods)
+  private void writeMethodInterface(Extendable ext, Method m)
   {
+    String sName = m.getShortMethodName();
+    Collection overloadedMethods = ext.getOverloadedMethodsByName(sName);    
+    //This is only empty in one wierd case for the method new.
+    if(overloadedMethods.isEmpty())
+      writeMethodInterface(sName);
+    else {
+      //This simply lists all the long method names as private (it only
+      //looks complex)
+      d_lw.print("private :: "); 
+      d_lw.tab();
+      for(Iterator i = overloadedMethods.iterator(); i.hasNext();) {
+        Method meth = (Method) i.next();
+        //Rarrays actually map to 2 different methods, one takes rarrays,
+        //the other sidl arrays.  We do this in F90 and C++
+        if(meth.hasRarray()) {
+          d_lw.println(meth.getLongMethodName() + "_1s, &");
+          d_lw.print(meth.getLongMethodName() + "_2s");
+        } else {
+          d_lw.print(meth.getLongMethodName() + "_s");
+        }
+        if(i.hasNext()) {
+          d_lw.println(", &");
+        }
+        else {
+          d_lw.println();
+        }
+      }
+      d_lw.backTab();
+      d_lw.println();
+      
+      //This maps the method short name to all the long names.
+      d_lw.println("interface " + sName);
+      d_lw.tab();
+      d_lw.print("module procedure ");
+      for(Iterator i = overloadedMethods.iterator(); i.hasNext();) {
+        Method meth = (Method) i.next();
+        if(meth.hasRarray()) {
+          d_lw.println(meth.getLongMethodName() + "_1s, &");
+          d_lw.print(meth.getLongMethodName() + "_2s");
+        } else {
+          d_lw.print(meth.getLongMethodName() + "_s");
+        }
+        if(i.hasNext()) {
+          d_lw.println(", &");
+        } else {
+          d_lw.println();
+        }
+      }
+      d_lw.backTab();
+      d_lw.println("end interface");
+    } 
+  }
+
+  private void writeMethodInterfaces(Extendable ext)
+  {
+    Collection methods = extendMethods(ext);
     Iterator i = methods.iterator();
     while (i.hasNext()) {
       Method m = (Method)i.next();
-      writeMethodInterface(m.getLongMethodName());
+      if(!interfacesWritten.contains(m.getShortMethodName())) {
+        writeMethodInterface(ext, m);
+        interfacesWritten.add(m.getShortMethodName());
+      }
     }
   }
 
 
   /**
-   * Generate the FORTRAN 90 module file for a SIDL class.  
+   * Generate the FORTRAN 90 module file for a sidl class.  
    * 
-   * @param ext    the SIDL class whose module is to be written.
+   * @param ext    the sidl class whose module is to be written.
    * @exception    gov.llnl.babel.backend.CodeGenerationException
    *   a catch all exception to indicate problems in the code generation
    *   phase.
@@ -430,10 +674,7 @@ public class ModuleSource {
     Collection parents = ext.getParents(true);
     final SymbolID id = ext.getSymbolID();
     final String name = Fortran.getModule(id);
-
-    d_lw.writeBanner(ext, Fortran.getModuleFile(id), false,
-                     CodeConstants.C_FORTRAN_MODULE_PREFIX 
-                     + id.getFullName());
+    interfacesWritten = new HashSet();
     d_lw.println();
     d_lw.writeComment(ext, false);
     d_lw.println();
@@ -441,22 +682,22 @@ public class ModuleSource {
     d_lw.println();
     d_lw.println("module " + name);
     d_lw.println();
-    d_lw.increaseTabLevel();
+    d_lw.tab();
 
     includeTypes(ext);
 
     writeCastInterface(parents.size()*2);
-    writeMethodInterfaces(methods);
+    writeMethodInterfaces(ext);
     writeMethodInterface("not_null");
     writeMethodInterface("is_null");
     writeMethodInterface("set_null");
 
     d_lw.println();
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("contains");
     d_lw.println();
 
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     Iterator i = methods.iterator();
     while (i.hasNext()) {
       d_lw.println();
@@ -468,15 +709,15 @@ public class ModuleSource {
     generateCastMethods(ext, parents);
     generateNullSubroutines(ext);
 
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println();
     d_lw.println("end module " + name);
   }
 
   /**
-   * Generate the FORTRAN 90 module file for a SIDL enumerated type.  
+   * Generate the FORTRAN 90 module file for a sidl enumerated type.  
    * 
-   * @param enm    the SIDL enumeration whose module is to be written.
+   * @param enm    the sidl enumeration whose module is to be written.
    * @exception    gov.llnl.babel.backend.CodeGenerationException
    *   a catch all exception to indicate problems in the code generation
    *   phase.
@@ -486,13 +727,10 @@ public class ModuleSource {
   {
     final SymbolID id = enm.getSymbolID();
     final String name = Fortran.getModule(id);
-    d_lw.writeBanner(enm, Fortran.getModuleFile(id), false,
-                     CodeConstants.C_FORTRAN_MODULE_PREFIX 
-                     + id.getFullName());
     d_lw.println();
     d_lw.println("module " + name);
     d_lw.writeComment(enm, false);
-    d_lw.increaseTabLevel();
+    d_lw.tab();
     d_lw.println();
     Iterator  i = enm.getEnumerators().iterator();
     while (i.hasNext()){
@@ -507,13 +745,13 @@ public class ModuleSource {
         d_lw.println();
       }
     }
-    d_lw.decreaseTabLevel();
+    d_lw.backTab();
     d_lw.println("end module " + name);
   }
 
 
   /**
-   * Generate the FORTRAN 90 module file for a SIDL class.  
+   * Generate the FORTRAN 90 module file for a sidl class.  
    *
    * Note:  This is the assumed entry point; otherwise, the test for
    * the version of the language should be repeated.
