@@ -1,11 +1,10 @@
 //
 // File:        IORHeader.java
 // Package:     gov.llnl.babel.backend.ior
-// Release:     $Name: release-0-8-8 $
-// Revision:    @(#) $Id: IORHeader.java,v 1.23 2003/10/16 21:44:17 epperly Exp $
+// Revision:    @(#) $Id: IORHeader.java 4434 2005-03-17 17:05:29Z epperly $
 // Description: generate IOR header to a pretty writer stream
 //
-// Copyright (c) 2000-2003, The Regents of the University of Calfornia.
+// Copyright (c) 2000-2004, The Regents of the University of Calfornia.
 // Produced at the Lawrence Livermore National Laboratory.
 // Written by the Components Team <components@llnl.gov>
 // UCRL-CODE-2002-054
@@ -35,6 +34,7 @@ import gov.llnl.babel.BabelConfiguration;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.IOR;
+import gov.llnl.babel.backend.c.C;
 import gov.llnl.babel.backend.ior.IORSource;
 import gov.llnl.babel.backend.SortComparator;
 import gov.llnl.babel.backend.Utilities;
@@ -50,7 +50,6 @@ import gov.llnl.babel.symbols.Package;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import java.util.Arrays;
-import java.util.Collection;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
@@ -64,8 +63,8 @@ import java.util.Set;
  * stream is not closed by this object.
  */
 public class IORHeader {
-   private final static String SIDL_EXCEPTION = 
-     BabelConfiguration.getBaseException();
+   private final static String SIDL_EXCEPTION_INTERFACE = 
+     BabelConfiguration.getBaseExceptionInterface();
 
    private LanguageWriterForC d_writer;
 
@@ -76,7 +75,8 @@ public class IORHeader {
     * exception is thrown if an error is detected.
     */
    public static void generateCode(Symbol symbol, LanguageWriterForC writer)
-         throws CodeGenerationException {
+      throws CodeGenerationException 
+   {
       IORHeader header = new IORHeader(writer);
       header.generateCode(symbol);
    }
@@ -101,14 +101,12 @@ public class IORHeader {
       if (symbol != null) {
          switch (symbol.getSymbolType()) {
          case Symbol.CLASS:
+         case Symbol.INTERFACE:
             generateExtendable((Extendable) symbol);
             break;
          case Symbol.ENUM:
             generateEnumeration((Enumeration) symbol);
             break;
-         case Symbol.INTERFACE:
-            generateExtendable((Extendable) symbol);
-            break;
          case Symbol.PACKAGE:
             generatePackage((Package) symbol);
             break;
@@ -129,16 +127,21 @@ public class IORHeader {
       d_writer.writeBanner(enm, header, CodeConstants.C_IS_NOT_IMPL, 
          CodeConstants.C_DESC_IOR_PREFIX + id.getFullName());
       d_writer.openHeaderGuard(header);
-      d_writer.generateInclude("SIDLType.h", true);
+      d_writer.generateInclude("sidlType.h", true);
       d_writer.openCxxExtern();
       d_writer.writeComment(enm, false);
 
+      d_writer.println();
+      d_writer.writeCommentLine("Opaque forward declaration of array struct");
+      d_writer.println(IOR.getArrayName(id) + ";");
+      d_writer.println();
+
       /*
        * Write out the enumeration symbol and increase the tab level.
        */
       d_writer.print(IOR.getEnumName(id));
       d_writer.println(" {");
-      d_writer.increaseTabLevel();
+      d_writer.tab();
 
       /*
        * Output each of the enumerators (with fill space) along with its
@@ -168,7 +171,7 @@ public class IORHeader {
        * Close the enumeration statement and write out the array structure
        * for the enumerated type.
        */
-      d_writer.decreaseTabLevel();
+      d_writer.backTab();
       d_writer.println("};");
       d_writer.println();
 
@@ -190,7 +193,8 @@ public class IORHeader {
     * for the header guards.
     */
    private void generateExtendable(Extendable ext)
-         throws CodeGenerationException {
+      throws CodeGenerationException 
+   {
       /*
        * Generate the file banner and open the header include guard.
        */
@@ -215,6 +219,13 @@ public class IORHeader {
       defined.add(id);
       generateForwardDeclarations(ext, defined);
 
+      /*
+       * Generate method indices, if necessary.
+       */
+      if (!ext.isInterface()) {
+        generateMethodDescData(ext);
+      }
+
       /*
        * Generate entry point vector(s).
        */
@@ -229,6 +240,7 @@ public class IORHeader {
       if (ext.isInterface()) {
          generateInterfaceObject((Interface) ext);
       } else {
+         generateControls(ext);
          generateClassObject((Class) ext);
          generateExternalStruct(ext);
          IORSource.generateExternalSignature(d_writer, ext, ";");
@@ -241,6 +253,7 @@ public class IORHeader {
        */
       d_writer.closeCxxExtern();
       d_writer.closeHeaderGuard();
+      
    }
 
    /**
@@ -271,29 +284,32 @@ public class IORHeader {
       d_writer.closeHeaderGuard();
    }
 
-  private void generateExternalStruct(Symbol sym)
-  {
+  private void generateExternalStruct(Symbol sym) {
     if (sym instanceof Class) {
       Class cls = (Class)sym;
       final SymbolID id = sym.getSymbolID();
       final String symbolType = IOR.getSymbolType(sym);
       final int maxDim = BabelConfiguration.getMaximumArray();
       d_writer.println(IOR.getExternalName(id) + " {");
-      d_writer.increaseTabLevel();
+      d_writer.tab();
       if (!cls.isAbstract()) {
         d_writer.println(symbolType);
         d_writer.println("(*createObject)(void);");
         d_writer.println();
       }
-      d_writer.println(symbolType);
-      d_writer.println("(*createRemote)(const char *url);");
-      d_writer.println();
       if (cls.hasStaticMethod(true)) {
         d_writer.println(IOR.getSEPVName(id) + "*");
         d_writer.println("(*getStaticEPV)(void);");
-        d_writer.println();
       }
-      d_writer.decreaseTabLevel();
+      if (cls.getParentClass() != null) { 
+	  Class parent = cls.getParentClass();
+	  if ( parent != null ) {
+	      SymbolID pid = parent.getSymbolID();
+	      d_writer.print(IOR.getEPVName(pid) + "*");
+	      d_writer.println("(*getSuperEPV)(void);");
+	  }
+      }
+      d_writer.backTab();
       d_writer.println("};");
       d_writer.println();
     }
@@ -307,8 +323,7 @@ public class IORHeader {
     * output.  If any of the symbols do not exist in the symbol table,
     * then throw a code generation exception.
     */
-   private Set generateIncludes(Extendable ext)
-         throws CodeGenerationException {
+   private Set generateIncludes(Extendable ext) throws CodeGenerationException {
       /*
        * Create the set of include symbol identifiers and add inheritance
        * parents from interfaces and (if a class) the superclass.
@@ -342,7 +357,21 @@ public class IORHeader {
       /*
        * Sort the entries and output the list of include files.
        */
-      d_writer.generateInclude("SIDL_header.h", true);
+      d_writer.generateInclude("sidl_header.h", true);
+      d_writer.generateInclude(C.getHeaderFile(
+        Utilities.lookupSymbol("sidl.io.Serializer").getSymbolID()), true);
+      d_writer.generateInclude(C.getHeaderFile(
+        Utilities.lookupSymbol("sidl.io.Deserializer").getSymbolID()), true);
+      if ( IOR.supportAssertions(ext) && (!ext.isInterface()) ) {
+        d_writer.generateInclude("sidlAsserts.h", true);
+        if (IOR.ASSERTION_LEVEL != BabelConfiguration.FULL_CHECKING_STATS) {
+          d_writer.print("//");
+        }
+        d_writer.println("#define " + IOR.S_FULL_STATS_MACRO);
+        d_writer.println("//#define " + IOR.S_TEXT_STATS_MACRO);
+        d_writer.generateInclude("stdlib.h", true);
+        d_writer.generateInclude("math.h", true);
+      }
       if (!includes.isEmpty()) {
          List entries = Utilities.sort(includes);
          for (Iterator i = entries.iterator(); i.hasNext(); ) {
@@ -385,25 +414,47 @@ public class IORHeader {
          d_writer.println(IOR.getNewName(id) + "(void);");
          d_writer.println();
       }
-      d_writer.println("extern " + type + "*");
-      d_writer.println(IOR.getRemoteName(id) + "(const char *url);");
-      d_writer.println();
+      boolean doStaticTypes = false;
       if (ext.hasStaticMethod(true)) {
          d_writer.println("extern " + sepv + "*");
          d_writer.println(IOR.getStaticsName(id) + "(void);");
+         if (IOR.supportAssertions(ext) || IOR.supportInterceptors(ext)) {
+           d_writer.println();
+           d_writer.println("extern " + sepv + "*");
+           d_writer.println(IOR.getLocalStaticsName(id) + "(int type);");
+           doStaticTypes = true;
+         }
          d_writer.println();
       }
       if (!ext.isInterface()) {
          d_writer.println("extern void " + IOR.getInitName(id) + "(");
-         d_writer.increaseTabLevel();
+         d_writer.tab();
          d_writer.println(type + "* self);");
-         d_writer.decreaseTabLevel();
+         d_writer.backTab();
          d_writer.println("extern void " + IOR.getFiniName(id) + "(");
-         d_writer.increaseTabLevel();
+         d_writer.tab();
          d_writer.println(type + "* self);");
-         d_writer.decreaseTabLevel();
-         d_writer.println("extern void " + IOR.getVersionName(id) + 
-                          "(int32_t *major, int32_t *minor);");
+         d_writer.backTab();
+         d_writer.println("extern void " + IOR.getVersionName(id) + "(int32_t *"
+           + "major, int32_t *minor);");
+         d_writer.println();
+      }
+      if (doStaticTypes) {
+         d_writer.writeComment("Define static structure options.", false);
+         d_writer.println("static const int " 
+           + IOR.getStaticTypeOption(id, IOR.SET_PUBLIC) + " = 0;");
+         int i = 1;
+         if (IOR.supportAssertions(ext)) {
+            d_writer.println("static const int " 
+              + IOR.getStaticTypeOption(id, IOR.SET_ASSERTIONS) + " = " + i 
+              + ";");
+            i++;
+         }
+         if (IOR.supportInterceptors(ext)) {
+            d_writer.println("static const int " 
+              + IOR.getStaticTypeOption(id, IOR.SET_INTERCEPTORS) + " = " + i 
+              + ";");
+         }
          d_writer.println();
       }
    }
@@ -417,7 +468,8 @@ public class IORHeader {
     * then throw a code generation exception.
     */
    private void generateForwardDeclarations(Extendable ext, Set defined)
-         throws CodeGenerationException {
+      throws CodeGenerationException 
+   {
       /*
        * Generate the set of dependencies by extracting the data
        * dependencies from each of the methods in the methods list.
@@ -428,7 +480,7 @@ public class IORHeader {
          Method method = (Method) i.next();
          references.addAll(method.getSymbolReferences());
          if (!method.getThrows().isEmpty()) {
-            Symbol symbol = Utilities.lookupSymbol(SIDL_EXCEPTION);
+            Symbol symbol = Utilities.lookupSymbol(SIDL_EXCEPTION_INTERFACE);
             references.add(symbol.getSymbolID());
          }
       }
@@ -441,15 +493,15 @@ public class IORHeader {
       if (!references.isEmpty()) {
 
          /*
-          * Generate the set of referenced symbols that are interfaces
-          * or classes.  These will be output as forward declarations.
+          * Generate the set of referenced symbols that are interfaces,
+          * classes or enums.  These will be output as forward declarations.
           */
          Set declare = new HashSet();
          for (Iterator i = references.iterator(); i.hasNext(); ) {
             SymbolID id = (SymbolID) i.next();
             Symbol symbol = Utilities.lookupSymbol(id);
             int type = symbol.getSymbolType();
-            if ((type == Symbol.INTERFACE) || (type == Symbol.CLASS)) {
+            if (type != Symbol.PACKAGE) {
                declare.add(id);
             }
          }
@@ -464,8 +516,12 @@ public class IORHeader {
             List entries = Utilities.sort(declare);
             for (Iterator i = entries.iterator(); i.hasNext(); ) {
                SymbolID id = (SymbolID) i.next();
+               Symbol symbol = Utilities.lookupSymbol(id);
+               int type = symbol.getSymbolType();
                d_writer.println(IOR.getArrayName(id) + ";");
-               d_writer.println(IOR.getObjectName(id) + ";");
+               if (type != Symbol.ENUM) {
+                 d_writer.println(IOR.getObjectName(id) + ";");
+               }
             }
             d_writer.println();
          }
@@ -473,68 +529,70 @@ public class IORHeader {
    }
 
   /**
-   * Generate a single entry in the definition of an entry
+   * Generate the actual method entry in the definition of an entry
    * point vector.
    * 
-   * @param m           a description of the method
+   * @param m           the method to be generated
    * @param self        a string with the type of the object
+   * @param addHooks    indicator of whether interceptors should be added.
    * @param alreadySeen set of method names already generated.
    *                    Each element is a <code>String</code>.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private void generateEPVEntry(Method m, String self, Set alreadySeen)
+  private void generateEPVEntry(Method m, String self, String suffix) 
+    throws CodeGenerationException
+  {
+    if (suffix.equals("")) {
+      d_writer.print(IOR.getReturnString(m.getReturnType(), true, false));
+    } else {
+      d_writer.print("void");
+    }
+    d_writer.print(" (*");
+    d_writer.print(IOR.getVectorEntry(m.getLongMethodName()) + suffix);
+    d_writer.println(")(");
+    d_writer.tab();
+
+    /* 
+     * Generate the argument list.
+     */
+    boolean has_throws = !m.getThrows().isEmpty() && (suffix.equals(""));
+    List args = m.getArgumentList();
+    C.generateArguments(d_writer, self, args, m.isStatic(), has_throws, 
+        (suffix.equals(IOR.GENERIC_POST_SUFFIX) ? m.getReturnType() : null), 
+        true, true, false);
+    d_writer.println(");");
+    d_writer.backTab();
+  }
+
+  /**
+   * Generate the Method entry(ies) in the definition of an entry
+   * point vector.
+   * 
+   * @param m           the method to be generated
+   * @param self        a string with the type of the object
+   * @param addHooks    indicator of whether interceptors should be added.
+   * @param alreadySeen set of method names already generated.
+   *                    Each element is a <code>String</code>.
+   * @exception gov.llnl.babel.backend.CodeGenerationException
+   *    this is a catch all exception. It can be caused by I/O trouble or
+   *    violations of the data type invariants.
+   */
+  private void generateEPVMethod(Method m, String self, boolean addHooks, 
+                                 Set alreadySeen)
     throws CodeGenerationException
   {
     String name = m.getLongMethodName();
     if (!alreadySeen.contains(name)) {
       alreadySeen.add(name);
-      d_writer.print(IOR.getReturnString(m.getReturnType()));
-      d_writer.print(" (*");
-      d_writer.print(IOR.getVectorEntry(name));
-      d_writer.println(")(");
-      d_writer.increaseTabLevel();
-
-      /*
-       * If the method is not static, then it will begin with an
-       * object reference.
-       */
-      boolean has_throws = !m.getThrows().isEmpty();
-      List args = m.getArgumentList();
-
-      if (!m.isStatic()) {
-        d_writer.print(self);
-        if ((args.size() > 0) || has_throws) {
-          d_writer.println(",");
-        }
+      if (addHooks) {
+        generateEPVEntry(m, self, IOR.GENERIC_PRE_SUFFIX);
+        generateEPVEntry(m, self, "");
+        generateEPVEntry(m, self, IOR.GENERIC_POST_SUFFIX);
       } else {
-        if ((args.size() == 0) && !has_throws) {
-          d_writer.print("void");
-        }
+        generateEPVEntry(m, self, "");
       }
-
-      /*
-       * Output each argument in turn.
-       */
-      for (Iterator a = args.iterator(); a.hasNext(); ) {
-        Argument arg = (Argument) a.next();
-        d_writer.print(IOR.getArgumentWithFormal(arg));
-        if (a.hasNext() || has_throws) {
-          d_writer.println(",");
-        }
-      }
-
-      /*
-       * If there is a throws clause, then return the exception type
-       * as the last in the argument list.
-       */
-      if (has_throws) {
-        d_writer.print(IOR.getExceptionType());
-        d_writer.print("*_ex");
-      }
-      d_writer.println(");");
-      d_writer.decreaseTabLevel();
     }
   }
 
@@ -548,15 +606,15 @@ public class IORHeader {
    * @param self               the type used for the self pointer.
    * @param doStatic           should <code>static</code> or
    *                           non-<code>static</code> entries be listed?
+   * @param addHooks           should interceptor hooks be listed?
    * @param methodsAlreadySeen set of method names already included.
    * @exception gov.llnl.babel.backend.CodeGenerationException
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private void generateLocalEntries(Extendable ext,
-                                    String     self,
-                                    boolean    doStatic,
-                                    Set        methodsAlreadySeen)
+  private void generateLocalEntries(Extendable ext, String self, 
+                                    boolean doStatic, boolean addHooks,
+                                    Set methodsAlreadySeen)
     throws CodeGenerationException
   {
     /*
@@ -573,14 +631,15 @@ public class IORHeader {
     }
     while (m.hasNext()) {
       Method method = (Method) m.next();
-      generateEPVEntry(method, self, methodsAlreadySeen);
+      generateEPVMethod(method, self, addHooks, methodsAlreadySeen);
     }
   }
 
   /**
    * Generate the implicit/builtin methods for this fundamental
    * class or interface. These methods always appear first in
-   * entry point vector (EPV).
+   * entry point vector (EPV).  Note that the built-in checks and
+   * interceptor methods are skipped for SIDL symbols.
    *
    * @param ext         the class/interface whose parents will be
    *                    asked to write their EPV entries.
@@ -592,25 +651,39 @@ public class IORHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private void generateBuiltinEntries(Extendable ext,
-                                      String     self,
-                                      boolean    doStatic,
-                                      Set        alreadySeen)
+  private void generateBuiltinEntries(Extendable ext, String self, 
+                                      boolean doStatic, Set alreadySeen)
     throws CodeGenerationException
   {
+    final SymbolID id          = ext.getSymbolID();
+    final int      numBuiltins = (ext.isInterface() 
+                                 ? IOR.INTERFACE_BUILTIN_METHODS
+                                 : IOR.CLASS_BUILTIN_METHODS);
+    final boolean  doAsserts   = IOR.supportAssertions(ext);
+    final boolean  doIceptors  = IOR.supportInterceptors(ext);
+
+    d_writer.writeCommentLine("Implicit builtin methods");
+
     /*
-     * If this is not a static EPV, then output standard function
-     * pointers for destructor, constructor, and virtual destructor.
+     * Output standard function pointers as appropriate.
      */
-    if (!doStatic) {
-      final SymbolID id = ext.getSymbolID();
-      final int numBuiltins = (ext.isInterface() 
-                               ? IOR.INTERFACE_BUILTIN_METHODS
-                               : IOR.CLASS_BUILTIN_METHODS);
-      d_writer.writeCommentLine("Implicit builtin methods");
+    if (doStatic) {
       for(int i = 0; i < numBuiltins; ++i){
-        Method b = IOR.getBuiltinMethod(i, id);
-        generateEPVEntry(b, self, alreadySeen); 
+        if (  IOR.hasStaticBuiltin(i)
+           && (  (IOR.isBuiltinAssert(i)  && IOR.supportAssertions(ext))
+              || ((i == IOR.INTERCEPTORS) && IOR.supportInterceptors(ext)) ) ) {
+          Method b = IOR.getBuiltinMethod(i, id, doStatic);
+          generateEPVMethod(b, self, false, alreadySeen); 
+        }
+      }
+    } else {
+      for(int i = 0; i < numBuiltins; ++i){
+        if (  IOR.isBuiltinBasic(i) 
+           || (IOR.isBuiltinAssert(i)  && IOR.supportAssertions(ext))
+           || ((i == IOR.INTERCEPTORS) && IOR.supportInterceptors(ext)) ) {
+          Method b = IOR.getBuiltinMethod(i, id, doStatic);
+          generateEPVMethod(b, self, false, alreadySeen); 
+        }
       }
     }
   }
@@ -625,6 +698,8 @@ public class IORHeader {
    * @param self               the type used for the self pointer.
    * @param doStatic           should <code>static</code> or
    *                           non-<code>static</code> entries be listed?
+   * @param addHooks           should the interceptor method entries be 
+   *                           listed?
    * @param methodsAlreadySeen set of method names already included.
    * @param extAlreadySeen     set of extendables whose methods have
    *                           already been included
@@ -632,17 +707,16 @@ public class IORHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private void generateParentEntries(Extendable ext,
-                                     String     self,
-                                     boolean    doStatic,
-                                     Set        methodsAlreadySeen,
-                                     Set        extAlreadySeen) 
+  private void generateParentEntries(Extendable ext, String self,
+                                     boolean doStatic, boolean addHooks,
+                                     Set methodsAlreadySeen,
+                                     Set extAlreadySeen) 
     throws CodeGenerationException
   {
     if (ext instanceof gov.llnl.babel.symbols.Class) {
       generateEPVEntries
-        (((gov.llnl.babel.symbols.Class)ext).getParentClass(),
-         self, doStatic, methodsAlreadySeen, extAlreadySeen);
+        (((gov.llnl.babel.symbols.Class)ext).getParentClass(), self, doStatic, 
+        addHooks, methodsAlreadySeen, extAlreadySeen);
     }
     /*
      * It is critical that we always visit parent interfaces in a canonical
@@ -652,21 +726,19 @@ public class IORHeader {
     Object[] parents = ext.getParentInterfaces(false).toArray();
     Arrays.sort(parents, new SortComparator());
     for(int i = 0; i < parents.length; ++i){
-      generateEPVEntries((Extendable)parents[i], self, doStatic, methodsAlreadySeen,
-                         extAlreadySeen);
+      generateEPVEntries((Extendable)parents[i], self, doStatic, addHooks,
+                         methodsAlreadySeen, extAlreadySeen);
     }
   }
 
   private static final boolean isBaseClass(Extendable ext) {
-    return 
-      ((ext instanceof gov.llnl.babel.symbols.Class) &&
+    return ((ext instanceof gov.llnl.babel.symbols.Class) &&
        (((gov.llnl.babel.symbols.Class)ext).getParentClass() == null));
        
   }
   
   private static final boolean hasParents(Extendable ext) {
-    return 
-      (!isBaseClass(ext)) || (!ext.getParentInterfaces(false).isEmpty());
+    return (!isBaseClass(ext)) || (!ext.getParentInterfaces(false).isEmpty());
   }
 
   /**
@@ -678,6 +750,8 @@ public class IORHeader {
    * @param self               the type used for the self pointer.
    * @param doStatic           should <code>static</code> or
    *                           non-<code>static</code> entries be listed?
+   * @param addHooks           should the interceptor method entries be
+   *                           listed?
    * @param methodsAlreadySeen set of method names already included.
    * @param extAlreadySeen     a set of extendables whose methodss have
    *                           already been included.
@@ -685,23 +759,72 @@ public class IORHeader {
    *    this is a catch all exception. It can be caused by I/O trouble or
    *    violations of the data type invariants.
    */
-  private void generateEPVEntries(Extendable ext, 
-                                  String     self,
-                                  boolean    doStatic,
-                                  Set        methodsAlreadySeen,
-                                  Set        extAlreadySeen) 
+  private void generateEPVEntries(Extendable ext, String self, boolean doStatic,
+                                  boolean addHooks, Set methodsAlreadySeen,
+                                  Set extAlreadySeen) 
     throws CodeGenerationException 
   {
     if ((ext != null) && !extAlreadySeen.contains(ext)) {
       if (hasParents(ext)) {
-        generateParentEntries(ext, self, doStatic, methodsAlreadySeen, extAlreadySeen);
+        generateParentEntries(ext, self, doStatic, addHooks, 
+                              methodsAlreadySeen, extAlreadySeen);
       }
-      generateLocalEntries(ext, self, doStatic, methodsAlreadySeen);
+      generateLocalEntries(ext, self, doStatic, addHooks, methodsAlreadySeen);
       extAlreadySeen.add(ext);
     }
   }
   
 
+   /**
+    * Generate the method description data, including indices to facilitate 
+    * access to the corresponding control information for assertion checking 
+    * purposes.
+    */
+   private void generateMethodDescData(Extendable ext) {
+     if (IOR.supportAssertions(ext)) {
+        d_writer.writeComment("Define method description data for assertion "
+          + "control", false);
+        int      num      = 0;
+        SymbolID id       = ext.getSymbolID();
+        List     methods  = (List)ext.getMethods(true);
+        String   typeDecl = "static const int32_t ";
+        String   minName  = IOR.getMethodIndex(id, "MIN");
+        int      width    = Utilities.getWidth(methods) + typeDecl.length()
+                            + (minName.length() - 3);
+
+        for (Iterator i = methods.iterator(); i.hasNext(); ) {
+           Method method = (Method) i.next();
+           d_writer.printAligned(typeDecl + IOR.getMethodIndex(id, method), 
+                                 width);
+           d_writer.println(" = " + num + ";");
+           num++;
+        }
+        d_writer.printAligned(typeDecl + minName, width);
+        d_writer.println(" = 0;");
+        d_writer.printAligned(typeDecl + IOR.getMethodIndex(id, "MAX"), width);
+        d_writer.println(" = " + (num-1) + ";");
+        d_writer.println();
+
+        d_writer.println("static const " + IOR.getMethodDescDataStruct(id) 
+          + " {");
+        d_writer.tab();
+        d_writer.println("char* name;");
+        d_writer.println("int   isStatic;");
+        d_writer.backTab();
+        d_writer.println("} " + IOR.getMethodDescDataName(id) + "[] = {");
+        d_writer.tab();
+        for (Iterator i = methods.iterator(); i.hasNext(); ) {
+           Method method = (Method) i.next();
+           String val    = method.isStatic() ? "1" : "0";
+           d_writer.println("{\"" + method.getLongMethodName() + "\", " + val 
+             + "},");
+        }
+        d_writer.backTab();
+        d_writer.println("};");
+        d_writer.println();
+     }
+   }
+
    /**
     * Generate the EPV (entry point vector) data structure for the methods
     * in the interface or class.  If the flag is true, then generate an EPV
@@ -710,8 +833,9 @@ public class IORHeader {
     * The EPV must be ordered correctly!  The parent entries must appear
     * before child entries.  This is required for runtime features.
     */
-   private void generateEPV(Extendable ext, boolean do_static)
-         throws CodeGenerationException {
+   private void generateEPV(Extendable ext, boolean doStatic)
+     throws CodeGenerationException 
+   {
       /*
        * Generate the name for this entry point vector as well as a pointer
        * to "self" for this structure.  For classes, self will be the object
@@ -719,40 +843,39 @@ public class IORHeader {
        */
       final SymbolID id = ext.getSymbolID();
       String self = null;
-      if (ext.isInterface()) {
-         self = "void* self";
-      } else {
-         self = IOR.getObjectName(id) + "* self";
+      if (!doStatic) {
+        if (ext.isInterface()) {
+           self = "void* self";
+        } else {
+           self = IOR.getObjectName(id) + "* self";
+        }
       }
 
       /*
        * Write a comment block to document the EPV structure.
        */
-      String comment = null;
-      if (do_static) {
-         comment = "Declare the static entry point vector.";
-      } else {
-         comment = "Declare the method entry point vector.";
-      }
-      d_writer.writeComment(comment, false);
+      String desc = doStatic ? "static " : "";
+      d_writer.writeComment("Declare the " + desc 
+        + "method entry point vector.", false);
 
       /*
        * Begin definition of the EPV structure and increase tab indent.
        */
-      if (do_static) {
+      if (doStatic) {
          d_writer.print(IOR.getSEPVName(id));
       } else {
          d_writer.print(IOR.getEPVName(id));
       }
       d_writer.println(" {");
-      d_writer.increaseTabLevel();
+      d_writer.tab();
       Set methodsAlreadySeen = new HashSet();
       Set extAlreadySeen = new HashSet();
-      generateBuiltinEntries(ext, self, do_static, methodsAlreadySeen);
-      generateEPVEntries(ext, self, do_static, methodsAlreadySeen, extAlreadySeen);
+      generateBuiltinEntries(ext, self, doStatic, methodsAlreadySeen);
+      generateEPVEntries(ext, self, doStatic, IOR.supportInterceptors(ext),
+                         methodsAlreadySeen, extAlreadySeen);
       methodsAlreadySeen = null;
       extAlreadySeen = null;
-      d_writer.decreaseTabLevel();
+      d_writer.backTab();
       d_writer.println("};");
       d_writer.println();
    }
@@ -764,26 +887,78 @@ public class IORHeader {
     */
    private void generateInterfaceObject(Interface ifc) {
       SymbolID id = ifc.getSymbolID();
-      String epv  = IOR.getEPVName(id);
       String type = IOR.getObjectName(id);
 
       d_writer.writeComment("Define the interface object structure.", false);
 
-      int width = epv.length() + 2;
-
       d_writer.println(type + " {");
-      d_writer.increaseTabLevel();
+      d_writer.tab();
 
-      d_writer.println(epv + "* d_epv;");
+      String epv   = IOR.getEPVName(id);
+      int    width = epv.length() + 2;
+
+      d_writer.printAligned(epv + "* ", width);
+      d_writer.println(IOR.getEPVVar(IOR.PUBLIC_EPV) + ";");
 
       d_writer.printAligned("void*", width);
       d_writer.println("d_object;");
 
-      d_writer.decreaseTabLevel();
+      d_writer.backTab();
       d_writer.println("};");
       d_writer.println();
    }
 
+   /**
+    * Generate the different versions of the assertion controls data structure.
+    */
+   private void generateControls(Extendable ext) {
+      boolean doAsserts  = IOR.supportAssertions(ext);
+      boolean doIceptors = IOR.supportInterceptors(ext);
+
+      if (doAsserts || doIceptors) {
+         d_writer.writeComment("Define the controls structure.", false);
+
+         SymbolID id  = ext.getSymbolID();
+         int      num = ext.getNumberOfMethods(true);
+         d_writer.println();
+         d_writer.println(IOR.getControlsStruct(id) + " {");
+         d_writer.tab();
+         if (doAsserts) {
+           d_writer.println("int32_t " + IOR.D_CHECKS + ";");
+           d_writer.println("double  " + IOR.D_RATE + ";");
+           d_writer.println(IOR.getMethodControlsStruct(id) + " {");
+           d_writer.printlnUnformatted("#ifdef " + IOR.S_FULL_STATS_MACRO);
+           d_writer.tab();
+           d_writer.println("int32_t " + IOR.D_CALLS + ";");
+           d_writer.println("int32_t " + IOR.D_COUNTDOWN + ";");
+           d_writer.println("int32_t " + IOR.D_METHOD_EXCEPT + ";");
+           d_writer.println("int32_t " + IOR.D_SUCCESSES + ";");
+           d_writer.println("int32_t " + IOR.D_TRIES + ";");
+           d_writer.printlnUnformatted("#else /* !" + IOR.S_FULL_STATS_MACRO 
+             + " */");
+           d_writer.println("int32_t " + IOR.D_COUNTDOWN + ";");
+           d_writer.printlnUnformatted("#endif /* " + IOR.S_FULL_STATS_MACRO 
+             + " */");
+           d_writer.backTab();
+           d_writer.println("} " + IOR.D_METHOD_CONTROLS + "[" + num + "];");
+         }
+         if (doIceptors) {
+           d_writer.println("int     " + IOR.D_INTERCEPTORS + ";");
+         }
+         d_writer.backTab();
+         d_writer.println("};");
+      }
+   }
+
+   /**
+    * Generate a version of the structure based on whether or not all of
+    * the statistics are to be retained.
+    */
+   private void generateControlsStats(SymbolID id, int num, boolean doAsserts,
+                                      boolean doIceptors, boolean all) 
+   {
+   }
+
    /**
     * Generate the object data structure for a class.  This structure
     * starts with the parent object (if not null) followed by new interfaces
@@ -818,40 +993,58 @@ public class IORHeader {
       /*
        * Extract information about the EPV structure and its width.
        */
-      SymbolID id = cls.getSymbolID();
-      String epv = IOR.getEPVName(id) + "*";
-      if (epv.length() > width) {
-         width = epv.length();
+      boolean addAsserts  = IOR.supportAssertions(cls);
+      boolean addIceptors = IOR.supportInterceptors(cls);
+      String  ctrls       = null;
+
+      SymbolID id  = cls.getSymbolID();
+      String   epv = IOR.getEPVName(id) + "*";
+      int      len = epv.length();
+      if (len > width) { width = len; }
+      if (addAsserts || addIceptors) {
+        ctrls = IOR.getControlsStruct(id);
+        len   = ctrls.length();
+        if (len > width) { width = len; }
       }
+      width = width + 1;
 
       /*
        * Output the object data structure.  Start with the parent class
        * (if it exists) followed by the new interface objects.
        */
       d_writer.println(IOR.getObjectName(id) + " {");
-      d_writer.increaseTabLevel();
+      d_writer.tab();
 
       if (parent != null) {
          d_writer.printAligned(ptype, width);
-         d_writer.println(" d_" + IOR.getSymbolName(pid).toLowerCase() + ";");
+         d_writer.println("d_" + IOR.getSymbolName(pid).toLowerCase() + ";");
       }
 
       for (Iterator i = unique.iterator(); i.hasNext(); ) {
          SymbolID sid = (SymbolID) i.next();
          d_writer.printAligned(IOR.getObjectName(sid), width);
-         d_writer.println(" d_" + IOR.getSymbolName(sid).toLowerCase() + ";");
+         d_writer.println("d_" + IOR.getSymbolName(sid).toLowerCase() + ";");
       }
 
       /*
        * Fill in the remainder of the data members in the structure.
        */
       d_writer.printAligned(epv, width);
-      d_writer.println(" d_epv;");
-      
-      d_writer.printAligned("void*", width);
-      d_writer.println(" d_data;");
+      d_writer.println(IOR.getEPVVar(IOR.PUBLIC_EPV) + ";");
+      if (IOR.supportBaseEPVAttr(cls)) {
+        d_writer.printAligned(epv, width);
+        d_writer.println(IOR.getEPVVar(IOR.BASE_EPV) + ";");
+      }
 
-      d_writer.decreaseTabLevel();
+      if (addAsserts || addIceptors) {
+        d_writer.printAligned(ctrls, width);
+        d_writer.println(IOR.D_CONTROLS + ";");
+      }
+
+      d_writer.printAligned("void*", width);
+      d_writer.println(IOR.D_DATA + ";");
+
+      d_writer.backTab();
       d_writer.println("};");
       d_writer.println();
    }
