@@ -1,9 +1,8 @@
 //
 // File:        Python.java
 // Package:     gov.llnl.babel.backend.python
-// Release:     $Name: release-0-8-8 $
-// Revision:    @(#) $Revision: 1.34 $
-// Date:        $Date: 2003/09/29 20:43:10 $
+// Revision:    @(#) $Revision: 4434 $
+// Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
 // Description: Python naming conventions shared by Python code generators
 // 
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -32,29 +31,24 @@
 
 package gov.llnl.babel.backend.python;
 
+import gov.llnl.babel.BabelConfiguration;
 import gov.llnl.babel.backend.CodeConstants;
 import gov.llnl.babel.backend.CodeGenerationException;
 import gov.llnl.babel.backend.CodeSplicer;
 import gov.llnl.babel.backend.FileManager;
 import gov.llnl.babel.backend.IOR;
-import gov.llnl.babel.backend.writers.LanguageWriter;
 import gov.llnl.babel.backend.writers.LanguageWriterForC;
 import gov.llnl.babel.backend.writers.LanguageWriterForPython;
 import gov.llnl.babel.symbols.Comment;
-import gov.llnl.babel.symbols.Argument;
 import gov.llnl.babel.symbols.Method;
 import gov.llnl.babel.symbols.Symbol;
 import gov.llnl.babel.symbols.SymbolID;
 import gov.llnl.babel.symbols.Type;
-import java.io.File;
-import java.io.IOException;
-import java.io.PrintWriter;
-import java.util.Iterator;
 
 /**
  * <p>
  * This class defines some of the fundamental mapping rules for translating
- * a symbol in the SIDL file into Python. These fundamental mapping rules
+ * a symbol in the sidl file into Python. These fundamental mapping rules
  * are used repeatedly in the generation of client and server side bindings,
  * so they are seperated into a class that can be shared by all Python code
  * generators.
@@ -361,7 +355,7 @@ public class Python {
   }
 
    /**
-    * Convert a SIDL symbol into the name of its associated set EPV
+    * Convert a sidl symbol into the name of its associated set EPV
     * method, which is the symbol name appended with "__set_epv".
     */
    public static String getSetEPVName(SymbolID id) {
@@ -369,13 +363,17 @@ public class Python {
    }
 
    /**
-    * Convert a SIDL symbol into the name of its associated set static
+    * Convert a sidl symbol into the name of its associated set static
     * EPV method, which is the symbol name appended with "__set_sepv".
     */
    public static String getSetSEPVName(SymbolID id) {
       return IOR.getSymbolName(id) + "__impl_set_sepv";
    }
 
+  public static String getImport(String className) {
+    return className.replace('.', '_')+"__import";
+  }
+
   public static String getExtendableImport(Symbol symbol) {
     String symbolName = symbol.getFullName().replace('.', '_');
     StringBuffer buf = new StringBuffer(symbolName.length() + 8);
@@ -424,6 +422,14 @@ public class Python {
     return buf.toString();
   }
 
+  public static String getExtendableType(Symbol symbol) {
+    String symbolName = symbol.getFullName();
+    StringBuffer buf = new StringBuffer(symbolName.length() + 7);
+    buf.append(symbolName.replace('.','_'))
+      .append("_PyType");
+    return buf.toString();
+  }
+
   public static String getExtendableAddRef(Symbol symbol) {
     String symbolName = symbol.getFullName();
     StringBuffer buf = new StringBuffer(symbolName.length() + 7);
@@ -437,58 +443,80 @@ public class Python {
   }
 
   public static String getBorrowArrayFromPython(Type arrayType) {
-    if (arrayType.getType() != Type.SYMBOL) {
-      return "SIDL_" + arrayType.getTypeString() + "__borrow_python_array";
+    if (arrayType == null) {
+      return "sidl_generic_borrow_python_array";
     }
     else {
-      return arrayType.getSymbolID().getFullName().replace('.', '_')
-        + "__convert_python_array";
+      if (arrayType.getType() != Type.SYMBOL) {
+        return "sidl_" + arrayType.getTypeString() + "__borrow_python_array";
+      }
+      else {
+        return arrayType.getSymbolID().getFullName().replace('.', '_')
+          + "__convert_python_array";
+      }
     }
   }
 
   public static String getBorrowArrayFromSIDL(Type arrayType) {
-    if (arrayType.getType() != Type.SYMBOL) {
-      return "SIDL_" + arrayType.getTypeString() + "__python_borrow_array";
+    if (arrayType == null) {
+      return BabelConfiguration.getBaseInterface().replace('.', '_')
+        + "__convert_generic_array";
     }
     else {
-      return arrayType.getSymbolID().getFullName().replace('.', '_')
-        + "__convert_sidl_array";
+      if (arrayType.getType() != Type.SYMBOL) {
+        return "sidl_python_borrow_array";
+      }
+      else {
+        return arrayType.getSymbolID().getFullName().replace('.', '_')
+          + "__convert_sidl_array";
+      }
     }
   }
 
   public static String getCopyArrayFromPython(Type arrayType) {
     final Type elemType = arrayType.getArrayType();
-    if (elemType.getType() != Type.SYMBOL) {
-      if (arrayType.getArrayOrder() == Type.COLUMN_MAJOR) {
-        return "SIDL_" + elemType.getTypeString() + 
-          "__clone_python_array_column";
-      }
-      return "SIDL_" + elemType.getTypeString() + 
-        "__clone_python_array_row";
+    if (elemType == null) {
+      return "sidl_generic_borrow_python_array";
     }
     else {
-      return elemType.getSymbolID().getFullName().replace('.', '_')
-        + "__convert_python_array";
+      if (elemType.getType() != Type.SYMBOL) {
+        if (arrayType.getArrayOrder() == Type.COLUMN_MAJOR) {
+          return "sidl_" + elemType.getTypeString() + 
+            "__clone_python_array_column";
+        }
+        else if (arrayType.getArrayOrder() == Type.ROW_MAJOR) {
+          return "sidl_" + elemType.getTypeString() + 
+            "__clone_python_array_row";
+        }
+        else {
+          return "sidl_" + elemType.getTypeString() + 
+            "__borrow_python_array";
+        }
+      }
+      else {
+        return elemType.getSymbolID().getFullName().replace('.', '_')
+          + "__convert_python_array";
+      }
     }
   }
 
   public static String getCopyArrayFromSIDL(Type arrayType) {
-    if (arrayType.getType() != Type.SYMBOL) {
-      return "SIDL_" + arrayType.getTypeString() + "__python_clone_array";
+    if (arrayType == null) {
+      return BabelConfiguration.getBaseInterface().replace('.', '_')
+        + "__convert_generic_array";
     }
     else {
-      return arrayType.getSymbolID().getFullName().replace('.', '_')
-        + "__convert_sidl_array";
+      if (arrayType.getType() != Type.SYMBOL) {
+        return "sidl_python_clone_array";
+      }
+      else {
+        return arrayType.getSymbolID().getFullName().replace('.', '_')
+          + "__convert_sidl_array";
+      }
     }
   }
 
   public static String getDestroyArray(Type arrayType) {
-    if (arrayType.getType() != Type.SYMBOL) {
-      return "SIDL_" + arrayType.getTypeString() + "__python_deleteRef_array";
-    }
-    else {
-      return arrayType.getSymbolID().getFullName().replace('.', '_')
-        + "__destroy_sidl_array";
-    }
+    return "sidl_python_deleteRef_array";
   }
 }
