@@ -1,8 +1,7 @@
 //
 // File:        PrettyWriter.java
 // Package:     gov.llnl.babel.backend.writers
-// Release:     $Name: release-0-8-8 $
-// Revision:    @(#) $Id: PrettyWriter.java,v 1.9 2002/08/20 16:55:31 epperly Exp $
+// Revision:    @(#) $Id: PrettyWriter.java 4434 2005-03-17 17:05:29Z epperly $
 // Description: a pretty writer class to aid in formatting backend output
 //
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -73,6 +72,8 @@ public class PrettyWriter {
    private int          d_tab;               // tab spacing between tab stops
    private int          d_tmp_first_tab;     // temporary tab position
    private PrintWriter  d_writer;            // output print writer
+  private boolean      d_strict_break = false; // No lines can be longer
+                                               // than the break column.
 
    /**
     * Create a new <code>PrettyWriter</code> instance that will send
@@ -159,6 +160,11 @@ public class PrettyWriter {
       d_break_enabled = false;
    }
 
+  public void setStrictBreaking()
+  {
+    d_strict_break = true;
+  }
+
    /**
     * Set the line breaking characters.  Lines will be broken using these
     * characters as separators.  Each character will be tested in order.
@@ -219,7 +225,7 @@ public class PrettyWriter {
     * Increase the tab level by one so that newlines begin at the next
     * tab stop.
     */
-   public void increaseTabLevel() {
+   public void tab() {
       changeTabLevel(1);
    }
 
@@ -227,7 +233,7 @@ public class PrettyWriter {
     * Decrease the tab level by one so that newlines begin at the previous
     * tab stop.
     */
-   public void decreaseTabLevel() {
+   public void backTab() {
       changeTabLevel(-1);
    }
 
@@ -257,27 +263,6 @@ public class PrettyWriter {
       d_tmp_first_tab = d_first_tab;
    }
 
-   /**
-    * Advance the specified number of tab stops.  If the argument is zero
-    * or negative, then no action is taken.
-    */
-   public void tab(int nstops) {
-      if (nstops > 0) {
-         flushLine();
-         int to_next_tab = d_tab - ((d_cursor-d_first_tab) % d_tab);
-         for (int i = 0; i < to_next_tab; i++) {
-            d_line.append(' ');
-         }
-      }
-   }
-
-   /**
-    * Advance to the next tab stop.
-    */
-   public void tab() {
-      tab(1);
-   }
-
    /**
     * Print the specified string to the print writer output stream.
     * Nothing is done if the string argument is null.  If the string
@@ -423,6 +408,7 @@ public class PrettyWriter {
        * there does not seem to be a good way to re-use the current string
        * buffer.
        */
+
       d_line = new StringBuffer(BUFSIZE);
    }
 
@@ -479,9 +465,18 @@ public class PrettyWriter {
              */
             } else {
                if (force) {
-                  String fragment = d_line.substring(start);
-                  d_writer.print(fragment);
-                  d_cursor += fragment.length();
+                 if (d_strict_break && (search > 0)) {
+                   String fragment = d_line.substring(start, start+search);
+                   d_writer.print(fragment);
+                   endContinuationLine();
+                   beginContinuationLine();
+                   breakLine(firstNonSpace(start+search), true);
+                 }
+                 else{
+                   String fragment = d_line.substring(start);
+                   d_writer.print(fragment);
+                   d_cursor += fragment.length();
+                 }
                } else {
                   endContinuationLine();
                   beginContinuationLine();
