@@ -1,8 +1,7 @@
 //
 // File:        Cookie.java
 // Package:     gov.llnl.babel.url.cookie
-// Release:     $Name: release-0-8-8 $
-// Revision:    @(#) $Id: Cookie.java,v 1.2 2002/08/20 16:55:32 epperly Exp $
+// Revision:    @(#) $Id: Cookie.java 4434 2005-03-17 17:05:29Z epperly $
 // Description: A class to store a HTTP Cookie for a User Agent
 // 
 // Copyright (c) 2000-2001, The Regents of the University of Calfornia.
@@ -31,10 +30,7 @@
 
 package gov.llnl.babel.url.cookie;
 
-import gov.llnl.babel.url.cookie.CookieCrumbleException;
-import gov.llnl.babel.url.cookie.Lexer;
 import java.io.Serializable;
-import java.net.URL;
 import java.util.Date;
 import java.util.StringTokenizer;
 
