@@ -2,13 +2,12 @@
 ## File:        runFortran.sh
 ## Package:     Babel cca hello examples
 ## Copyright:   (c) 2000-2001 The Regents of the University of California
-## Release:     $Name: release-0-8-8 $
-## Revision:    $Revision: 1.1 $
-## Modified:    $Date: 2001/09/19 18:09:52 $
+## Revision:    $Revision: 4434 $
+## Modified:    $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
 ## Description: script to run F77 dynamically loading CCA hello implementations
 ##
 
-SIDL_DLL_PATH="libDriver.la;../libdecaf/libdecaf.la;../hello-server/libHelloServer.la;../hello-client/libHelloClient.la"
+SIDL_DLL_PATH="libDriver.scl;../libdecaf/libdecaf.scl;../hello-server/libHelloServer.scl;../hello-client/libHelloClient.scl"
 export SIDL_DLL_PATH
 
 ./HelloDriver
