@@ -1,15 +1,14 @@
 /*
  * File:        helloclient.c
  * Copyright:   (c) 2001 The Regents of the University of California
- * Release:     $Name: release-0-8-8 $
- * Revision:    @(#) $Revision: 1.3 $
- * Date:        $Date: 2002/09/30 23:00:59 $
+ * Revision:    @(#) $Revision: 4434 $
+ * Date:        $Date: 2005-03-17 09:05:29 -0800 (Thu, 17 Mar 2005) $
  * Description: Simple Hello World C client
  *
  */
 
 #include "Hello_World.h"
-#include "SIDL_header.h"
+#include "sidl_header.h"
 #include <stdio.h>
 #include <stdlib.h>
 
