@@ -1,32 +1,30 @@
 /*
  * File:        arraytests.c
  * Copyright:   (c) 2001-2003 The Regents of the University of California
- * Release:     $Name: release-0-8-8 $
- * Revision:    @(#) $Revision: 1.15 $
- * Date:        $Date: 2003/09/05 23:12:54 $
+ * Revision:    @(#) $Revision: 4453 $
+ * Date:        $Date: 2005-03-21 14:30:45 -0800 (Mon, 21 Mar 2005) $
  * Description: Simple test on the ArrayTest static methods
  *
  */
 
 #include "ArrayTest_ArrayOps.h"
-#include "SIDL_header.h"
+#include "sidl_header.h"
 #include <stdlib.h>
+#include <stddef.h>
+#include <time.h>
 #include <string.h>
 #include <stdio.h>
-
-static const char * result_string[] = { "FAIL", "XFAIL", "PASS" };
-const int pass = 2;
-const int xfail = 1;
-const int fail = 0;
+#include "synch.h"
 
 static const int TEST_SIZE = 345; /* size of one dimensional arrays */
 static const int TEST_DIM1 = 17; /* first dimension of 2-d arrays */
 static const int TEST_DIM2 = 13; /* second dimension of 2-d arrays */
 static const int TEST_DIM3 = 4;  /* third dimension of 3-d arrays */
 
-static void declare_part( int * part_no ) {
-  ++(*part_no);
-  printf("PART %d\n", *part_no );
+static void declare_part(synch_RegOut tracker,
+                         int * part_no )
+{
+  synch_RegOut_startPart(tracker,  ++(*part_no));
 }
 
 static int isPrime(const int64_t num) {
@@ -53,11 +51,11 @@ nextPrime(int64_t prev) {
   }
 }
 
-static void end_part( int part_no, int result, int *final_result ) {
-  printf("RESULT %d %s\n", part_no, result_string[ result ] );
-  if ( result < *final_result ) { 
-    *final_result = result;
-  }
+static void end_part(synch_RegOut tracker,
+                     int part_no,
+                     enum synch_ResultType__enum result) 
+{
+  synch_RegOut_endPart(tracker, part_no, result);
 }
 
 /**
@@ -75,61 +73,78 @@ int clearstack(int magicNumber) {
 }
 
 #define MYASSERT( CMT, AAA ) \
-  declare_part( &part_no ); \
+  declare_part( tracker,  &part_no ); \
   magicNumber = clearstack(magicNumber); \
-  printf("COMMENT: %s: %s\n", #CMT, #AAA ); \
-  if ( AAA ) result = pass; \
-  else result = fail;  \
-  end_part( part_no, result, &final_result );
+  synch_RegOut_writeComment(tracker,  #CMT ": " #AAA); \
+  if ( AAA ) result = synch_ResultType_PASS; \
+  else result = synch_ResultType_FAIL;  \
+  end_part( tracker, part_no, result);
 
 int main(int argc, char **argv) {
-  int result = pass;
-  int final_result = pass;
+  enum synch_ResultType__enum result = synch_ResultType_PASS;
   int part_no = 0;
   int magicNumber = 1;
+  synch_RegOut tracker = synch_RegOut__create();
 
-  struct SIDL_bool__array     *barray;
-  struct SIDL_char__array     *carray;
-  struct SIDL_int__array      *iarray;
-  struct SIDL_dcomplex__array *dcarray;
-  struct SIDL_fcomplex__array *fcarray;
-  struct SIDL_long__array     *larray;
-  struct SIDL_double__array   *darray;
-  struct SIDL_float__array    *farray;
-  struct SIDL_string__array   *sarray;
+  struct sidl_bool__array     *barray;
+  struct sidl_char__array     *carray;
+  struct sidl_int__array      *iarray;
+  struct sidl_dcomplex__array *dcarray;
+  struct sidl_fcomplex__array *fcarray;
+  struct sidl_long__array     *larray;
+  struct sidl_double__array   *darray;
+  struct sidl_float__array    *farray;
+  struct sidl_string__array   *sarray;
+  struct sidl__array          *garray;
 
   struct ArrayTest_ArrayOps__array *oarray;
+  srand(time(NULL));
 
-  printf("NPARTS -1\n");
+  synch_RegOut_setExpectations(tracker, 159);
+
+  barray = NULL;
+  carray = NULL;
+  iarray = NULL;
+  dcarray = NULL;
+  fcarray = NULL;
+  larray = NULL;
+  darray = NULL;
+  sarray = NULL;
+  farray = NULL;
+  garray = NULL;
 
   magicNumber = clearstack(magicNumber);
   barray = ArrayTest_ArrayOps_createBool(TEST_SIZE);
   MYASSERT(createBool, barray);
   MYASSERT(createBool, ArrayTest_ArrayOps_checkBool(barray) == TRUE);
-  MYASSERT(createBool, ArrayTest_ArrayOps_reverseBool(&barray, TRUE) == TRUE);
+   MYASSERT(createBool, ArrayTest_ArrayOps_reverseBool(&barray, TRUE) == TRUE);
   magicNumber = clearstack(magicNumber);
-  SIDL_bool__array_deleteRef(barray);
-  barray = NULL;
+  sidl_bool__array_deleteRef(barray);
+  /* initialize out array reference to a random value to check that it can
+     be arbitrary */
+  barray = (struct sidl_bool__array *)(ptrdiff_t)rand();
+
   ArrayTest_ArrayOps_makeBool(218, &barray);
   MYASSERT(makeBool218, ArrayTest_ArrayOps_checkBool(barray) == TRUE);
   MYASSERT(makeBool218, 
-           ArrayTest_ArrayOps_reverseBool(&barray, FALSE) == TRUE);
+            ArrayTest_ArrayOps_reverseBool(&barray, FALSE) == TRUE);
   MYASSERT(makeBool218, ArrayTest_ArrayOps_checkBool(barray) == FALSE);
   magicNumber = clearstack(magicNumber);
-  SIDL_bool__array_deleteRef(barray);
-  barray = NULL;
+  
+  sidl_bool__array_deleteRef(barray);
+  barray = (struct sidl_bool__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeBool(9, &barray);
   MYASSERT(makeBool9, ArrayTest_ArrayOps_reverseBool(&barray, FALSE) == TRUE);
   MYASSERT(makeBool9, ArrayTest_ArrayOps_checkBool(barray) == TRUE);
   magicNumber = clearstack(magicNumber);
-  SIDL_bool__array_deleteRef(barray);
-  barray = NULL;
-
+  sidl_bool__array_deleteRef(barray);
+  
+  barray = (struct sidl_bool__array *)(ptrdiff_t)rand();
   MYASSERT(createBoolNegOne, ArrayTest_ArrayOps_createBool(-1) == NULL);
   ArrayTest_ArrayOps_makeBool(-1, &barray);
   MYASSERT(makeBoolNegOne, barray == NULL);
   if (barray) {
-    SIDL_bool__array_deleteRef(barray);
+    sidl_bool__array_deleteRef(barray);
     barray = NULL;
   }
   
@@ -139,21 +154,22 @@ int main(int argc, char **argv) {
   MYASSERT(createChar, ArrayTest_ArrayOps_checkChar(carray) == TRUE);
   MYASSERT(createChar, ArrayTest_ArrayOps_reverseChar(&carray, TRUE) == TRUE);
   magicNumber = clearstack(magicNumber);
-  SIDL_char__array_deleteRef(carray);
-  carray = NULL;
+  sidl_char__array_deleteRef(carray);
+
+  carray = (struct sidl_char__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeChar(218, &carray);
   MYASSERT(makeChar, ArrayTest_ArrayOps_checkChar(carray) == TRUE);
   MYASSERT(makeChar, ArrayTest_ArrayOps_reverseChar(&carray, FALSE) == TRUE);
   MYASSERT(makeChar, ArrayTest_ArrayOps_checkChar(carray) == FALSE);
   magicNumber = clearstack(magicNumber);
-  SIDL_char__array_deleteRef(carray);
-  carray = NULL;
+  sidl_char__array_deleteRef(carray);
 
+  carray = (struct sidl_char__array *)(ptrdiff_t)rand();
   MYASSERT(createCharNegOne, ArrayTest_ArrayOps_createChar(-1) == NULL);
   ArrayTest_ArrayOps_makeChar(-1, &carray);
   MYASSERT(makeCharNegOne, carray == NULL);
   if (carray) {
-    SIDL_char__array_deleteRef(carray);
+    sidl_char__array_deleteRef(carray);
     carray = NULL;
   }
   
@@ -163,21 +179,22 @@ int main(int argc, char **argv) {
   MYASSERT(createInt, ArrayTest_ArrayOps_checkInt(iarray) == TRUE);
   MYASSERT(createInt, ArrayTest_ArrayOps_reverseInt(&iarray, TRUE) == TRUE);
   magicNumber = clearstack(magicNumber);
-  SIDL_int__array_deleteRef(iarray);
-  iarray = NULL;
+  sidl_int__array_deleteRef(iarray);
+
+  iarray = (struct sidl_int__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeInt(218, &iarray);
   MYASSERT(makeInt, ArrayTest_ArrayOps_checkInt(iarray) == TRUE);
   MYASSERT(makeInt, ArrayTest_ArrayOps_reverseInt(&iarray, FALSE) == TRUE);
   MYASSERT(makeInt, ArrayTest_ArrayOps_checkInt(iarray) == FALSE);
   magicNumber = clearstack(magicNumber);
-  SIDL_int__array_deleteRef(iarray);
-  iarray = NULL;
+  sidl_int__array_deleteRef(iarray);
 
+  iarray = (struct sidl_int__array *)(ptrdiff_t)rand();
   MYASSERT(createIntNegOne, ArrayTest_ArrayOps_createInt(-1) == NULL);
   ArrayTest_ArrayOps_makeInt(-1, &iarray);
   MYASSERT(makeIntNegOne, iarray == NULL);
   if (iarray) {
-    SIDL_int__array_deleteRef(iarray);
+    sidl_int__array_deleteRef(iarray);
     iarray = NULL;
   }
   
@@ -188,21 +205,21 @@ int main(int argc, char **argv) {
   MYASSERT(createLong, ArrayTest_ArrayOps_checkLong(larray) == TRUE);
   MYASSERT(createLong, ArrayTest_ArrayOps_reverseLong(&larray, TRUE) == TRUE);
   magicNumber = clearstack(magicNumber);
-  SIDL_long__array_deleteRef(larray);
-  larray = NULL;
+  sidl_long__array_deleteRef(larray);
+  larray = (struct sidl_long__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeLong(218, &larray);
   MYASSERT(makeLong, ArrayTest_ArrayOps_checkLong(larray) == TRUE);
   MYASSERT(makeLong, ArrayTest_ArrayOps_reverseLong(&larray, FALSE) == TRUE);
   MYASSERT(makeLong, ArrayTest_ArrayOps_checkLong(larray) == FALSE);
   magicNumber = clearstack(magicNumber);
-  SIDL_long__array_deleteRef(larray);
-  larray = NULL;
+  sidl_long__array_deleteRef(larray);
 
   MYASSERT(createLongNegOne, ArrayTest_ArrayOps_createLong(-1) == NULL);
+  larray = (struct sidl_long__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeLong(-1, &larray);
   MYASSERT(makeLongNegOne, larray == NULL);
   if (larray) {
-    SIDL_long__array_deleteRef(larray);
+    sidl_long__array_deleteRef(larray);
     larray = NULL;
   }
   
@@ -213,22 +230,22 @@ int main(int argc, char **argv) {
   MYASSERT(createString, 
            ArrayTest_ArrayOps_reverseString(&sarray, TRUE) == TRUE);
   magicNumber = clearstack(magicNumber);
-  SIDL_string__array_deleteRef(sarray);
-  sarray = NULL;
+  sidl_string__array_deleteRef(sarray);
+  sarray = (struct sidl_string__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeString(218, &sarray);
   MYASSERT(makeString, ArrayTest_ArrayOps_checkString(sarray) == TRUE);
   MYASSERT(makeString, 
            ArrayTest_ArrayOps_reverseString(&sarray, FALSE) == TRUE);
   MYASSERT(makeString, ArrayTest_ArrayOps_checkString(sarray) == FALSE);
   magicNumber = clearstack(magicNumber);
-  SIDL_string__array_deleteRef(sarray);
-  sarray = NULL;
+  sidl_string__array_deleteRef(sarray);
 
   MYASSERT(createStringNegOne, ArrayTest_ArrayOps_createString(-1) == NULL);
+  sarray = (struct sidl_string__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeString(-1, &sarray);
   MYASSERT(makeStringNegOne, sarray == NULL);
   if (sarray) {
-    SIDL_string__array_deleteRef(sarray);
+    sidl_string__array_deleteRef(sarray);
     sarray = NULL;
   }
   
@@ -239,22 +256,22 @@ int main(int argc, char **argv) {
   MYASSERT(createDouble, 
            ArrayTest_ArrayOps_reverseDouble(&darray, TRUE) == TRUE);
   magicNumber = clearstack(magicNumber);
-  SIDL_double__array_deleteRef(darray);
-  darray = NULL;
+  sidl_double__array_deleteRef(darray);
+  darray = (struct sidl_double__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeDouble(218, &darray);
   MYASSERT(makeDouble, ArrayTest_ArrayOps_checkDouble(darray) == TRUE);
   MYASSERT(makeDouble, 
            ArrayTest_ArrayOps_reverseDouble(&darray, FALSE) == TRUE);
   MYASSERT(makeDouble, ArrayTest_ArrayOps_checkDouble(darray) == FALSE);
   magicNumber = clearstack(magicNumber);
-  SIDL_double__array_deleteRef(darray);
-  darray = NULL;
+  sidl_double__array_deleteRef(darray);
 
   MYASSERT(createDoubleNegOne, ArrayTest_ArrayOps_createDouble(-1) == NULL);
+  darray = (struct sidl_double__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeDouble(-1, &darray);
   MYASSERT(makeDoubleNegOne, darray == NULL);
   if (darray) {
-    SIDL_double__array_deleteRef(darray);
+    sidl_double__array_deleteRef(darray);
     darray = NULL;
   }
   
@@ -265,25 +282,24 @@ int main(int argc, char **argv) {
   MYASSERT(createFloat, 
            ArrayTest_ArrayOps_reverseFloat(&farray, TRUE) == TRUE);
   magicNumber = clearstack(magicNumber);
-  SIDL_float__array_deleteRef(farray);
-  farray = NULL;
+  sidl_float__array_deleteRef(farray);
+  farray = (struct sidl_float__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeFloat(218, &farray);
   MYASSERT(makeFloat, ArrayTest_ArrayOps_checkFloat(farray) == TRUE);
   MYASSERT(makeFloat, 
            ArrayTest_ArrayOps_reverseFloat(&farray, FALSE) == TRUE);
   MYASSERT(makeFloat, ArrayTest_ArrayOps_checkFloat(farray) == FALSE);
   magicNumber = clearstack(magicNumber);
-  SIDL_float__array_deleteRef(farray);
-  farray = NULL;
+  sidl_float__array_deleteRef(farray);
   
   MYASSERT(createFloatNegOne, ArrayTest_ArrayOps_createFloat(-1) == NULL);
+  farray = (struct sidl_float__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeFloat(-1, &farray);
   MYASSERT(makeFloatNegOne, farray == NULL);
   if (farray) {
-    SIDL_float__array_deleteRef(farray);
+    sidl_float__array_deleteRef(farray);
     farray = NULL;
   }
-  
   magicNumber = clearstack(magicNumber);
   fcarray = ArrayTest_ArrayOps_createFcomplex(TEST_SIZE);
   MYASSERT(createFcomplex, fcarray);
@@ -291,23 +307,23 @@ int main(int argc, char **argv) {
   MYASSERT(createFcomplex, 
            ArrayTest_ArrayOps_reverseFcomplex(&fcarray, TRUE) == TRUE);
   magicNumber = clearstack(magicNumber);
-  SIDL_fcomplex__array_deleteRef(fcarray);
-  fcarray = NULL;
+  sidl_fcomplex__array_deleteRef(fcarray);
+  fcarray = (struct sidl_fcomplex__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeFcomplex(218, &fcarray);
   MYASSERT(makeFcomplex, ArrayTest_ArrayOps_checkFcomplex(fcarray) == TRUE);
   MYASSERT(makeFcomplex, 
            ArrayTest_ArrayOps_reverseFcomplex(&fcarray, FALSE) == TRUE);
   MYASSERT(makeFcomplex, ArrayTest_ArrayOps_checkFcomplex(fcarray) == FALSE);
   magicNumber = clearstack(magicNumber);
-  SIDL_fcomplex__array_deleteRef(fcarray);
-  fcarray = NULL;
+  sidl_fcomplex__array_deleteRef(fcarray);
   
   MYASSERT(createFcomplexNegOne, 
            ArrayTest_ArrayOps_createFcomplex(-1) == NULL);
+  fcarray = (struct sidl_fcomplex__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeFcomplex(-1, &fcarray);
   MYASSERT(makeFcomplexNegOne, fcarray == NULL);
   if (fcarray) {
-    SIDL_fcomplex__array_deleteRef(fcarray);
+    sidl_fcomplex__array_deleteRef(fcarray);
     fcarray = NULL;
   }
   
@@ -318,30 +334,30 @@ int main(int argc, char **argv) {
   MYASSERT(createDcomplex, 
            ArrayTest_ArrayOps_reverseDcomplex(&dcarray, TRUE) == TRUE);
   magicNumber = clearstack(magicNumber);
-  SIDL_dcomplex__array_deleteRef(dcarray);
-  dcarray = NULL;
+  sidl_dcomplex__array_deleteRef(dcarray);
+  dcarray = (struct sidl_dcomplex__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeDcomplex(218, &dcarray);
   MYASSERT(makeDcomplex, ArrayTest_ArrayOps_checkDcomplex(dcarray) == TRUE);
   MYASSERT(makeDcomplex, 
            ArrayTest_ArrayOps_reverseDcomplex(&dcarray, FALSE) == TRUE);
   MYASSERT(makeDcomplex, ArrayTest_ArrayOps_checkDcomplex(dcarray) == FALSE);
   magicNumber = clearstack(magicNumber);
-  SIDL_dcomplex__array_deleteRef(dcarray);
-  dcarray = NULL;
+  sidl_dcomplex__array_deleteRef(dcarray);
 
   MYASSERT(createDcomplexNegOne, 
            ArrayTest_ArrayOps_createDcomplex(-1) == NULL);
+  dcarray = (struct sidl_dcomplex__array *)(ptrdiff_t)rand();
   ArrayTest_ArrayOps_makeDcomplex(-1, &dcarray);
   MYASSERT(makeDcomplexNegOne, dcarray == NULL);
   if (dcarray) {
-    SIDL_dcomplex__array_deleteRef(dcarray);
+    sidl_dcomplex__array_deleteRef(dcarray);
     dcarray = NULL;
   }
   
   magicNumber = clearstack(magicNumber);
   iarray = ArrayTest_ArrayOps_create2Int(TEST_DIM1,TEST_DIM2);
   MYASSERT(create2Int, ArrayTest_ArrayOps_check2Int(iarray) == TRUE);
-  SIDL_int__array_deleteRef(iarray);
+  sidl_int__array_deleteRef(iarray);
   iarray = NULL;
 
   MYASSERT(create2IntNegOne, ArrayTest_ArrayOps_create2Int(-1,-1) == NULL);
@@ -349,7 +365,7 @@ int main(int argc, char **argv) {
   magicNumber = clearstack(magicNumber);
   darray = ArrayTest_ArrayOps_create2Double(TEST_DIM1,TEST_DIM2);
   MYASSERT(create2Double, ArrayTest_ArrayOps_check2Double(darray) == TRUE);
-  SIDL_double__array_deleteRef(darray);
+  sidl_double__array_deleteRef(darray);
   darray = NULL;
 
   MYASSERT(create2DoubleNegOne, ArrayTest_ArrayOps_create2Double(-1,-1) == NULL);
@@ -357,7 +373,7 @@ int main(int argc, char **argv) {
   magicNumber = clearstack(magicNumber);
   farray = ArrayTest_ArrayOps_create2Float(TEST_DIM1,TEST_DIM2);
   MYASSERT(create2Float, ArrayTest_ArrayOps_check2Float(farray) == TRUE);
-  SIDL_float__array_deleteRef(farray);
+  sidl_float__array_deleteRef(farray);
   farray = NULL;
 
   MYASSERT(create2FloatNegOne, ArrayTest_ArrayOps_create2Float(-1,-1) == NULL);
@@ -366,7 +382,7 @@ int main(int argc, char **argv) {
   dcarray = ArrayTest_ArrayOps_create2Dcomplex(TEST_DIM1,TEST_DIM2);
   MYASSERT(create2Dcomplex, 
            ArrayTest_ArrayOps_check2Dcomplex(dcarray) == TRUE);
-  SIDL_dcomplex__array_deleteRef(dcarray);
+  sidl_dcomplex__array_deleteRef(dcarray);
   dcarray = NULL;
 
   MYASSERT(create2DcomplexNegOne, ArrayTest_ArrayOps_create2Dcomplex(-1,-1) == NULL);
@@ -375,7 +391,7 @@ int main(int argc, char **argv) {
   fcarray = ArrayTest_ArrayOps_create2Fcomplex(TEST_DIM1,TEST_DIM2);
   MYASSERT(create2Fcomplex, 
            ArrayTest_ArrayOps_check2Fcomplex(fcarray) == TRUE);
-  SIDL_fcomplex__array_deleteRef(fcarray);
+  sidl_fcomplex__array_deleteRef(fcarray);
   fcarray = NULL;
 
   MYASSERT(create2FcomplexNegOne, ArrayTest_ArrayOps_create2Fcomplex(-1,-1) == NULL);
@@ -383,236 +399,458 @@ int main(int argc, char **argv) {
   magicNumber = clearstack(magicNumber);
   iarray = ArrayTest_ArrayOps_create3Int();
   MYASSERT(create3Int, ArrayTest_ArrayOps_check3Int(iarray) == TRUE);
-  SIDL_int__array_deleteRef(iarray);
+  sidl_int__array_deleteRef(iarray);
   iarray = NULL;
 
   magicNumber = clearstack(magicNumber);
   iarray = ArrayTest_ArrayOps_create4Int();
   MYASSERT(create4Int, ArrayTest_ArrayOps_check4Int(iarray) == TRUE);
-  SIDL_int__array_deleteRef(iarray);
+  sidl_int__array_deleteRef(iarray);
   iarray = NULL;
 
+  magicNumber = clearstack(magicNumber);
+  iarray = ArrayTest_ArrayOps_create5Int();
+  MYASSERT(create5Int, ArrayTest_ArrayOps_check5Int(iarray) == TRUE);
+  sidl_int__array_deleteRef(iarray);
+  iarray = NULL;
+
+  magicNumber = clearstack(magicNumber);
+  iarray = ArrayTest_ArrayOps_create6Int();
+  MYASSERT(create6Int, ArrayTest_ArrayOps_check6Int(iarray) == TRUE);
+  sidl_int__array_deleteRef(iarray);
+  iarray = NULL;
+
+  magicNumber = clearstack(magicNumber);
+  iarray = ArrayTest_ArrayOps_create7Int();
+  MYASSERT(create7Int, ArrayTest_ArrayOps_check7Int(iarray) == TRUE);
+  sidl_int__array_deleteRef(iarray);
+  iarray = NULL;
+
+  barray = NULL;
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutBool(&barray, 218);
   MYASSERT(makeInOutBool, ArrayTest_ArrayOps_checkBool(barray) == TRUE);
-  SIDL_bool__array_deleteRef(barray);
+  sidl_bool__array_deleteRef(barray);
   barray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutBool(&barray, -1);
   MYASSERT(makeInOutBoolNegOne, barray == NULL);
   if (barray) { 
-    SIDL_bool__array_deleteRef(barray);
+    sidl_bool__array_deleteRef(barray);
     barray = NULL;
   }
 
   magicNumber = clearstack(magicNumber);
+  carray = NULL;
   ArrayTest_ArrayOps_makeInOutChar(&carray, 218);
   MYASSERT(makeInOutChar, ArrayTest_ArrayOps_checkChar(carray) == TRUE);
-  SIDL_char__array_deleteRef(carray);
+  sidl_char__array_deleteRef(carray);
   carray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutChar(&carray, -1);
   MYASSERT(makeInOutCharNegOne, carray == NULL);
   if (carray) {
-    SIDL_char__array_deleteRef(carray);
+    sidl_char__array_deleteRef(carray);
     carray = NULL;
   }
 
   magicNumber = clearstack(magicNumber);
+  iarray = NULL;
   ArrayTest_ArrayOps_makeInOutInt(&iarray, 218);
   MYASSERT(makeInOutInt, ArrayTest_ArrayOps_checkInt(iarray) == TRUE);
-  SIDL_int__array_deleteRef(iarray);
+  sidl_int__array_deleteRef(iarray);
   iarray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutInt(&iarray, -1);
   MYASSERT(makeInOutIntNegOne, iarray == NULL);
   if (iarray) {
-    SIDL_int__array_deleteRef(iarray);
+    sidl_int__array_deleteRef(iarray);
     iarray = NULL;
   }
 
+  larray = NULL;
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutLong(&larray, 218);
   MYASSERT(makeInOutLong, ArrayTest_ArrayOps_checkLong(larray) == TRUE);
-  SIDL_long__array_deleteRef(larray);
+  sidl_long__array_deleteRef(larray);
   larray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutLong(&larray, -1);
   MYASSERT(makeInOutLongNegOne, larray == NULL);
   if (larray) {
-    SIDL_long__array_deleteRef(larray);
+    sidl_long__array_deleteRef(larray);
     larray = NULL;
   }
 
   magicNumber = clearstack(magicNumber);
+  sarray = NULL;
   ArrayTest_ArrayOps_makeInOutString(&sarray, 218);
   MYASSERT(makeInOutString, ArrayTest_ArrayOps_checkString(sarray) == TRUE);
-  SIDL_string__array_deleteRef(sarray);
+  sidl_string__array_deleteRef(sarray);
   sarray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutString(&sarray, -1);
   MYASSERT(makeInOutStringNegOne, sarray == NULL);
   if (sarray) {
-    SIDL_string__array_deleteRef(sarray);
+    sidl_string__array_deleteRef(sarray);
     sarray = NULL;
   }
-
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutDouble(&darray, 218);
   MYASSERT(makeInOutDouble, ArrayTest_ArrayOps_checkDouble(darray) == TRUE);
-  SIDL_double__array_deleteRef(darray);
+  sidl_double__array_deleteRef(darray);
   darray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutDouble(&darray, -1);
   MYASSERT(makeInOutDoubleNegOne, darray == NULL);
   if (darray) {
-    SIDL_double__array_deleteRef(darray);
+    sidl_double__array_deleteRef(darray);
     darray = NULL;
   }
 
+  farray = NULL;
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutFloat(&farray, 218);
   MYASSERT(makeInOutFloat, ArrayTest_ArrayOps_checkFloat(farray) == TRUE);
-  SIDL_float__array_deleteRef(farray);
+  sidl_float__array_deleteRef(farray);
   farray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutFloat(&farray, -1);
   MYASSERT(makeInOutFloatNegOne, farray == NULL);
   if (farray) {
-    SIDL_float__array_deleteRef(farray);
+    sidl_float__array_deleteRef(farray);
     farray = NULL;
   }
-
+  
+  dcarray = NULL;
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutDcomplex(&dcarray, 218);
   MYASSERT(makeInOutDcomplex, 
            ArrayTest_ArrayOps_checkDcomplex(dcarray) == TRUE);
-  SIDL_dcomplex__array_deleteRef(dcarray);
+  sidl_dcomplex__array_deleteRef(dcarray);
   dcarray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutDcomplex(&dcarray, -1);
   MYASSERT(makeInOutDcomplexNegOne, dcarray == NULL);
   if (dcarray) {
-    SIDL_dcomplex__array_deleteRef(dcarray);
+    sidl_dcomplex__array_deleteRef(dcarray);
     dcarray = NULL;
   }
 
+  fcarray = NULL;
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutFcomplex(&fcarray, 218);
   MYASSERT(makeInOutFcomplex, 
            ArrayTest_ArrayOps_checkFcomplex(fcarray) == TRUE);
-  SIDL_fcomplex__array_deleteRef(fcarray);
+  sidl_fcomplex__array_deleteRef(fcarray);
   fcarray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOutFcomplex(&fcarray, -1);
   MYASSERT(makeInOutFcomplexNegOne,  fcarray == NULL);
   if (fcarray) {
-    SIDL_fcomplex__array_deleteRef(fcarray);
+    sidl_fcomplex__array_deleteRef(fcarray);
     fcarray = NULL;
   }
 
+  iarray = NULL;
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut2Int(&iarray, TEST_DIM1, TEST_DIM2);
   MYASSERT(makeInOut2Int, ArrayTest_ArrayOps_check2Int(iarray) == TRUE);
-  SIDL_int__array_deleteRef(iarray);
+  sidl_int__array_deleteRef(iarray);
   iarray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut2Int(&iarray, -1, -1);
   MYASSERT(makeInOut2IntNegOne, iarray == NULL);
   if (iarray) {
-    SIDL_int__array_deleteRef(iarray);
+    sidl_int__array_deleteRef(iarray);
     iarray = NULL;
   }
 
+  darray = NULL;
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut2Double(&darray, TEST_DIM1, TEST_DIM2);
   MYASSERT(makeInOut2Double, ArrayTest_ArrayOps_check2Double(darray) == TRUE);
-  SIDL_double__array_deleteRef(darray);
+  sidl_double__array_deleteRef(darray);
   darray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut2Double(&darray, -1, -1);
   MYASSERT(makeInOut2DoubleNegOne, darray == NULL);
   if (darray) {
-    SIDL_double__array_deleteRef(darray);
+    sidl_double__array_deleteRef(darray);
     darray = NULL;
   }
 
+  farray = NULL;
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut2Float(&farray, TEST_DIM1, TEST_DIM2);
   MYASSERT(makeInOut2Float, ArrayTest_ArrayOps_check2Float(farray) == TRUE);
-  SIDL_float__array_deleteRef(farray);
+  sidl_float__array_deleteRef(farray);
   farray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut2Float(&farray, -1, -1);
   MYASSERT(makeInOut2FloatNegOne, farray == NULL);
   if (farray) {
-    SIDL_float__array_deleteRef(farray);
+    sidl_float__array_deleteRef(farray);
     farray = NULL;
   }
 
+  dcarray = NULL;
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut2Dcomplex(&dcarray, TEST_DIM1, TEST_DIM2);
   MYASSERT(makeInOut2Dcomplex, 
             ArrayTest_ArrayOps_check2Dcomplex(dcarray) == TRUE);
-  SIDL_dcomplex__array_deleteRef(dcarray);
+  sidl_dcomplex__array_deleteRef(dcarray);
   dcarray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut2Dcomplex(&dcarray, -1, -1);
   MYASSERT(makeInOut2DcomplexNegOne, dcarray == NULL);
   if (dcarray) {
-    SIDL_dcomplex__array_deleteRef(dcarray);
+    sidl_dcomplex__array_deleteRef(dcarray);
     dcarray = NULL;
   }
 
+  fcarray = NULL;
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut2Fcomplex(&fcarray, TEST_DIM1, TEST_DIM2);
   MYASSERT(makeInOut2Fcomplex, 
            ArrayTest_ArrayOps_check2Fcomplex(fcarray) == TRUE);
-  SIDL_fcomplex__array_deleteRef(fcarray);
+  sidl_fcomplex__array_deleteRef(fcarray);
   fcarray = NULL;
 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut2Fcomplex(&fcarray, -1, -1);
   MYASSERT(makeInOut2FcomplexNegOne, fcarray == NULL);
   if (fcarray) {
-    SIDL_fcomplex__array_deleteRef(fcarray);
+    sidl_fcomplex__array_deleteRef(fcarray);
     fcarray = NULL;
   }
-
+  
+  iarray = NULL;
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut3Int(&iarray);
   MYASSERT(makeInOut3Int, ArrayTest_ArrayOps_check3Int(iarray) == TRUE);
-  SIDL_int__array_deleteRef(iarray);
+  sidl_int__array_deleteRef(iarray);
   iarray = NULL;
-
+ 
   magicNumber = clearstack(magicNumber);
   ArrayTest_ArrayOps_makeInOut4Int(&iarray);
   MYASSERT(makeInOut4Int, ArrayTest_ArrayOps_check4Int(iarray) == TRUE);
-  SIDL_int__array_deleteRef(iarray);
+  sidl_int__array_deleteRef(iarray);
   iarray = NULL;
 
+  magicNumber = clearstack(magicNumber);
+  ArrayTest_ArrayOps_makeInOut5Int(&iarray);
+  MYASSERT(makeInOut5Int, ArrayTest_ArrayOps_check5Int(iarray) == TRUE);
+  sidl_int__array_deleteRef(iarray);
+  iarray = NULL;
+
+  magicNumber = clearstack(magicNumber);
+  ArrayTest_ArrayOps_makeInOut6Int(&iarray);
+  MYASSERT(makeInOut6Int, ArrayTest_ArrayOps_check6Int(iarray) == TRUE);
+  sidl_int__array_deleteRef(iarray);
+  iarray = NULL;
+  
+  iarray = NULL;
+  magicNumber = clearstack(magicNumber);
+  ArrayTest_ArrayOps_makeInOut7Int(&iarray);
+  fflush(stdout);
+  MYASSERT(makeInOut7Int, ArrayTest_ArrayOps_check7Int(iarray) == TRUE);
+  sidl_int__array_deleteRef(iarray);
+  iarray = NULL;
+  
+  oarray = NULL;
   magicNumber = clearstack(magicNumber);
   MYASSERT(createObjectNegOne, ArrayTest_ArrayOps_createObject(-1) == NULL);
   oarray = ArrayTest_ArrayOps_createObject(16);
   MYASSERT(checkObject, ArrayTest_ArrayOps_checkObject(oarray) == 16);
-  if (oarray) {
+  if (oarray) { 
     ArrayTest_ArrayOps__array_deleteRef(oarray);
     oarray = NULL;
   }
+  fflush(stdout);
   MYASSERT(checkObjectNull, !ArrayTest_ArrayOps_checkObject(NULL));
+  oarray = NULL;
+  
+  {
+    int dimen = 0;
+    int type = 0;
+    int dimen2 = 0;
+    int type2 = 0;
+    struct sidl__array *garrayret = NULL;
+    struct sidl__array *garrayout = NULL;
+    struct sidl__array *garrayinout = NULL;
+    ArrayTest_ArrayOps_checkGeneric(garray, &dimen, &type);
+    MYASSERT(Generic array is still Null, (garray == NULL));
+    MYASSERT(NULL Generic array has no dimension, (dimen == 0));
+    MYASSERT(NULL Generic array has no type, (type == 0));
+    dimen = 1;
+    type = sidl_int_array;
+    garray = ArrayTest_ArrayOps_createGeneric(dimen, type);
+    MYASSERT(Generic (int) array is not Null, (garray != NULL));
+    MYASSERT(Generic (int) array has 1 dimension, (dimen == sidl__array_dimen(garray)));
+    MYASSERT(Generic (int) array has int type, (type == sidl__array_type(garray)));
+
+    ArrayTest_ArrayOps_checkGeneric(garray, &dimen2, &type2);
+    MYASSERT(checkGeneric (int) array has 1 dimension, (dimen == dimen2));
+    MYASSERT(checkGeneric (int) array has int type, (type == type2));
+    garrayout = (struct sidl__array *)(ptrdiff_t)rand();
+
+    garrayret = ArrayTest_ArrayOps_passGeneric(garray, &garrayinout,
+					       &garrayout);
+    
+    MYASSERT(Generic returned array not NULL, garrayret != NULL);
+    MYASSERT(Generic returned array correct dimension, 
+	     sidl__array_dimen(garrayret) == sidl__array_dimen(garrayret));
+    MYASSERT(Generic returned array correct type, 
+	     sidl__array_type(garrayret) == sidl__array_type(garrayret));
+    MYASSERT(Generic returned array correct length, 
+	     sidl__array_length(garrayret,0) == sidl__array_length(garrayret,0));
+
+    MYASSERT(Generic out array not NULL, garrayout != NULL);
+    MYASSERT(Generic out array correct dimension, 
+	     sidl__array_dimen(garray) == sidl__array_dimen(garrayout));
+    MYASSERT(Generic out array correct type, 
+	     sidl__array_type(garray) == sidl__array_type(garrayout));
+    MYASSERT(Generic out array correct length, 
+	     sidl__array_length(garray,0) == sidl__array_length(garrayout,0));
+
+
+    MYASSERT(Generic inout is correct, 
+	     ArrayTest_ArrayOps_check2Int((struct sidl_int__array *)garrayinout) == TRUE);
+    
+    if (garray) sidl__array_deleteRef(garray);
+    if (garrayinout) sidl__array_deleteRef(garrayinout);
+    if (garrayout) sidl__array_deleteRef(garrayout);
+    if (garrayret) sidl__array_deleteRef(garrayret);
+  }
+
+  {
+    int32_t* irarray = NULL;
+    magicNumber = clearstack(magicNumber);
+    iarray = ArrayTest_ArrayOps_createInt(TEST_SIZE);
+    MYASSERT(createInt, iarray);
+    MYASSERT(createInt, ArrayTest_ArrayOps_checkInt(iarray) == TRUE);
+
+    irarray = iarray->d_firstElement;
+    MYASSERT(Check rarray int 1, ArrayTest_ArrayOps_checkRarray1Int(irarray, TEST_SIZE) == TRUE);
+    sidl_int__array_deleteRef(iarray);
+    iarray = NULL;
+    irarray = NULL;
+  }
+
+  {
+    int32_t* irarray = NULL;
+    struct sidl_int__array *iarray2;
+    int32_t n,m,o;
+    magicNumber = clearstack(magicNumber);
+    iarray = ArrayTest_ArrayOps_create3Int();
+    MYASSERT(create3Int, ArrayTest_ArrayOps_check3Int(iarray) == TRUE);
+    iarray2 = sidl_int__array_ensure(iarray,3,sidl_column_major_order);
+    irarray = iarray2->d_firstElement;
+    n = sidlLength(iarray2,0);
+    m = sidlLength(iarray2,1);
+    o = sidlLength(iarray2,2);
+    MYASSERT(Check rarray int 3, ArrayTest_ArrayOps_checkRarray3Int(irarray, n,m,o) == TRUE);
+    sidl_int__array_deleteRef(iarray);
+    sidl_int__array_deleteRef(iarray2);
+    iarray = NULL;
+    irarray = NULL;
+  }
+
+    {
+    int32_t* irarray = NULL;
+    struct sidl_int__array *iarray2;
+    int32_t n,m,o,p,q,r,s;
+    magicNumber = clearstack(magicNumber);
+    iarray = ArrayTest_ArrayOps_create7Int();
+    MYASSERT(create7Int, ArrayTest_ArrayOps_check7Int(iarray) == TRUE);
+    iarray2 = sidl_int__array_ensure(iarray,7,sidl_column_major_order);
+    irarray = iarray2->d_firstElement;
+    n = sidlLength(iarray2,0);
+    m = sidlLength(iarray2,1);
+    o = sidlLength(iarray2,2);
+    p = sidlLength(iarray2,3);
+    q = sidlLength(iarray2,4);
+    r = sidlLength(iarray2,5);
+    s = sidlLength(iarray2,6);
+    MYASSERT(Check rarray int 7, ArrayTest_ArrayOps_checkRarray7Int(irarray, n,m,o,p,q,r,s) == TRUE);
+    sidl_int__array_deleteRef(iarray);
+    sidl_int__array_deleteRef(iarray2);
+    iarray = NULL;
+  }
+
+
+  {
+    int32_t irarray[TEST_SIZE]; 
+    memset(irarray, rand(), sizeof(irarray));
+    magicNumber = clearstack(magicNumber);
+    ArrayTest_ArrayOps_initRarray1Int(irarray, TEST_SIZE);
+    MYASSERT(Check rarray int 1, ArrayTest_ArrayOps_checkRarray1Int(irarray, TEST_SIZE) == TRUE);
+  }
+
+  {
+    int32_t irarray[24]; /* 2*3*4 */
+    int32_t n=2, m=3 , o=4;
+    magicNumber = clearstack(magicNumber);
+    memset(irarray, rand(), sizeof(irarray));
+    ArrayTest_ArrayOps_initRarray3Int(irarray, n,m,o);
+    MYASSERT(Check rarray int 3, ArrayTest_ArrayOps_checkRarray3Int(irarray, n,m,o) == TRUE);
+
+  }
+
+  {
+    int32_t irarray[432]; /* 2*2*2*2*3*3*3  */
+    int32_t n=2, m=2 , o=2, p=2, q=3,r=3,s=3;
+    magicNumber = clearstack(magicNumber);
+    memset(irarray, rand(), sizeof(irarray));
+    ArrayTest_ArrayOps_initRarray7Int(irarray, n,m,o,p,q,r,s);
+    MYASSERT(Check rarray int 7, ArrayTest_ArrayOps_checkRarray7Int(irarray, n,m,o,p,q,r,s) == TRUE);
+
+  }
+
+
+  {
+    double drarray[TEST_SIZE]; 
+    magicNumber = clearstack(magicNumber);
+    memset(drarray, rand(), sizeof(drarray));
+    ArrayTest_ArrayOps_initRarray1Double(drarray, TEST_SIZE);
+    MYASSERT(Check rarray double 1, ArrayTest_ArrayOps_checkRarray1Double(drarray, TEST_SIZE) == TRUE);
+  }
+
+  {
+    struct sidl_dcomplex dcrarray[TEST_SIZE]; 
+    magicNumber = clearstack(magicNumber);
+    memset(dcrarray, rand(), sizeof(dcrarray));
+    ArrayTest_ArrayOps_initRarray1Dcomplex(dcrarray, TEST_SIZE);
+    MYASSERT(Check rarray Dcomplex 1, ArrayTest_ArrayOps_checkRarray1Dcomplex(dcrarray, TEST_SIZE) == TRUE);
+  }
+
+  {
+    int32_t n = 3, m = 3, o = 2, a[9], b[6], x[6];
+    int i = 0;
+    for(i = 0; i < 9; ++i) {
+      a[i]=i;
+    }
+    for(i = 0; i < 6; ++i) {
+      b[i]=i;
+    }
+    ArrayTest_ArrayOps_matrixMultiply(a,b,x,n,m,o);
+    MYASSERT(Check Matrix Multiply, ArrayTest_ArrayOps_checkMatrixMultiply(a,b,x,n,m,o) == TRUE);
+    
+  }
 
   {
     const int32_t len = TEST_SIZE;
@@ -623,20 +861,20 @@ int main(int argc, char **argv) {
     int32_t *end = data + upper;
     int32_t i;
     int32_t prime = nextPrime(0);
-    iarray = SIDL_int__array_borrow(end, 1, &lower, &upper, &negOne);
+    iarray = sidl_int__array_borrow(end, 1, &lower, &upper, &negOne);
     MYASSERT(borrow, iarray);
     if (iarray) {
       for(i = lower; i <= upper; ++i, prime = nextPrime(prime) ) {
         switch(rand() % 3) {
-        case 0:  SIDLArrayElem1(iarray, i) = prime;      break;
-        case 1:  SIDL_int__array_set1(iarray, i, prime); break;
-        default: SIDL_int__array_set(iarray, &i, prime); break;
+        case 0:  sidlArrayElem1(iarray, i) = prime;      break;
+        case 1:  sidl_int__array_set1(iarray, i, prime); break;
+        default: sidl_int__array_set(iarray, &i, prime); break;
         }
       }
     }
     MYASSERT(negativeStride, 
              iarray && ArrayTest_ArrayOps_checkInt(iarray) == TRUE);
-    if (iarray) SIDL_int__array_deleteRef(iarray);
+    if (iarray) sidl_int__array_deleteRef(iarray);
     free(data);
   }
   {
@@ -648,24 +886,25 @@ int main(int argc, char **argv) {
     int32_t *data = malloc(numElems*sizeof(int32_t));
     int32_t *end = data + numElems - 1;
     int32_t i, j, k, part;
-    iarray = SIDL_int__array_borrow(end, 3, lower, upper, stride);
+    iarray = sidl_int__array_borrow(end, 3, lower, upper, stride);
     MYASSERT(borrowThree, iarray);
     if (iarray) {
       for(i = lower[0]; i <= upper[0]; ++i) {
         for(j = lower[1]; j <= upper[1]; ++j) {
           part = (i + 1) * (j + 2);
           for(k = lower[2]; k <= upper[2]; ++k) {
-            SIDLArrayElem3(iarray, i, j, k) = part*(k+3);
+            sidlArrayElem3(iarray, i, j, k) = part*(k+3);
           }
         }
       }
     }
     MYASSERT(negativeStrideThree,
              iarray && ArrayTest_ArrayOps_check3Int(iarray) == TRUE);
-    if (iarray) SIDL_int__array_deleteRef(iarray);
+    if (iarray) sidl_int__array_deleteRef(iarray);
     free(data);
   }
-
-  printf("TEST_RESULT %s\n", result_string[ final_result ] );
+  
+  synch_RegOut_close(tracker);
+  synch_RegOut_deleteRef(tracker);
   return 0;
 }
