@@ -21,4 +21,4 @@
 # You should have recieved a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-all = [ "SIDL", "SIDLObjA" "SIDLPyArrays" ]
+all = [ "sidl", "sidlObjA" "sidlPyArrays" ]
